@@ -26,6 +26,7 @@ const char* langNames[] = {
 	"Slovenščina",
 	"Srpski",
 	"Svenska",
+	//"Türkçe",
 	"Украї́нська",
 	"tiếng Việt"
 };
@@ -57,12 +58,14 @@ const wxLanguage langIds[] = {
 	wxLANGUAGE_SLOVENIAN,
 	wxLANGUAGE_SERBIAN,
 	wxLANGUAGE_SWEDISH,
+	wxLANGUAGE_TURKISH,
 	wxLANGUAGE_UKRAINIAN,
 	wxLANGUAGE_VIETNAMESE
 };
 
 #include "bitmaps/armenia.png.h"
 #include "bitmaps/china.png.h"
+#include "bitmaps/taiwan.png.h"
 #include "bitmaps/czech.png.h"
 #include "bitmaps/denmark.png.h"
 #include "bitmaps/germany.png.h"
@@ -86,6 +89,7 @@ const wxLanguage langIds[] = {
 #include "bitmaps/slovenia.png.h"
 #include "bitmaps/serbia.png.h"
 #include "bitmaps/sweden.png.h"
+#include "bitmaps/turkey.png.h"
 #include "bitmaps/ukraine.png.h"
 #include "bitmaps/vietnam.png.h"
 
@@ -94,7 +98,7 @@ const BitmapArray& GetLangBitmapArray() {
 	if (s_langBitmapArray.size() == 0) {
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(armenia));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(china));
-		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(china));
+		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(taiwan));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(czech));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(denmark));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(germany));
@@ -118,6 +122,7 @@ const BitmapArray& GetLangBitmapArray() {
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(slovenia));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(serbia));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(sweden));
+		//s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(turkey));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(ukraine));
 		s_langBitmapArray.Add(wxBITMAP_FROM_MEMORY(vietnam));
 	}
