@@ -3,7 +3,7 @@
 // Purpose:     About dialog
 // Author:      Alex Thuering
 // Created:     6.11.2003
-// RCS-ID:      $Id: About.cpp,v 1.70 2011/02/23 20:32:54 ntalex Exp $
+// RCS-ID:      $Id: About.cpp,v 1.77 2011/12/04 13:10:34 ntalex Exp $
 // Copyright:  (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -122,29 +122,29 @@ About::About(wxWindow* parent): wxDialog(parent, -1, wxEmptyString,
 	const wxString translations[][2] = {
 		{ wxT("Armenian"), wxT("Hrant Ohanyan <hrantohanyan at mail.am>") },
 		{ wxT("Chinese (Simplified)"), wxString(wxT("程序民工 <lyj8063 at sohu.com>, 郁晓旭 <yuxiaoxu at msn.com>"), wxConvUTF8) },
-		{ wxT("Chinese (Traditional)"), wxString(wxT("紀明村 <tw.chi.ming at gmail.com>"), wxConvUTF8) },
+		{ wxT("Chinese (Traditional)"), wxString(wxT("紀明村 <tw.chi.ming at gmail.com>, Sophina Liu <Sophina-liu at hotmail.com>"), wxConvUTF8) },
 		{ wxT("Basque"), wxT("Piarres Beobide <pi at beobide.net>") },
 		{ wxT("Czech"), wxString(wxT("Vlastimil Skácel <vlastiks at gmail.com>"), wxConvUTF8) },
-		{ wxT("Danish"), wxT("Patrick Fust <patrick_fust at hotmail.com>, Jørgen Rasmussen <l10n at joergenr.dk>") },
-		{ wxT("Dutch"), wxT("Maikel Egberink <maikel.egberink at gmail.com>, Ron Vennix <ron.vennix at gmail.com>") },
-		{ wxT("French"), wxT("Philippe MACE <philippe.mace at kioskeo.fr>, Pierre Buard <pierre.buard at gmail.com>") },
+		{ wxT("Danish"), wxT("Jørgen Rasmussen <l10n at joergenr.dk>") },
+		{ wxT("Dutch"), wxT("Frank Meijer <fajmmeijer at hotmail.com>, Maikel Egberink <maikel.egberink at gmail.com>") },
+		{ wxT("French"), wxT("Pierre Buard <pierre.buard at gmail.com>, R. Berzolla <rberzolla at free.fr>") },
 		{ wxT("German"), wxT("Alex Th&uuml;ring <alex at thuering.biz>") },
 		{ wxT("Greek"), wxT("Aristotelis Pitaridis <pitaridis at gmail.com>") },
 		{ wxT("Hungarian"), wxString(wxT("Fáczán László <faczan at freemail.hu>"), wxConvUTF8) },
-		{ wxT("Italian"), wxT("Alessio Leonarduzzi <alessio.leonarduzzi at gmail.com>") },
+		{ wxT("Italian"), wxT("Marco Ciampa <ciampix at libero.it>, Alessio Leonarduzzi <alessio.leonarduzzi at gmail.com>") },
 		{ wxT("Japanese"), wxT("OKANO Takayoshi <kano at na.rim.or.jp>") },
 		{ wxT("Korean"), wxString(wxT("김프사랑 <yongdoria at gmail.com>"), wxConvUTF8) },
 		{ wxT("Macedonian"), wxT("Damjan Dimitrioski <damjandimitrioski at gmail.com>") },
 		{ wxT("Norwegian"), wxString(wxT("Olav Inge Frøynes <olavinge at gmail.com>"), wxConvUTF8) },
-		{ wxT("Polish"), wxString(wxT("Michal Jusyk <drewnozlasu at o2.pl>, Michał Trzebiatowski <hippie_1968 at hotmail.com>"), wxConvUTF8) },
+		{ wxT("Polish"), wxString(wxT("Michał Trzebiatowski <hippie_1968 at hotmail.com>"), wxConvUTF8) },
 		{ wxT("Portuguese"), wxString(wxT("João Frade / / 100 NOME PT <100nome.portugal at gmail.com>"), wxConvUTF8) },
-		{ wxT("Portuguese (Brazilian)"), wxT("Henrique Lopes Barone <henriquebarone at quadrochave.com>") },
+		{ wxT("Portuguese (Brazilian)"), wxT("Henrique Lopes Barone <henrique.blender at gmail.com>, Igor Ruckert <igorruckert at yahoo.com.br>") },
 		{ wxT("Russian"), wxT("Alex Th&uuml;ring <alex at thuering.biz>") },
 		{ wxT("Slovenian"), wxT("Martin Srebotnjak <miles at filmsi.net>") },
 		{ wxT("Serbian"), wxT("CICIC Ivana <ozzii23 at gmail.com>") },
 		{ wxT("Slovak"), wxT("Michal Kovac <deadsock at deadsock.co.uk>") },
 		{ wxT("Spanish"), wxT("Redy Rodriguez <redy at telefonica.net>, Fabrice Coutadeur <coutadeurf at gmail.com>") },
-		{ wxT("Swedish"), wxT("Daniel Nylander <po at danielnylander.se>") },
+		{ wxT("Swedish"), wxT("Daniel Nylander <po at danielnylander.se>, Tobias Holm <dvdstyler at eftprod.com>") },
 		{ wxT("Ukrainian"), wxT("Федір Зінчук <getthesite at gmail.com>") },
 		{ wxT("Vietnamese"), wxT("Phan Anh <ppanhh at gmail.com>") },
 	};
