@@ -3,7 +3,7 @@
 // Purpose:     Burn dialog
 // Author:      Alex Thuering
 // Created:     23.03.2003
-// RCS-ID:      $Id: BurnDlg.cpp,v 1.32 2011/02/13 16:54:50 ntalex Exp $
+// RCS-ID:      $Id: BurnDlg.cpp,v 1.33 2011/07/31 13:25:02 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -482,7 +482,11 @@ void BurnDlg::UpdateCtrls(bool def) {
 	m_previewCheck->SetValue(s_config.GetPreviewDo(def));
 	m_generateRadioBt->SetValue(s_config.GetGenerateDo(def));
 	m_isoRadioBt->SetValue(s_config.GetIsoDo(def));
-	m_isoText->SetValue(s_config.GetIsoSaveTo(def));
+	wxString fname = wxT("dvd");
+	if (m_dvd->GetFilename().length())
+		wxFileName::SplitPath(m_dvd->GetFilename(), NULL, &fname, NULL);
+	m_isoText->SetValue(m_dvd->GetIsoFile().length() > 0 ? m_dvd->GetIsoFile()
+			: wxPathOnly(s_config.GetIsoSaveTo(def)) + wxFILE_SEP_PATH + fname + wxT(".iso"));
 	m_addECCCheck->SetValue(s_config.GetAddECCDo(def));
 	m_burnRadioBt->SetValue(s_config.GetBurnDo(def));
 	m_formatCheck->SetValue(s_config.GetFormatDo(def));
@@ -615,8 +619,19 @@ void BurnDlg::OnReset(wxCommandEvent& event) {
 }
 
 void BurnDlg::OnOk(wxCommandEvent& event) {
-	wxString device = GetDevice().Trim().Trim(false);
+	if (m_isoRadioBt->GetValue()) {
+		wxString isoFile = GetIsoFile().Trim().Trim(false);
+		if (isoFile.length() == 0) {
+			wxMessageBox(_("Please enter file name to save ISO"), _("Burn"), wxOK|wxICON_ERROR, this);
+			return;
+		}
+		if (wxFileExists(isoFile) && wxMessageBox(wxString::Format(
+				_("File '%s' already exist. Do you want to overwrite it?"), isoFile.c_str()),
+				_("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
+			return;
+	}
 	if (m_burnRadioBt->GetValue()) {
+		wxString device = GetDevice().Trim().Trim(false);
 		if (device.length() == 0) {
 			wxMessageBox(_("Please enter device name"), _("Burn"), wxOK|wxICON_ERROR, this);
 			return;
@@ -650,7 +665,10 @@ void BurnDlg::OnOk(wxCommandEvent& event) {
 	s_config.SetPreviewDo(m_previewCheck->GetValue());
 	s_config.SetGenerateDo(m_generateRadioBt->GetValue());
 	s_config.SetIsoDo(m_isoRadioBt->GetValue());
-	s_config.SetIsoSaveTo(m_isoText->GetValue());
+	if (m_isoRadioBt->GetValue()) {
+		m_dvd->SetIsoFile(GetIsoFile());
+		s_config.SetIsoSaveTo(wxPathOnly(GetIsoFile()) + wxFILE_SEP_PATH);
+	}
 	s_config.SetAddECCDo(m_addECCCheck->GetValue());
 	s_config.SetBurnDo(m_burnRadioBt->GetValue());
 	s_config.SetFormatDo(m_formatCheck->GetValue());
