@@ -3,7 +3,7 @@
 // Purpose:     Manage cache of transcoded files
 // Author:      Alex Thuering
 // Created:     29.08.2008
-// RCS-ID:      $Id: Cache.cpp,v 1.5 2011/03/10 20:10:18 ntalex Exp $
+// RCS-ID:      $Id: Cache.cpp,v 1.8 2011/08/13 12:07:23 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -16,6 +16,32 @@
 
 const wxString CACHE_SUB_DIR = wxT("dvd-cache");
 
+/**
+ * Stores cache entry information
+ */
+class CacheEntry {
+public:
+	CacheEntry(Vob* vob, DVD* dvd, int idx);
+	bool equals(const CacheEntry& entry);
+	inline int GetIdx() { return m_idx; }
+	inline bool IsUsed() { return m_used; }
+	inline void SetUsed(bool used) { m_used = used; }
+	
+private:
+	wxString m_videoFile;
+	wxArrayString m_audioFiles;
+	wxArrayString m_subtitleFiles;
+	wxArrayInt m_formats;
+    int m_videoBitrate;
+    int m_audioBitrate;
+	double m_startTime;
+	double m_recordingTime;
+	wxString m_videoFilters;
+    int m_idx;
+    bool m_used;
+};
+
+
 CacheEntry::CacheEntry(Vob* vob, DVD* dvd, int idx) {
 	m_videoFile = vob->GetFilename();
 	m_audioFiles = vob->GetAudioFilenames();
@@ -25,6 +51,9 @@ CacheEntry::CacheEntry(Vob* vob, DVD* dvd, int idx) {
 		m_formats.Add(vob->GetStreams()[stIdx]->GetDestinationFormat());
 	m_videoBitrate = dvd->GetVideoBitrate();
 	m_audioBitrate = s_config.GetAudioBitrate();
+	m_startTime = vob->GetStartTime();
+	m_recordingTime = vob->GetRecordingTime();
+	m_videoFilters = vob->GetAllVideoFilters();
 	m_idx = idx;
 	m_used = false;
 }
@@ -35,6 +64,9 @@ bool CacheEntry::equals(const CacheEntry& entry) {
 			|| m_subtitleFiles != entry.m_subtitleFiles
 			|| m_videoBitrate != entry.m_videoBitrate
 			|| m_audioBitrate != entry.m_audioBitrate
+			|| m_startTime != entry.m_startTime
+			|| m_recordingTime != entry.m_recordingTime
+			|| m_videoFilters != entry.m_videoFilters
 			|| m_formats.Count() != entry.m_formats.Count())
 		return false;
 	for (unsigned int i = 0; i < m_formats.GetCount(); i++)
@@ -137,6 +169,7 @@ void Cache::Clear() {
 	while (d.GetFirst(&fname, wxEmptyString, wxDIR_FILES))
 		if (!wxRemoveFile(m_dir + wxFILE_SEP_PATH + fname))
 			wxLogError(wxString::Format(_("Can't remove file '%s'"), fname.c_str()));
+	d.Open(wxGetHomeDir());
 	wxRmdir(m_dir);
 }
 
@@ -146,13 +179,15 @@ void Cache::BeginClean() {
 }
 
 void Cache::EndClean() {
-	for (CacheMap::iterator it = m_cacheMap.begin(); it != m_cacheMap.end(); it++) {
+	CacheMap::iterator it = m_cacheMap.begin();
+	while (it != m_cacheMap.end()) {
 		CacheEntry* entry = it->second;
 		if (!entry->IsUsed()) {
 			wxRemoveFile(GetFileName(entry->GetIdx()));
-			m_cacheMap.erase(it->first);
+			m_cacheMap.erase(it++);
 			delete entry;
-		}
+		} else
+			++it;
 	}
 }
 
