@@ -3,7 +3,7 @@
 // Purpose:     Stores a DVD Action
 // Author:      Alex Thuering
 // Created:     03.09.2009
-// RCS-ID:      $Id: DVDAction.cpp,v 1.21 2011/02/20 18:52:16 ntalex Exp $
+// RCS-ID:      $Id: DVDAction.cpp,v 1.23 2011/04/28 21:39:38 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -18,9 +18,8 @@ extern "C" {
 #include "dvdvm.h"
 }
 
-DVDAction::DVDAction(bool vmg, DVDActionType actionType, wxString id) {
+DVDAction::DVDAction(bool vmg, wxString id) {
 	m_vmg = vmg;
-	m_actionType = actionType;
 	m_id = id;
 	m_menu = false;
 	m_tsi = -2; // current titleset
@@ -31,10 +30,6 @@ DVDAction::DVDAction(bool vmg, DVDActionType actionType, wxString id) {
 	m_subtitle = -1;
 }
 
-wxString DVDAction::GetId(bool translate) {
-	return m_id;
-}
-
 /* entry menu types */
 const wxString s_entriesArr[6] = {wxT("title"), wxT("root"), wxT("subtitle"), wxT("audio"), wxT("angle"), wxT("ptt")};
 const wxArrayString s_entries(6, s_entriesArr);
@@ -206,7 +201,7 @@ bool validateJumpTarget(wxString msgPrefix, DVD* dvd, int tsi, int pgci, bool me
 				wxLogError(msgPrefix + _("cannot jump to menu; none exist"));
 			return false;
 		} else if (entry.length() > 0) { // menu entry
-			StringSet entrySet = menus.GetEntries();
+			const StringSet& entrySet = menus.GetEntries();
 			if (entry != wxT("default") && entrySet.find(entry) == entrySet.end()){
 				const wxString entries[6] = {_("title menu"), _("root menu"), _("subtitle menu"), _("audio menu"),
 						_("angle menu"), _("chapter menu")};
@@ -434,6 +429,10 @@ bool validateCall(wxString msgPrefix, DVD* dvd, const struct vm_statement *cs, i
 	int i2 = cs->i2; // menu number or menu entry ID + 120 or title number + 128
 	int i3 = cs->i3; // chapter number if nonzero and less than 65536 or program number + 65536 or cell number + 131072
 	int i4 = cs->i4; // resume cell if specified, else zero
+	int i5 = cs->i5; // last menu if specified, else zero
+	
+	if (i5)
+		return true;
 	
 	if (!validateTargetNumbers(msgPrefix, i1, i2, i3)) {
 		return false;
@@ -580,12 +579,58 @@ bool DVDAction::IsCustomValid(wxString msgPrefix, wxString source, DVD* dvd, int
 	return validateVm(dvd, dvd_vm_parsed_cmd, tsi, pgci, ismenu, msgPrefix);
 }
 
+int findFirstJump(const struct vm_statement *cs, int tsi) {
+	while (cs) {
+		switch (cs->op) {
+		case VM_IF: {
+			int result = findFirstJump(cs->param->next->param, tsi);
+			if (result != -1)
+				return result;
+			if (cs->param->next->next)
+				result = findFirstJump(cs->param->next->next, tsi); 
+			if (result != -1)
+				return result;
+			break;
+		}
+		case VM_JUMP: {
+			int i1 = cs->i1; // if nonzero, 1 for VMGM, or titleset nr + 1
+			int i2 = cs->i2; // menu number or menu entry ID + 120 or title number + 128
+			int i3 = cs->i3; // chapter number if nonzero and less than 65536 or program number + 65536 or cell number + 131072
+			if (i1 == 0)
+				i1 = tsi + 2;
+			if (i1 > 1 && i2 > 128)
+				return DVD::MakeId(i1 - 2, i2 - 129, i3 < 256 ? i3 : 0, false);
+		}
+		default:
+			break;
+		}
+		cs = cs->next;
+	}
+	return -1;
+}
+
+/** Returns first jump target or -1 if no jump title action found */
+int DVDAction::FindFirstJump(int tsi) {
+	// parse command
+	const char* const cmd = strdup((const char*)m_custom.ToAscii());
+	dvdvm_buffer_state buf = dvdvm_scan_string(cmd);
+	dvd_vm_parsed_cmd = 0;
+	s_parseErrorMsg = wxT("");
+	s_vmlErrorMsg = wxT("");
+	int result = dvdvmparse();
+	dvdvm_delete_buffer(buf);
+	free((void *) cmd);
+	if (result)
+		return -1;
+	return findFirstJump(dvd_vm_parsed_cmd, tsi);
+}
+
 wxSvgXmlNode* DVDAction::GetXML(DVDFileType type, DVD* dvd) {
 	wxSvgXmlNode* rootNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("action"));
 	switch (type) {
 	case DVDSTYLER_XML:
-		if (m_actionType == MENU_ACTION)
-			rootNode->AddProperty(wxT("id"), GetId());
+		if (m_id.length())
+			rootNode->AddProperty(wxT("id"), m_id);
 		if (!GetCustom().length()) {
 			if (GetTsi() >= -1)
 				rootNode->AddProperty(wxT("tsi"), wxString::Format(wxT("%d"), GetTsi()));
@@ -605,10 +650,10 @@ wxSvgXmlNode* DVDAction::GetXML(DVDFileType type, DVD* dvd) {
 			rootNode->AddProperty(wxT("subtitle"), wxString::Format(wxT("%d"), GetSubtitle()));
 		break;
 	case SPUMUX_XML:
-		rootNode->AddProperty(wxT("name"), GetId());
+		rootNode->AddProperty(wxT("name"), m_id);
 		break;
 	case DVDAUTHOR_XML:
-		rootNode->AddProperty(wxT("name"), GetId());
+		rootNode->AddProperty(wxT("name"), m_id);
 		rootNode->AddChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString, AsString(dvd)));
 		break;
 	default:
@@ -618,8 +663,7 @@ wxSvgXmlNode* DVDAction::GetXML(DVDFileType type, DVD* dvd) {
 }
 
 bool DVDAction::PutXML(wxSvgXmlNode* node) {
-	if (m_actionType == MENU_ACTION)
-		node->GetPropVal(wxT("id"), &m_id);
+	node->GetPropVal(wxT("id"), &m_id);
 	long lval;
 	wxString val;
 	if (node->GetPropVal(wxT("tsi"), &val) && val.ToLong(&lval))
