@@ -3,7 +3,7 @@
 // Purpose:     Stores a DVD Action
 // Author:      Alex Thuering
 // Created:     03.09.2009
-// RCS-ID:      $Id: DVDAction.h,v 1.11 2010/04/28 07:27:56 ntalex Exp $
+// RCS-ID:      $Id: DVDAction.h,v 1.12 2011/04/28 21:39:38 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -14,15 +14,14 @@
 #include <wx/wx.h>
 
 enum DVDFileType { DVDSTYLER_XML, DVDAUTHOR_XML, SPUMUX_XML };
-enum DVDActionType { MENU_ACTION, BUTTON_ACTION };
 class DVD;
 class wxSvgXmlNode;
 
 class DVDAction {
 public:
-	DVDAction(bool vmg, DVDActionType actionType, wxString id = wxT(""));
+	DVDAction(bool vmg = false, wxString id = wxT(""));
     
-    wxString GetId(bool translate = false);
+    wxString GetId() { return m_id; }
     wxString AsString(DVD* dvd, bool translate = false);
 	
 	inline bool IsCustom() { return m_custom.length()>0; }
@@ -51,7 +50,9 @@ public:
 	/** Returns true if action is valid */
 	bool IsValid(DVD* dvd, int tsi, int pgci, bool menu, wxString buttonId = wxT(""),
 			bool showMessage = false, bool showSource = true, bool skipInvalidTarget = false);
-	
+	/** Returns first jump target or -1 if no jump title action found */
+	int FindFirstJump(int tsi);
+
 	wxSvgXmlNode* GetXML(DVDFileType type, DVD* dvd);
 	bool PutXML(wxSvgXmlNode* node);
 	/** Stores object data to string */
@@ -61,7 +62,6 @@ public:
     
 private:
 	bool m_vmg;
-	DVDActionType m_actionType;
     wxString m_id;
     wxString m_custom;
 	int m_tsi;
