@@ -3,14 +3,13 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.144 2011/02/22 18:42:05 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.152 2011/12/20 21:28:19 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "MainWin.h"
 #include "BurnDlg.h"
-#include "DonateDlg.h"
 #include "ProgressDlg.h"
 #include "About.h"
 #include "NewProjectDlg.h"
@@ -26,6 +25,7 @@
 #include <wx/textfile.h>
 #include <wx/artprov.h>
 #include <wx/msgdlg.h>
+#include <wx/stdpaths.h>
 #include "math.h"
 
 #ifdef __WXMSW__
@@ -75,8 +75,8 @@ enum {
 	BUTTONS_BT_ID,
 	DIR_BOOK_ID,
 	THUMBMENU_BACKGROUND_ID,
-	REFRESH_ID,
-	DELETE_ID
+	THUMBMENU_REFRESH_ID,
+	THUMBMENU_DELETE_ID
 };
 
 BEGIN_EVENT_TABLE(MainWin, wxFrame)
@@ -120,10 +120,11 @@ BEGIN_EVENT_TABLE(MainWin, wxFrame)
 	EVT_NOTEBOOK_PAGE_CHANGED(DIR_BOOK_ID, MainWin::OnPageChanged)
 	
 	EVT_MENU(THUMBMENU_BACKGROUND_ID, MainWin::OnSetBackground)
-	EVT_UPDATE_UI(THUMBMENU_BACKGROUND_ID, MainWin::OnUpdateUISetBackground)
-	EVT_MENU(DELETE_ID, MainWin::OnDelete)
-	EVT_UPDATE_UI(DELETE_ID, MainWin::OnDeleteUpdateUI)
-	EVT_MENU(REFRESH_ID, MainWin::OnRefresh)
+	EVT_UPDATE_UI(THUMBMENU_BACKGROUND_ID, MainWin::OnSetBackgroundUI)
+	EVT_MENU(THUMBMENU_DELETE_ID, MainWin::OnDelete)
+	EVT_UPDATE_UI(THUMBMENU_DELETE_ID, MainWin::OnDeleteUI)
+	EVT_MENU(THUMBMENU_REFRESH_ID, MainWin::OnRefresh)
+	EVT_UPDATE_UI(THUMBMENU_REFRESH_ID, MainWin::OnRefreshUI)
 	
 	EVT_SIZE(MainWin::OnResize)
 	EVT_SPLITTER_SASH_POS_CHANGED(SPLIITTER_TITLE_ID, MainWin::OnSplitterTitleSash)
@@ -250,15 +251,15 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     wxMenu* thumbMenu = new wxMenu;
     thumbMenu->Append(THUMBMENU_BACKGROUND_ID, _("&Assign to background"));
     thumbMenu->AppendSeparator();
-    item = new wxMenuItem(thumbMenu, DELETE_ID, _("&Delete"));
+    item = new wxMenuItem(thumbMenu, THUMBMENU_DELETE_ID, _("&Delete"));
 	item->SetBitmap(wxArtProvider::GetBitmap(wxART_DELETE, wxART_MENU));
 	thumbMenu->Append(item);
     thumbMenu->AppendSeparator();
-	thumbMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
+	thumbMenu->Append(THUMBMENU_REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
 	m_thumbnails->SetPopupMenu(thumbMenu);
 	
     wxMenu* thumbGlobalMenu = new wxMenu;
-    thumbGlobalMenu->Append(REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
+    thumbGlobalMenu->Append(THUMBMENU_REFRESH_ID, _("&Refresh") + wxString(wxT("\tF5")));
     m_thumbnails->SetGlobalPopupMenu(thumbGlobalMenu);
     
     m_fileHistory.Load(*s_config.GetConfigBase());
@@ -410,8 +411,25 @@ void MainWin::OnDirTreeBt(wxCommandEvent& event) {
 void MainWin::OnBackgroundsBt(wxCommandEvent& event) {
 	ShowDirTree(false);
 	m_thumbnails->ShowFileNames(false);
-	m_thumbnails->ShowDir(BACKGROUNDS_DIR);
+	m_thumbnails->Clear();
+	wxString fname = wxFindFirstFile(BACKGROUNDS_DIR + wxT("*"));
+	while (!fname.IsEmpty()) {
+		if (wxImage::FindHandler(fname.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY))
+			m_thumbnails->InsertItem(new wxThumb(fname));
+		fname = wxFindNextFile();
+	}
+	wxStandardPaths paths;
+	wxString bgDir = paths.GetUserDataDir() + wxFILE_SEP_PATH + wxT("backgrounds") + wxFILE_SEP_PATH;
+	if (wxDirExists(bgDir)) {
+		fname = wxFindFirstFile(bgDir + wxT("*"));
+		while (!fname.IsEmpty()) {
+			if (wxImage::FindHandler(fname.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY))
+				m_thumbnails->InsertItem(new wxThumb(fname));
+			fname = wxFindNextFile();
+		}
+	}
 	m_thumbnails->SetCaption(_("Backgrounds"));
+	m_thumbnails->SortItems();
 }
 
 void MainWin::OnButtonsBt(wxCommandEvent& event) {
@@ -565,20 +583,23 @@ void MainWin::OnSetBackground(wxCommandEvent& event) {
 	m_menuEditor->SetBackground(filename);
 }
 
-void MainWin::OnUpdateUISetBackground(wxUpdateUIEvent& event) {
-	if (m_thumbnails->GetSelected() == -1)
-		event.Enable(false);
-	else {
+void MainWin::OnSetBackgroundUI(wxUpdateUIEvent& event) {
+	if (m_thumbnails->GetSelected() >= 0) {
 		wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
 		event.Enable(wxImage::FindHandler(filename.AfterLast(wxT('.')).Lower(), wxBITMAP_TYPE_ANY)
 				|| wxThumbnails::IsVideo(filename));
-	}
+	} else
+		event.Enable(false);
 }
 
 void MainWin::OnRefresh(wxCommandEvent& event) {
 	m_thumbnails->UpdateItems();
 }
 
+void MainWin::OnRefreshUI(wxUpdateUIEvent& event) {
+	event.Enable(IsButtonSelected(DIR_TREE_BT_ID) && m_thumbnails->GetSelected() >= 0);
+}
+
 void MainWin::OnDelete(wxCommandEvent& event) {
 	wxString filename = m_thumbnails->GetSelectedItem()->GetFilename();
 	if (!wxFile::Exists(filename))
@@ -592,7 +613,7 @@ void MainWin::OnDelete(wxCommandEvent& event) {
 	m_thumbnails->UpdateItems();
 }
 
-void MainWin::OnDeleteUpdateUI(wxUpdateUIEvent& event) {
+void MainWin::OnDeleteUI(wxUpdateUIEvent& event) {
 	event.Enable(IsButtonSelected(DIR_TREE_BT_ID) && m_thumbnails->GetSelected() >= 0);
 }
 
@@ -765,6 +786,9 @@ void MainWin::DoChangedDVD(bool doUndo) {
 	long total = m_dvd->GetCapacityValue();
 	m_statusbar->SetFillStatus(size, total, b[1], b[2]/60, b[0],
 			m_dvd->IsVideoBitrateAuto(), s_config.GetAudioBitrate());
+	// update selection
+	if (m_menuEditor->GetSelected().length())
+		m_menuEditor->Select(m_menuEditor->GetSelected());
 	m_menuEditor->ShowSafeTV();
 	m_menuEditor->Refresh();
 }
@@ -794,7 +818,7 @@ void MainWin::OnNew(wxCommandEvent& event) {
 		return;
 	NewProjectDlg dlg(this);
 	if (dlg.ShowModal() == wxID_OK) {
-		TemplateDlg templateDlg(this);
+		TemplateDlg templateDlg(this, dlg.GetAspectRatio());
 		wxString templateFile = templateDlg.ShowModal() == wxID_OK ? templateDlg.GetTemplate() : wxT("");
 		NewDVD(templateFile, templateDlg.GetTitle(), dlg.GetLabel(), dlg.GetCapacity(), dlg.GetVideoBitrate(),
 				dlg.GetVideoFormat(), dlg.GetAudioFormat(), dlg.GetAspectRatio());
@@ -895,12 +919,8 @@ please drag a video file into the bottom section of the window."), emptyTsi+1),
 	// Progress dialog
 	ProgressDlg progressDlg(this, &m_cache, autoStart);
 	bool ok = progressDlg.Start(&burnDlg, m_dvd);
-	if (autoStart) {
+	if (autoStart)
 		Close(true);
-	} else if (ok) {
-		DonateDlg donateDlg(this);
-		donateDlg.ShowModal();
-	}
 }
 
 void MainWin::OnExit(wxCommandEvent& event) {
@@ -1032,7 +1052,7 @@ void MainWin::OnAddTitleset(wxCommandEvent& event) {
 }
 
 void MainWin::OnAddTitlesetUpdateUI(wxUpdateUIEvent& event) {
-	int cnt = m_dvd->GetTitlesets().Count();
+	int cnt = m_dvd != NULL ? m_dvd->GetTitlesets().Count() : 0;
 	event.Enable(cnt > 0 && m_dvd->GetTitlesets()[cnt - 1]->GetTitles().Count());
 }
 
