@@ -3,7 +3,7 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:	10.10.2003
-// RCS-ID:      $Id: MainWin.h,v 1.41 2010/09/05 10:57:29 ntalex Exp $
+// RCS-ID:      $Id: MainWin.h,v 1.42 2011/09/15 05:35:16 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -154,10 +154,11 @@ public:
     void OnSelectMenu(wxCommandEvent& event);
     
     void OnSetBackground(wxCommandEvent& event);
-	void OnUpdateUISetBackground(wxUpdateUIEvent& event);
+	void OnSetBackgroundUI(wxUpdateUIEvent& event);
 	void OnDelete(wxCommandEvent& event);
-	void OnDeleteUpdateUI(wxUpdateUIEvent& event);
+	void OnDeleteUI(wxUpdateUIEvent& event);
 	void OnRefresh(wxCommandEvent& event);
+	void OnRefreshUI(wxUpdateUIEvent& event);
 	
 	virtual void OnNew(wxCommandEvent& event); // wxGlade: <event_handler>
 	virtual void OnOpen(wxCommandEvent& event); // wxGlade: <event_handler>
