@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	04.11.2003
-// RCS-ID:      $Id: Menu.cpp,v 1.95 2011/03/10 20:03:53 ntalex Exp $
+// RCS-ID:      $Id: Menu.cpp,v 1.105 2011/11/22 20:27:02 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -21,6 +21,7 @@
 #include <wx/mstream.h>
 #include <wx/filename.h>
 #include <wx/sstream.h>
+#include <wx/stdpaths.h>
 
 #define OBJECTS_DIR wxFindDataDirectory(_T("objects"))
 #define BACKGROUNDS_DIR wxFindDataDirectory(_T("backgrounds"))
@@ -38,8 +39,8 @@ Menu::Menu(DVD* dvd, int tsi, int pgci, VideoFormat videoFormat, AspectRatio asp
 	m_videoFormat = videoFormat;
 	m_aspectRatio = aspectRatio;
 	m_startTime = _T("00:00:00.00");
-	m_rememberLastButton = false;
-	m_transpColour = wxColour(8, 8, 8);
+	m_rememberLastButton = true;
+	m_rememberLastButtonRegister = -1;
 	m_svg = CreateSVG(GetResolution().GetWidth(), GetResolution().GetHeight());
 	SetBackgroundColour(wxColour(0, 0, 0));
 }
@@ -79,14 +80,19 @@ void Menu::SetAspectRatio(AspectRatio aspectRatio) {
 	m_svg->GetRootElement()->SetWidth(GetResolution().GetWidth());
 	m_svg->GetRootElement()->SetHeight(GetResolution().GetHeight());
 	// background
-	wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
+	wxSVGElement* bgElement = m_svg->GetElementById(wxT("backgroundColour"));
+	if (bgElement && bgElement->GetDtd() == wxSVG_RECT_ELEMENT) {
+		((wxSVGRectElement*) bgElement)->SetWidth(GetResolution().GetWidth());
+		((wxSVGRectElement*) bgElement)->SetHeight(GetResolution().GetHeight());
+	}
+	bgElement = m_svg->GetElementById(wxT("background"));
 	if (bgElement) {
-		if (bgElement->GetDtd() == wxSVG_RECT_ELEMENT) {
-			((wxSVGRectElement*) bgElement)->SetWidth(GetResolution().GetWidth());
-			((wxSVGRectElement*) bgElement)->SetHeight(GetResolution().GetHeight());
-		} else if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT) {
+		if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT) {
 			((wxSVGImageElement*) bgElement)->SetWidth(GetResolution().GetWidth());
 			((wxSVGImageElement*) bgElement)->SetHeight(GetResolution().GetHeight());
+		} else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT) {
+			((wxSVGVideoElement*) bgElement)->SetWidth(GetResolution().GetWidth());
+			((wxSVGVideoElement*) bgElement)->SetHeight(GetResolution().GetHeight());
 		}
 	}
 	// change object coordinates and size
@@ -98,7 +104,7 @@ void Menu::SetAspectRatio(AspectRatio aspectRatio) {
 		font.SetPointSize(round(font.GetPointSize()*f));
 		obj->SetParamFont(wxT("text"), font);
 		// update object Y-coordinate and size
-		obj->SetY(round(obj->GetY()*f));
+		obj->SetY(round(obj->GetY()*f/2)*2);
 		if (!obj->IsDefaultSize() || obj->GetObjectParam(wxT("text")) == NULL) {
 			obj->SetDefaultSize(false);
 			obj->SetWidth(round(obj->GetWidth()*f));
@@ -121,17 +127,36 @@ wxSize Menu::GetFrameResolution() {
 	return wxSize(720, 576); // vfPAL
 }
 
-void Menu::SetBackground(wxString fileName) {
-	SetBackground(m_svg->GetRootElement(), fileName);
-}
+/** Returns filename of background image/video */
 wxString Menu::GetBackground() {
 	return GetBackground(m_svg->GetRootElement());
 }
 
+wxString Menu::GetBackground(wxSVGSVGElement* root) {
+	wxSVGElement* bgElement = (wxSVGElement*) root->GetElementById(wxT("background"));
+	if (bgElement) {
+		if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT)
+			return ((wxSVGImageElement*) bgElement)->GetHref();
+		else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT)
+			return ((wxSVGVideoElement*) bgElement)->GetHref();
+	}
+	return wxT("");
+}
+
+/** Sets background image/video */
+void Menu::SetBackground(wxString fileName) {
+	SetBackground(m_svg->GetRootElement(), fileName);
+}
+
 void Menu::SetBackground(wxSVGSVGElement* root, wxString fileName) {
 	bool image = wxImage::FindHandler(fileName.AfterLast(wxT('.')).Lower(), wxBITMAP_TYPE_ANY) != NULL;
 	// test if element exists
 	wxSVGElement* bgElement = (wxSVGElement*) root->GetElementById(wxT("background"));
+	if (fileName.length() == 0) {
+		if (bgElement) // remove image/video elment
+			bgElement->GetParent()->RemoveChild(bgElement);
+		return;
+	}
 	if (!bgElement || (image && bgElement->GetDtd() != wxSVG_IMAGE_ELEMENT)
 			|| (!image && bgElement->GetDtd() != wxSVG_VIDEO_ELEMENT)) {
 		if (bgElement) // remove old
@@ -146,7 +171,9 @@ void Menu::SetBackground(wxSVGSVGElement* root, wxString fileName) {
 			((wxSVGVideoElement*) bgElement)->SetHeight(GetResolution().GetHeight());
 		}
 		bgElement->SetId(wxT("background"));
-		root->InsertChild(bgElement, root->GetChildren());
+		root->InsertChild(bgElement,
+				root->GetChildren() && ((wxSVGElement*) root->GetChildren())->GetId() == wxT("backgroundColour")
+				? root->GetChildren()->GetNextSibling() : root->GetChildren());
 	}
 	// set href
 	if (image)
@@ -155,25 +182,70 @@ void Menu::SetBackground(wxSVGSVGElement* root, wxString fileName) {
 		((wxSVGVideoElement*) bgElement)->SetHref(fileName);
 }
 
-wxString Menu::GetBackground(wxSVGSVGElement* root) {
-	wxSVGElement* bgElement = (wxSVGElement*) root->GetElementById(wxT("background"));
+/** Returns align of background image */
+wxSVG_PRESERVEASPECTRATIO Menu::GetBackgroundAlign() {
+	wxSVGElement* bgElement = (wxSVGElement*) m_svg->GetRootElement()->GetElementById(wxT("background"));
 	if (bgElement) {
 		if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT)
-			return ((wxSVGImageElement*) bgElement)->GetHref();
+			return ((wxSVGImageElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal().GetAlign();
 		else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT)
-			return ((wxSVGVideoElement*) bgElement)->GetHref();
+			return ((wxSVGVideoElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal().GetAlign();
 	}
-	return wxT("");
+	return wxSVG_PRESERVEASPECTRATIO_UNKNOWN;
 }
 
+/** Sets align of background image */
+void Menu::SetBackgroundAlign(wxSVG_PRESERVEASPECTRATIO align) {
+	wxSVGElement* bgElement = (wxSVGElement*) m_svg->GetRootElement()->GetElementById(wxT("background"));
+	if (bgElement) {
+		if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT) {
+			wxSVGPreserveAspectRatio ar = ((wxSVGImageElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal();
+			ar.SetAlign(align);
+			((wxSVGImageElement*) bgElement)->SetPreserveAspectRatio(ar);
+		} else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT) {
+			wxSVGPreserveAspectRatio ar = ((wxSVGVideoElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal();
+			ar.SetAlign(align);
+			((wxSVGVideoElement*) bgElement)->SetPreserveAspectRatio(ar);
+		}
+	}
+}
+
+/** Returns meetOrSlice of background image */
+wxSVG_MEETORSLICE Menu::GetBackgroundMeetOrSlice() {
+	wxSVGElement* bgElement = (wxSVGElement*) m_svg->GetRootElement()->GetElementById(wxT("background"));
+	if (bgElement) {
+		if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT)
+			return ((wxSVGImageElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal().GetMeetOrSlice();
+		else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT)
+			return ((wxSVGVideoElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal().GetMeetOrSlice();
+	}
+	return wxSVG_MEETORSLICE_UNKNOWN;
+}
+
+/** Sets meetOrSlice of background image */
+void Menu::SetBackgroundMeetOrSlice(wxSVG_MEETORSLICE meetOrSlice) {
+	wxSVGElement* bgElement = (wxSVGElement*) m_svg->GetRootElement()->GetElementById(wxT("background"));
+	if (bgElement) {
+		if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT) {
+			wxSVGPreserveAspectRatio ar = ((wxSVGImageElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal();
+			ar.SetMeetOrSlice(meetOrSlice);
+			((wxSVGImageElement*) bgElement)->SetPreserveAspectRatio(ar);
+		} else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT) {
+			wxSVGPreserveAspectRatio ar = ((wxSVGVideoElement*) bgElement)->GetPreserveAspectRatio().GetBaseVal();
+			ar.SetMeetOrSlice(meetOrSlice);
+			((wxSVGVideoElement*) bgElement)->SetPreserveAspectRatio(ar);
+		}
+	}
+}
+
+/** Sets background colour */
 void Menu::SetBackgroundColour(wxColour colour) {
-	wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
-	wxSVGRectElement* bgRect = (wxSVGRectElement*) bgElement;
-	if (!bgElement || bgElement->GetDtd() != wxSVG_RECT_ELEMENT) {
-		if (bgElement)
-			bgElement->GetParent()->RemoveChild(bgElement);
+	wxSVGRectElement* bgRect = (wxSVGRectElement*) m_svg->GetElementById(wxT("backgroundColour"));
+	if (!bgRect || bgRect->GetDtd() != wxSVG_RECT_ELEMENT) {
+		if (bgRect)
+			bgRect->GetParent()->RemoveChild(bgRect);
 		bgRect = new wxSVGRectElement;
-		bgRect->SetId(wxT("background"));
+		bgRect->SetId(wxT("backgroundColour"));
 		bgRect->SetWidth(GetResolution().GetWidth());
 		bgRect->SetHeight(GetResolution().GetHeight());
 		m_svg->GetRootElement()->InsertChild(bgRect,
@@ -183,7 +255,7 @@ void Menu::SetBackgroundColour(wxColour colour) {
 }
 
 wxColour Menu::GetBackgroundColour() {
-	wxSVGRectElement* bgRect = (wxSVGRectElement*) m_svg->GetElementById(wxT("background"));
+	wxSVGRectElement* bgRect = (wxSVGRectElement*) m_svg->GetElementById(wxT("backgroundColour"));
 	if (bgRect && bgRect->GetDtd() == wxSVG_RECT_ELEMENT)
 		return bgRect->GetFill().GetRGBColor();
 	return wxColour(0, 0, 0);
@@ -194,21 +266,6 @@ bool Menu::HasVideoBackground() {
 	return bgElement && bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT;
 }
 
-void Menu::SetTransparentColor() {
-	m_transpColour = wxColour(0, 0, 0);
-	wxImageHistogram h1, h2, h3;
-	wxImage* images = GetSubPictures(ssmNORMAL);
-	images[0].ComputeHistogram(h1);
-	images[1].ComputeHistogram(h2);
-	images[2].ComputeHistogram(h3);
-	for (int i = 1; i < 30; i++) {
-		m_transpColour = wxColour(i * 8, i * 8, i * 8);
-		unsigned long colour = (i * 8 << 16) + (i * 8 << 8) + i * 8;
-		if (h1.find(colour) == h1.end() && h2.find(colour) == h2.end() && h3.find(colour) == h3.end())
-			break;
-	}
-}
-
 bool Menu::IsDefElement(wxSVGElement* element) {
 	if (!element || element->GetDtd() == wxSVG_SVG_ELEMENT)
 		return false;
@@ -233,17 +290,13 @@ void Menu::RemoveChangeable(wxSVGElement* element, MenuObject* obj) {
 			bool isChangeable = false;
 			for (int i = 0; i < obj->GetObjectParamsCount(); i++) {
 				MenuObjectParam* param = obj->GetObjectParam(i);
-				if (param->element == elem->GetId() && param->changeable) {
+				if (param->element == elem->GetId() && param->isChangeable()) {
+					elem->SetAttribute(param->attribute, wxT("none"));
 					isChangeable = true;
-					break;
 				}
 			}
-			if (isChangeable) {
-				// todo: don't remove if it has not changeable attributes, but
-				// remove all changeable attributes: SetFill(none),SetStroke(none)
-				elem->GetParent()->RemoveChild(elem);
+			if (isChangeable)
 				continue;
-			}
 		}
 		// remove changeable children
 		RemoveChangeable(elem, obj);
@@ -269,8 +322,11 @@ bool Menu::RemoveNotChangeable(wxSVGElement* element, MenuObject* obj) {
 			for (int i = 0; i < obj->GetObjectParamsCount(); i++) {
 				MenuObjectParam* param = obj->GetObjectParam(i);
 				if (param->element == elem->GetId() && param->changeable) {
-					isChangeable = true;
-					break;
+					if (param->normalColour == param->highlightedColour
+							&& param->normalColour == param->selectedColour)
+						elem->SetAttribute(param->attribute, wxT("none"));
+					else
+						isChangeable = true;
 				}
 			}
 			if (isChangeable) {
@@ -305,12 +361,16 @@ void CopyCachedImages(wxSVGDocument* doc, wxSVGElement* parent1,
 wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 	wxSVGSVGElement* svgNode = (wxSVGSVGElement*) m_svg->GetRoot()->CloneNode();
 	CopyCachedImages(m_svg, (wxSVGElement*) m_svg->GetRoot(), svgNode);
-	// remove selection rectangle
-	wxSvgXmlElement* elem = svgNode->GetElementById(wxT("selection"));
-	if (elem)
-		elem->GetParent()->RemoveChild(elem);
+	// remove selection rectangles
+	wxSVGElement* child = (wxSVGElement*) svgNode->GetChildren();
+	while (child != NULL) {
+		wxSVGElement* childToRemove = child->GetId().StartsWith(wxT("selection_")) ? child : NULL;
+		child = (wxSVGElement*) child->GetNextSibling();
+		if (childToRemove)
+			svgNode->RemoveChild(childToRemove);
+	}
 	// remove safeTV rectangle
-	elem = svgNode->GetElementById(wxT("safeTV"));
+	wxSvgXmlElement* elem = svgNode->GetElementById(wxT("safeTV"));
 	if (elem)
 		elem->GetParent()->RemoveChild(elem);
 	// remove grid
@@ -341,19 +401,12 @@ wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 		}
 	} else if (drawType == mdBUTTONS_NORMAL || drawType == mdBUTTONS_HIGHLIGHTED || drawType == mdBUTTONS_SELECTED) {
 		// remove background and static elements of objects
-		wxSVGElement* bgElement = (wxSVGElement*) svgNode->GetElementById(wxT("background"));
-		wxSVGRectElement* bgRect = (wxSVGRectElement*) bgElement;
-		if (!bgElement || bgElement->GetDtd() != wxSVG_RECT_ELEMENT) {
-			if (bgElement)
-				bgElement->GetParent()->RemoveChild(bgElement);
-			bgRect = new wxSVGRectElement;
-			bgRect->SetId(wxT("background"));
-			bgRect->SetWidth(GetResolution().GetWidth());
-			bgRect->SetHeight(GetResolution().GetHeight());
-			svgNode->InsertChild(bgRect, svgNode->GetChildren());
-		}
-		bgRect->SetFill(wxSVGPaint(m_transpColour.Red(),
-				m_transpColour.Green(), m_transpColour.Blue()));
+		wxSVGElement* bgElement = (wxSVGElement*) svgNode->GetElementById(wxT("backgroundColour"));
+		if (bgElement)
+			bgElement->GetParent()->RemoveChild(bgElement);
+		bgElement = (wxSVGElement*) svgNode->GetElementById(wxT("background"));
+		if (bgElement)
+			bgElement->GetParent()->RemoveChild(bgElement);
 		for (unsigned int i = 0; i < GetObjectsCount(); i++) {
 			MenuObject* obj = GetObject(i);
 			if (obj->IsButton() && obj->GetAction().IsValid(m_dvd, m_tsi, m_pgci, true)
@@ -363,7 +416,7 @@ wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 					RemoveNotChangeable(symbol, obj);
 				for (int i = 0; i < obj->GetObjectParamsCount(); i++) {
 					MenuObjectParam* param = obj->GetObjectParam(i);
-					if (param->changeable && drawType != mdBUTTONS_NORMAL) {
+					if (param->isChangeable() && drawType != mdBUTTONS_NORMAL) {
 						wxSVGElement* elem = (wxSVGElement*) symbol->GetElementById(param->element);
 						if (elem && param->attribute.length()) {
 							wxSVGPaint paint(drawType == mdBUTTONS_SELECTED ? param->selectedColour
@@ -389,10 +442,10 @@ wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
 ////////////////////////////// Render ////////////////////////////////////////
 
 wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height, const wxSVGRect* rect,
-		bool preserveAspectRatio) {
+		bool preserveAspectRatio, bool alpha) {
 	wxSVGDocument svg;
 	svg.AppendChild(GetSVGCopy(drawType));
-	return svg.Render(width, height, rect, preserveAspectRatio);
+	return svg.Render(width, height, rect, preserveAspectRatio, alpha);
 }
 
 wxImage Menu::GetImage(int width, int height) {
@@ -405,37 +458,38 @@ wxImage Menu::GetBackgroundImage() {
 	return RenderImage(mdBACKGROUND, width, height);
 }
 
-bool Menu::ReduceColours() {
+int Menu::ReduceColours() {
 	// create 3d palette
 	Palette3D palette;
 	for (unsigned int i = 0; i < GetObjectsCount(); i++) {
 		MenuObject& obj = *GetObject(i);
-		if (!obj.IsButton())
+		if (!obj.IsButton() || !obj.GetAction().IsValid(m_dvd, m_tsi, m_pgci, true))
 			continue;
 		for (int j = 0; j < obj.GetObjectParamsCount(); j++) {
 			MenuObjectParam* param = obj.GetObjectParam(j);
-			if (param->changeable) {
+			if (param->isChangeable()) {
 				palette.Add(s_config.GetDrawButtonsOnBackground() ? wxColour() : param->normalColour,
 						param->highlightedColour, param->selectedColour);
 			}
 		}
 	}
 	palette.Add(wxColour(), wxColour(), wxColour());
+	wxLogMessage(wxT("Menu has %d group(s) of changeable colours."), palette.GetColoursCount() - 1);
 	// reduce the number of colours
 	if (palette.GetColoursCount() <= BUTTON_MAX_COLORS)
-		return false;
+		return palette.GetColoursCount();
+	wxLogMessage(wxT("Warning: Count of changeable colours will be reduced to %d groups."), BUTTON_MAX_COLORS - 1);
 	palette.ReduceColours(BUTTON_MAX_COLORS);
 	// apply palette
 	for (unsigned int i = 0; i < GetObjectsCount(); i++) {
 		MenuObject& obj = *GetObject(i);
-		if (!obj.IsButton())
+		if (!obj.IsButton() || !obj.GetAction().IsValid(m_dvd, m_tsi, m_pgci, true))
 			continue;
-		for (int i = 0; i < obj.GetObjectParamsCount(); i++) {
-			MenuObjectParam* param = obj.GetObjectParam(i);
-			if (param->changeable) {
+		for (int j = 0; j < obj.GetObjectParamsCount(); j++) {
+			MenuObjectParam* param = obj.GetObjectParam(j);
+			if (param->isChangeable()) {
 				wxColour tmp;
-				if (palette.Apply(s_config.GetDrawButtonsOnBackground() ? tmp : param->normalColour,
-						param->highlightedColour, param->selectedColour)) {
+				if (palette.Apply(param, s_config.GetDrawButtonsOnBackground())) {
 					if (!s_config.GetDrawButtonsOnBackground()) 
 						obj.SetParamColour(param->name, param->normalColour, mbsNORMAL);
 					obj.SetParamColour(param->name, param->highlightedColour, mbsHIGHLIGHTED);
@@ -444,13 +498,20 @@ bool Menu::ReduceColours() {
 			}
 		}
 	}
-	return true;
+	
+	return BUTTON_MAX_COLORS;
+}
+
+inline unsigned char GetAlpha(int alphaCount, unsigned char srcAlpha) {
+	if (alphaCount == 3)
+		return srcAlpha > 192 ? 255 : srcAlpha > 127 ? 170 : 85;
+	else
+		return 255;
 }
 
 /** Returns subPictures */
 wxImage* Menu::GetSubPictures(SubStreamMode mode) {
 	ReduceColours();
-
 	// render images
 	int width = GetFrameResolution().GetWidth();
 	int height = GetFrameResolution().GetHeight();
@@ -465,42 +526,67 @@ wxImage* Menu::GetSubPictures(SubStreamMode mode) {
 		width *= 4.0/3; // increase width to get same width as without crop rect
 	}
 	wxImage* images = new wxImage[3];
-	images[0] = RenderImage(mdBUTTONS_NORMAL, width, height, cropRect);
-	images[1] = RenderImage(mdBUTTONS_HIGHLIGHTED, width, height, cropRect);
-	images[2] = RenderImage(mdBUTTONS_SELECTED, width, height, cropRect);
+	images[0] = RenderImage(mdBUTTONS_NORMAL, width, height, cropRect, false, true);
+	images[1] = RenderImage(mdBUTTONS_HIGHLIGHTED, width, height, cropRect, false, true);
+	images[2] = RenderImage(mdBUTTONS_SELECTED, width, height, cropRect, false, true);
 
 	// make aliasing for buttons
 	for (unsigned int i = 0; i < GetObjectsCount(); i++) {
 		MenuObject& obj = *GetObject(i);
-		if (!obj.IsButton())
+		if (!obj.IsButton() || !obj.GetAction().IsValid(m_dvd, m_tsi, m_pgci, true))
 			continue;
 		// make palette
-		MenuPalettes objPalette(obj, m_transpColour, s_config.GetDrawButtonsOnBackground());
+		MenuPalettes objPalette(obj, s_config.GetDrawButtonsOnBackground());
+		int alphaCount = obj.GetChangebaleColourCount(s_config.GetDrawButtonsOnBackground()) == 1 ? 3 : 1;
 
 		// apply palette
 		wxRect rect = obj.GetFrameBBox(mode, true);
 		unsigned char* img1 = images[0].GetData() + rect.GetY() * images[0].GetWidth() * 3 + rect.GetX() * 3;
+		unsigned char* img1Alpha = images[0].GetAlpha() + rect.GetY() * images[0].GetWidth() + rect.GetX();
 		unsigned char* img2 = images[1].GetData() + rect.GetY() * images[1].GetWidth() * 3 + rect.GetX() * 3;
+		unsigned char* img2Alpha = images[1].GetAlpha() + rect.GetY() * images[1].GetWidth() + rect.GetX();
 		unsigned char* img3 = images[2].GetData() + rect.GetY() * images[2].GetWidth() * 3 + rect.GetX() * 3;
+		unsigned char* img3Alpha = images[2].GetAlpha() + rect.GetY() * images[2].GetWidth() + rect.GetX();
 		for (int y = 0; y < rect.GetHeight(); y++) {
 			for (int x = 0; x < rect.GetWidth(); x++) {
-				objPalette.Apply(img1, img1);
-				objPalette.Apply(img1, img2, img2);
-				objPalette.Apply(img1, img2, img3, img3);
+				unsigned char alpha = 0;
+				objPalette.Apply(img1, img1, img1Alpha);
+				if (*img1Alpha > 0) {
+					alpha = GetAlpha(alphaCount, *img1Alpha);
+					*img1Alpha = alpha;
+				}
+				objPalette.Apply(img1, *img1Alpha, img2, img2, img2Alpha);
+				if (*img2Alpha > 0) {
+					if (alpha == 0)
+						alpha = GetAlpha(alphaCount, *img2Alpha);
+					*img2Alpha = alpha;
+				}
+				objPalette.Apply(img1, *img1Alpha, img2, *img2Alpha, img3, img3, img3Alpha);
+				if (*img3Alpha > 0) {
+					if (alpha == 0)
+						alpha = GetAlpha(alphaCount, *img3Alpha);
+					*img3Alpha = alpha;
+				}
 				img1 += 3;
+				img1Alpha++;
 				img2 += 3;
+				img2Alpha++;
 				img3 += 3;
+				img3Alpha++;
 			}
 			img1 += (images[0].GetWidth() - rect.GetWidth()) * 3;
+			img1Alpha += images[0].GetWidth() - rect.GetWidth();
 			img2 += (images[1].GetWidth() - rect.GetWidth()) * 3;
+			img2Alpha += images[1].GetWidth() - rect.GetWidth();
 			img3 += (images[2].GetWidth() - rect.GetWidth()) * 3;
+			img3Alpha += images[2].GetWidth() - rect.GetWidth();
 		}
 	}
 
 	// all pixels that don't belong to objects must be transparent
-	unsigned char* img1 = images[0].GetData();
-	unsigned char* img2 = images[1].GetData();
-	unsigned char* img3 = images[2].GetData();
+	unsigned char* img1 = images[0].GetAlpha();
+	unsigned char* img2 = images[1].GetAlpha();
+	unsigned char* img3 = images[2].GetAlpha();
 	for (int y = 0; y < images[0].GetHeight(); y++) {
 		for (int x = 0; x < images[0].GetWidth(); x++) {
 			bool found = false;
@@ -517,19 +603,13 @@ wxImage* Menu::GetSubPictures(SubStreamMode mode) {
 				}
 			}
 			if (!found) {
-				img1[0] = m_transpColour.Red();
-				img1[1] = m_transpColour.Green();
-				img1[2] = m_transpColour.Blue();
-				img2[0] = m_transpColour.Red();
-				img2[1] = m_transpColour.Green();
-				img2[2] = m_transpColour.Blue();
-				img3[0] = m_transpColour.Red();
-				img3[1] = m_transpColour.Green();
-				img3[2] = m_transpColour.Blue();
+				*img1 = 0;
+				*img2 = 0;
+				*img3 = 0;
 			}
-			img1 += 3;
-			img2 += 3;
-			img3 += 3;
+			img1++;
+			img2++;
+			img3++;
 		}
 	}
 	
@@ -537,7 +617,7 @@ wxImage* Menu::GetSubPictures(SubStreamMode mode) {
 		for (int i = 0; i < 3; i++) {
 			wxSize size(images[i].GetWidth(), images[i].GetHeight() + 2*padY);
 			wxPoint pos(0, padY);
-			images[i].Resize(size, pos, m_transpColour.Red(), m_transpColour.Green(), m_transpColour.Blue());
+			images[i].Resize(size, pos);
 		}
 	}
 
@@ -638,12 +718,13 @@ bool Menu::UpdateButtonImageFor(int actionTsi, int actionPgci, DVD* dvd) {
 				if (param->type == _T("image")) {
 					PgcArray& pgcs = dvd->GetPgcArray(actionTsi == -2 ? m_tsi : actionTsi, false);
 					wxString uri;
-					if (actionPgci < (int)pgcs.size() && pgcs[actionPgci]->GetVobs().size() > 0)
-						uri = pgcs[actionPgci]->GetVobs()[0]->GetVideoFrameUri(obj->GetAction().GetChapter());
+					if (actionPgci < (int)pgcs.size())
+						uri = pgcs[actionPgci]->GetVideoFrameUri(obj->GetAction().GetChapter());
 					wxString uriFname = uri.Find(wxT('#')) >= 0 ? uri.BeforeLast(wxT('#')) : uri;
 					if (!obj->IsCustomVideoFrame() || uri.length() == 0
 							|| uriFname != obj->GetParam(param->name).BeforeLast(wxT('#'))) {
 						obj->SetParam(param->name, uri);
+						obj->UpdateSize();
 						result = true;
 					}
 				}
@@ -683,7 +764,6 @@ bool Menu::SaveSpumux(wxString fileName, SubStreamMode mode, wxString btFile, wx
 	spuNode->AddProperty(_T("image"), btFile);
 	spuNode->AddProperty(_T("highlight"), hlFile);
 	spuNode->AddProperty(_T("select"), selFile);
-	spuNode->AddProperty(_T("transparent"), SConv::ToString(m_transpColour, false));
 	spuNode->AddProperty(_T("force"), _T("yes"));
 	// buttons
 	for (int i=0; i<(int)GetObjectsCount(); i++)
@@ -694,7 +774,7 @@ bool Menu::SaveSpumux(wxString fileName, SubStreamMode mode, wxString btFile, wx
 	}
 	// actions
 	for (unsigned int i=0; i<m_actions.Count(); i++)
-		spuNode->AddChild(m_actions[i]->GetXML(SPUMUX_XML, false));
+		spuNode->AddChild(m_actions[i]->GetXML(SPUMUX_XML, NULL));
 	streamNode->AddChild(spuNode);
 	root->AddChild(streamNode);
 	xml.SetRoot(root);
@@ -712,6 +792,8 @@ wxSvgXmlNode* Menu::GetXML(DVDFileType type, wxSvgXmlNode* node) {
 			node->AddProperty(_T("startTime"), GetStartTime());
 		if (GetEndTime().Length())
 			node->AddProperty(_T("endTime"), GetEndTime());
+		if (m_rememberLastButton)
+			node->AddProperty(wxT("rememberLastButton"), wxString::Format(wxT("%d"), m_rememberLastButtonRegister));
 		
 		// add svg
 		if (m_svg && m_svg->GetRoot()) {
@@ -720,8 +802,12 @@ wxSvgXmlNode* Menu::GetXML(DVDFileType type, wxSvgXmlNode* node) {
 			wxString bgFile = GetBackground(svg);
 			if (bgFile.length() > 0) {
 				wxString dir = BACKGROUNDS_DIR;
+				wxStandardPaths paths;
+				wxString bgDir = paths.GetUserDataDir() + wxFILE_SEP_PATH + wxT("backgrounds") + wxFILE_SEP_PATH;
 				if (bgFile.StartsWith(dir))
 	 				SetBackground(svg, bgFile.substr(dir.length()));
+				else if (bgFile.StartsWith(bgDir))
+	 				SetBackground(svg, bgFile.substr(bgDir.length()));
 			}
 			// set relative path
 			wxSVGElement* elem = svg;
@@ -783,6 +869,9 @@ bool Menu::PutXML(wxSvgXmlNode* node) {
 		    m_aspectRatio = AspectRatio(lval);
 		node->GetPropVal(wxT("startTime"), &m_startTime);
 		node->GetPropVal(wxT("endTime"), &m_endTime);
+		m_rememberLastButton = node->GetPropVal(wxT("rememberLastButton"), &val) && val.ToLong(&lval);
+		if (m_rememberLastButton)
+			m_rememberLastButtonRegister = lval;
 		
 		wxSvgXmlNode* svgNode = XmlFindNode(node, wxT("svg"));
 		if (svgNode) {
@@ -792,14 +881,22 @@ bool Menu::PutXML(wxSvgXmlNode* node) {
 			xml.Save(output);
 			wxMemoryInputStream input(output);
 			m_svg->Load(input);
+			// fix background
+			wxSVGRectElement* bgRect = (wxSVGRectElement*) m_svg->GetElementById(wxT("background"));
+			if (bgRect && bgRect->GetDtd() == wxSVG_RECT_ELEMENT)
+				bgRect->SetId(wxT("backgroundColour"));
 			// fix path
 			if (GetBackground().length() > 0) {
+				wxStandardPaths paths;
+				wxString bgDir = paths.GetUserDataDir() + wxFILE_SEP_PATH + wxT("backgrounds") + wxFILE_SEP_PATH;
 				wxFileName fn(GetBackground());
 				if (fn.IsRelative()) {
 					if (wxFileExists(m_dvd->GetPath() + fn.GetFullPath()))
 						SetBackground(m_dvd->GetPath() + fn.GetFullPath());
 					else if (wxFileExists(m_dvd->GetPath() + fn.GetFullName()))
 						SetBackground(m_dvd->GetPath() + fn.GetFullName());
+					else if (wxFileExists(bgDir + fn.GetFullName()))
+						SetBackground(bgDir + fn.GetFullName());
 					else if (wxFileExists(BACKGROUNDS_DIR + fn.GetFullName()))
 						SetBackground(BACKGROUNDS_DIR + fn.GetFullName());
 					else
@@ -873,20 +970,25 @@ bool Menu::PutXML(wxSvgXmlNode* node) {
 			m_svg->GetRootElement()->SetWidth(GetResolution().GetWidth());
 			m_svg->GetRootElement()->SetHeight(GetResolution().GetHeight());
 			// background
-			wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
+			wxSVGElement* bgElement = m_svg->GetElementById(wxT("backgroundColour"));
+			if (bgElement && bgElement->GetDtd() == wxSVG_RECT_ELEMENT) {
+				((wxSVGRectElement*) bgElement)->SetWidth(GetResolution().GetWidth());
+				((wxSVGRectElement*) bgElement)->SetHeight(GetResolution().GetHeight());
+			}
+			bgElement = m_svg->GetElementById(wxT("background"));
 			if (bgElement) {
-				if (bgElement->GetDtd() == wxSVG_RECT_ELEMENT) {
-					((wxSVGRectElement*) bgElement)->SetWidth(GetResolution().GetWidth());
-					((wxSVGRectElement*) bgElement)->SetHeight(GetResolution().GetHeight());
-				} else if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT) {
+				if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT) {
 					((wxSVGImageElement*) bgElement)->SetWidth(GetResolution().GetWidth());
 					((wxSVGImageElement*) bgElement)->SetHeight(GetResolution().GetHeight());
+				} else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT) {
+					((wxSVGVideoElement*) bgElement)->SetWidth(GetResolution().GetWidth());
+					((wxSVGVideoElement*) bgElement)->SetHeight(GetResolution().GetHeight());
 				}
 			}
 			// change object coordinates
 			for (unsigned int obji = 0; obji < m_objects.GetCount(); obji++) {
 				MenuObject* obj = m_objects[obji];
-				obj->SetY(round(obj->GetY()*f));
+				obj->SetY(round(obj->GetY()*f/2)*2);
 				if (!obj->IsDefaultSize()) {
 					obj->SetWidth(round(obj->GetWidth()*f));
 					obj->SetHeight(round(obj->GetHeight()*f));
@@ -917,7 +1019,7 @@ void Menu::Deserialize(const wxString& data) {
 
 wxString Menu::AddObject(wxSvgXmlNode* node, bool fixPosition) {
     if (node->GetName() == wxT("action")) {
-        DVDAction* action = new DVDAction(m_tsi == -1, MENU_ACTION);
+        DVDAction* action = new DVDAction(m_tsi == -1);
         if (!action->PutXML(node)) {
             delete action;
             return wxT("");
@@ -925,9 +1027,20 @@ wxString Menu::AddObject(wxSvgXmlNode* node, bool fixPosition) {
         m_actions.Add(action);
         return action->GetId();
     } else if (node->GetName() == wxT("button") || node->GetName() == wxT("object")) {
-    	if (node->GetName() == wxT("button") && GetButtonsCount() > 33) {
-			wxLogError(_("DVD menu can contain maximal 34 buttons"));
-			return wxT("");
+    	if (node->GetName() == wxT("button")) {
+    		if (GetAspectRatio() == ar16_9) {
+    			bool wtAuto = m_dvd == NULL || m_dvd->GetPgcArray(m_tsi, true).GetVideo().GetWidescreen() == wtAUTO;
+    			if (wtAuto && GetButtonsCount() >= 12) {
+					wxLogError(_("Wide screen DVD menu can contain maximal 12 buttons"));
+					return wxT("");
+				} else if (GetButtonsCount() >= 18) {
+					wxLogError(_("Wide screen DVD menu (nopanscan/noletterbox) can contain maximal 18 buttons"));
+					return wxT("");
+				}
+			} else if (GetButtonsCount() >= 34) {
+				wxLogError(_("DVD menu can contain maximal 34 buttons"));
+				return wxT("");
+			}
 		}
     		
         MenuObject* obj = new MenuObject(this, m_tsi == -1);
