@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	    04.11.2003
-// RCS-ID:      $Id: Menu.h,v 1.54 2010/09/05 10:53:06 ntalex Exp $
+// RCS-ID:      $Id: Menu.h,v 1.57 2011/08/14 08:43:49 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,6 +13,7 @@
 
 #include "MenuObject.h"
 #include "mediaenc.h"
+#include <wxSVG/SVGPreserveAspectRatio.h>
 
 enum MenuDrawType {
 	mdALL,
@@ -39,6 +40,7 @@ public:
 	~Menu();
 	
 	inline wxSVGDocument* GetSVG() { return m_svg; }
+	inline int GetTsi() { return m_tsi; }
 	
 	/** Returns video format */
 	inline VideoFormat GetVideoFormat() { return m_videoFormat; }
@@ -58,10 +60,23 @@ public:
 	inline wxString GetEndTime() { return m_endTime; }
 	inline void 	SetEndTime(wxString time) { m_endTime = time; }
 	
-	void SetBackground(wxString fileName);
+	/** Returns filename of background image/video */
 	wxString GetBackground();
-	void SetBackgroundColour(wxColour colour);
+	/** Sets background image/video */
+	void SetBackground(wxString fileName);
+	/** Returns align of background image */
+	wxSVG_PRESERVEASPECTRATIO GetBackgroundAlign();
+	/** Sets align of background image */
+	void SetBackgroundAlign(wxSVG_PRESERVEASPECTRATIO align);
+	/** Returns meetOrSlice of background image */
+    wxSVG_MEETORSLICE GetBackgroundMeetOrSlice();
+    /** Sets meetOrSlice of background image */
+    void SetBackgroundMeetOrSlice(wxSVG_MEETORSLICE meetOrSlice);
+	/** Returns background colour */
 	wxColour GetBackgroundColour();
+	/** Sets background colour */
+	void SetBackgroundColour(wxColour colour);
+	/** Returns true if background is a video */
 	bool HasVideoBackground();
 	
 	wxString AddImage(wxString fileName, int x = -1, int y = -1);
@@ -104,12 +119,15 @@ public:
 	inline bool GetRememberLastButton() { return m_rememberLastButton; }
 	/** Sets if the last selected button must be remembered */
 	inline void SetRememberLastButton(bool rememberLastButton) { m_rememberLastButton = rememberLastButton; }
+	/** Returns g-register for "remember last selected button" */
+	inline int GetRememberLastButtonRegister() { return m_rememberLastButtonRegister; }
+	/** Sets g-register for "remember last selected button" */
+	inline void SetRememberLastButtonRegister(int gregister) { m_rememberLastButtonRegister = gregister; }
 
-	void SetTransparentColor();
 	wxImage GetImage(int width, int height);
 	wxImage GetBackgroundImage();
 	
-	bool ReduceColours();
+	int ReduceColours();
 	/** Returns subPictures */
 	wxImage* GetSubPictures(SubStreamMode mode);
 	wxSVGDocument* GetBackgroundSVG();
@@ -141,10 +159,10 @@ private:
 	VideoFormat m_videoFormat;
 	AspectRatio m_aspectRatio;
 	wxString 	m_startTime, m_endTime;
-	wxColour 	m_transpColour;
 	MenuObjects m_objects;
 	DVDActions m_actions;
 	bool m_rememberLastButton;
+	int m_rememberLastButtonRegister;
 	void SetBackground(wxSVGSVGElement* svg, wxString fileName);
 	wxString GetBackground(wxSVGSVGElement* svg);
 	void FixPosition(MenuObject* obj); /** for copy & paste */
@@ -153,7 +171,7 @@ private:
 	bool RemoveNotChangeable(wxSVGElement* element, MenuObject* obj);
 	wxSVGSVGElement* GetSVGCopy(MenuDrawType drawType = mdALL);
 	wxImage RenderImage(MenuDrawType drawType, int width, int height, const wxSVGRect* rect = NULL,
-			bool preserveAspectRatio = false);
+			bool preserveAspectRatio = false, bool alpha = false);
 	wxImage GetImage(MenuDrawType drawType, int width = -1, int height = -1);
 };
 
