@@ -3,7 +3,7 @@
 // Purpose:     The widget to edit a DVD Menu
 // Author:      Alex Thuering
 // Created:     11.10.2003
-// RCS-ID:      $Id: MenuEditor.cpp,v 1.49 2010/08/15 17:54:16 ntalex Exp $
+// RCS-ID:      $Id: MenuEditor.cpp,v 1.60 2011/11/23 22:01:30 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -53,21 +53,21 @@ public:
 	
 	wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def) {
 		if (!GetData())
-			return wxDragNone;
+			return wxDragError;
 		
 		wxDataObjectComposite* dobj = (wxDataObjectComposite*) m_dataObject;
 		
 		if (dobj->GetReceivedFormat() == wxDF_FILENAME) {
-			return OnDropFiles( x, y, m_fileDataObject->GetFilenames() ) ? def : wxDragNone;
+			return OnDropFiles( x, y, m_fileDataObject->GetFilenames() ) ? def : wxDragError;
 		} else if (dobj->GetReceivedFormat().GetId() == DATAFORMAT_ACTION) {
 			wxString actionStr((const char*) m_actionDataObject->GetData(), wxConvISO8859_1, m_actionDataObject->GetSize());
-			DVDAction action(false, MENU_ACTION);
+			DVDAction action;
 			action.Deserialize(actionStr);
 			AddButton(x, y, action.GetTsi(), action.GetPgci(), action.IsMenu());
 			return def;
 		}
 		
-		return wxDragNone;
+		return wxDragError;
 	}
 
 	bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames) {
@@ -147,9 +147,9 @@ public:
 			MenuObject* obj = m_menuEditor->GetMenu()->GetObject(objId);
 			MenuObjectParam* initParam = obj->GetInitParam();
 			if (initParam && initParam->type == wxT("image")) {
-				Vob* vob = m_menuEditor->GetDVD()->GetVob(tsi, false, pgci, 0);
-				if (vob) {
-					obj->SetParam(initParam->name, vob->GetVideoFrameUri(0));
+				Pgc* pgc = m_menuEditor->GetDVD()->GetPgc(tsi, false, pgci);
+				if (pgc) {
+					obj->SetParam(initParam->name, pgc->GetVideoFrameUri(0));
 					m_menuEditor->Select(objId);
 				}
 			}
@@ -234,6 +234,7 @@ BEGIN_EVENT_TABLE(MenuEditor, wxSVGCtrl)
   EVT_LEFT_DCLICK(MenuEditor::OnMouseDClick)
   EVT_KEY_DOWN(MenuEditor::OnKeyDown)
   EVT_MENU(MENUEDITOR_PROP_ID, MenuEditor::OnProperties)
+  EVT_UPDATE_UI(MENUEDITOR_PROP_ID, MenuEditor::OnUpdateUIProperties)
   EVT_MENU(MENUEDITOR_OBJMENU_CUT_ID, MenuEditor::OnObjectCut)
   EVT_MENU(MENUEDITOR_OBJMENU_COPY_ID, MenuEditor::OnObjectCopy)
   EVT_MENU(MENUEDITOR_OBJMENU_PASTE_ID, MenuEditor::OnObjectPaste)
@@ -275,7 +276,9 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id),
 	m_safeTV = true;
 	m_showGrid = false;
 	m_gridOverObejcts = false;
-	m_grid = 3;
+	m_grid = 2;
+	m_doCopy = false;
+	m_copied = false;
 	SetDropTarget(new MenuDnDFile(this));
 	SetFitToFrame(true);
 	
@@ -297,7 +300,7 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id),
 	int addId = MENUEDITOR_ADDMENU_OBJECT_ID;
 	while (!fname.IsEmpty()) {
 		MenuObject obj(NULL, false, fname);
-		m_addMenu->Append(addId++, wxGetTranslation((const wxChar*)obj.GetTitle().GetData()));
+		m_addMenu->Append(addId++, wxGetTranslation(obj.GetTitle().c_str()));
 		m_addMenuObjects.Add(obj.GetFileName());
 		fname = wxFindNextFile();
 	}
@@ -356,6 +359,15 @@ MenuEditor::MenuEditor(wxWindow *parent, wxWindowID id): wxSVGCtrl(parent, id),
 }
 
 void MenuEditor::SetMenu(DVD* dvd, Menu* menu, int tsi, int pgci) {
+	if (menu != NULL) {
+		// remove old selection
+		for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+			wxSVGRectElement* selRect = (wxSVGRectElement*) m_doc->GetElementById(wxT("selection_") + *it);
+			if (selRect)
+				selRect->GetParent()->RemoveChild(selRect);
+		}
+		m_selectedIds.clear();
+	}
 	m_dvd = dvd;
 	m_menu = menu;
 	m_tsi = tsi;
@@ -366,7 +378,7 @@ void MenuEditor::SetMenu(DVD* dvd, Menu* menu, int tsi, int pgci) {
 		ShowGrid();
 	} else
 		Clear();
-	DoSelect();
+	Refresh();
 }
 
 bool MenuEditor::SetBackground(wxString fname) {
@@ -433,9 +445,14 @@ bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool
 	if (!m_menu || !wxFileExists(fname))
 		return false;
 	
-	if (m_menu->GetAspectRatio() == ar16_9 && m_menu->GetButtonsCount() >= 12) {
-		wxLogError(_("Wide screen DVD menu can contain maximal 12 buttons"));
-		return false;
+	if (m_menu->GetAspectRatio() == ar16_9) {
+		if (m_dvd->GetPgcArray(m_tsi, true).GetVideo().GetWidescreen() == wtAUTO && m_menu->GetButtonsCount() >= 12) {
+			wxLogError(_("Wide screen DVD menu can contain maximal 12 buttons"));
+			return false;
+		} else if (m_menu->GetButtonsCount() >= 18) {
+			wxLogError(_("Wide screen DVD menu (nopanscan/noletterbox) can contain maximal 18 buttons"));
+			return false;
+		}
 	} else if (m_menu->GetButtonsCount() >= 34) {
 		wxLogError(_("DVD menu can contain maximal 34 buttons"));
 		return false;
@@ -450,18 +467,18 @@ bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool
 	}
 
 	wxString param;
-	tsi = tsi >= 0 ? tsi : m_tsi >= 0 ? m_tsi : 0;
+	tsi = tsi >= -1 ? tsi : m_tsi >= -1 ? m_tsi : 0;
 
 	MenuObject tmpObj(NULL, false, fname);
 	MenuObjectParam* initParam = tmpObj.GetInitParam();
 	if (initParam) {
-		Vob* vob = m_dvd->GetVob(tsi, false, pgci >= 0 ? pgci : 0, 0);
+		Pgc* pgc = m_dvd->GetPgc(tsi, false, pgci >= 0 ? pgci : 0);
 		if (initParam->type == wxT("image")) {
-			if (vob)
-				param = vob->GetVideoFrameUri(0);
+			if (pgc)
+				param = pgc->GetVideoFrameUri(0);
 		} else if (pgci >= 0 && !isMenu) {
-			if (vob)
-				param = wxFileName(vob->GetVideoFrameUri(0)).GetName();
+			if (pgc)
+				param = wxFileName(pgc->GetVideoFrameUri(0)).GetName();
 		} else {
 			if (isMenu) {
 				param = wxString::Format(tsi == -1 ? _("vmMenu %d") : _("Menu %d"), pgci+1);
@@ -475,12 +492,24 @@ bool MenuEditor::AddButton(wxString fname, int x, int y, int pgci, int tsi, bool
 	obj->GetAction().SetTsi(tsi == m_tsi ? -2 : tsi);
 	obj->GetAction().SetPgci(pgci >= 0 ? pgci : 0);
 	obj->GetAction().SetMenu(isMenu);
-
+	
 	// fix position
-	int mx = 0;
-	int my = 0;
-	MoveObjectInt(obj, mx, my);
-
+	wxRect rect = obj->GetBBox();
+	x = obj->GetX() >> m_grid << m_grid;
+	y = obj->GetY() >> m_grid << m_grid;
+	int maxWidth = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal();
+	int maxHeight = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal();
+	if (x < 0)
+		x = 0;
+	else if (x + rect.width >= maxWidth)
+		x = maxWidth - rect.width - 1;
+	if (y < 0)
+		y = 0;
+	else if (y + rect.height >= maxHeight)
+		y = maxHeight - rect.height - 1;
+	obj->SetX(x);
+	obj->SetY(y);
+	
 	// select
 	DoSelect(objId, true, true);
 	return true;
@@ -538,45 +567,83 @@ wxString MenuEditor::HitTest(int x, int y) {
 	return id;
 }
 
-void MenuEditor::DoSelect(wxString id, bool refresh, bool sendEvent) {
+void MenuEditor::DoSelect(wxString id, bool refresh, bool sendEvent, bool ctrlDown) {
 	if (!m_menu)
 		return;
-
-	m_selected = id;
-
-	// remove old selection
-	wxSVGRectElement* selRect = (wxSVGRectElement*) m_doc->GetElementById(wxT("selection"));
-	if (selRect) {
-		if (refresh)
-			RefreshRect(ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY()).Inflate(4));
-		selRect->GetParent()->RemoveChild(selRect);
+	
+	if (!ctrlDown) {
+		// remove old selection
+		wxRect rect;
+		for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+			wxSVGRectElement* selRect = (wxSVGRectElement*) m_doc->GetElementById(wxT("selection_") + *it);
+			if (selRect) {
+				if (refresh)
+					rect.Union(ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY()));
+				selRect->GetParent()->RemoveChild(selRect);
+			}
+		}
+		m_selectedIds.clear();
+		if (refresh && !rect.IsEmpty())
+			RefreshRect(rect.Inflate(8));
 	}
+	
+	if (m_selectedIds.Index(id) != wxNOT_FOUND) {
+		// remove old selection
+		wxSVGRectElement* selRect = (wxSVGRectElement*) m_doc->GetElementById(wxT("selection_") + id);
+		if (selRect) {
+			if (refresh)
+				RefreshRect(ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY()).Inflate(8));
+			selRect->GetParent()->RemoveChild(selRect);
+		}
+		m_selectedIds.Remove(id);
+		return;
+	}
+	if (id.length())
+		m_selectedIds.push_back(id);
 
 	// create selection rectangle
 	if (id.length()) {
-		MenuObject* obj = m_menu->GetObject(id);
-		if (!obj)
-			return;
-		wxRect bbox = obj->GetBBox();
-
-		selRect = new wxSVGRectElement;
-		selRect->SetId(wxT("selection"));
-		selRect->SetX(bbox.GetX());
-		selRect->SetY(bbox.GetY());
-		selRect->SetWidth(bbox.GetWidth());
-		selRect->SetHeight(bbox.GetHeight());
-		selRect->SetStroke(wxSVGPaint(255, 0, 0));
-		selRect->SetFill(wxSVGPaint());
-		m_doc->GetRootElement()->AppendChild(selRect);
-
-		if (refresh)
-			RefreshRect(ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY()).Inflate(4));
+		wxRect rect = CreateSelection(id);
+		if (refresh && !rect.IsEmpty())
+			RefreshRect(rect.Inflate(8));
 	}
 
 	if (sendEvent)
 		SendChangedEvent();
 }
 
+wxRect MenuEditor::CreateSelection(wxString id) {
+	MenuObject* obj = m_menu->GetObject(id);
+	if (!obj)
+		return wxRect();
+	
+	wxRect refreshRect;
+	wxSVGRectElement* selRect = (wxSVGRectElement*) m_doc->GetRootElement()->GetElementById(wxT("selection_") + id);
+	if (selRect == NULL) {
+		selRect = new wxSVGRectElement;
+		selRect->SetId(wxT("selection_") + id);
+		selRect->SetStroke(wxSVGPaint(255, 0, 0));
+		selRect->SetFill(wxSVGPaint());
+		m_doc->GetRootElement()->AppendChild(selRect);
+	} else
+		refreshRect = ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY());
+	selRect->SetX(obj->GetX());
+	selRect->SetY(obj->GetY());
+	selRect->SetWidth(obj->GetWidth());
+	selRect->SetHeight(obj->GetHeight());
+	selRect->SetCanvasItem(NULL);
+	
+	return refreshRect.Union(ScaleRect(selRect->GetBBox(), GetScaleX(), GetScaleY()));
+}
+
+void MenuEditor::RefreshSelected() {
+	wxRect rect;
+	for (wxArrayString::iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++)
+		rect.Union(CreateSelection(*it));
+	if (!rect.IsEmpty())
+		RefreshRect(rect.Inflate(8));
+}
+
 void MenuEditor::OnMouseLeftButton(wxMouseEvent &event) {
 	SetFocus();
 	if (!m_menu)
@@ -585,32 +652,33 @@ void MenuEditor::OnMouseLeftButton(wxMouseEvent &event) {
 		objectIsMoving = false;
 		SendChangedEvent();
 	}
-	int x = event.GetX();
-	int y = event.GetY();
-	m_tt = GetTransformType(x, y);
-	m_oldx = x;
-	m_oldy = y;
-	m_copy = event.ControlDown();
-	if (event.LeftDown() && m_tt == ttOut) {
-		// select object
-		DoSelect(HitTest(x, y), true);
-		// change mouse cursor
-		event.m_leftDown = false;
-		OnMouseMove(event);
+	if (event.LeftDown()) {
+		m_action = GetEditorAction(event.GetX(), event.GetY());
+		m_oldx = event.GetX();
+		m_oldy = event.GetY();
+		m_doCopy = event.ControlDown();
+		m_copied = false;
+	} else {
+		if (m_action == eaNONE || (event.ControlDown() && !m_copied)) {
+			// select object
+			DoSelect(HitTest(event.GetX(), event.GetY()), true, false, event.ControlDown());
+			// change mouse cursor
+			OnMouseMove(event);
+		}
 	}
 }
 
 void MenuEditor::OnMouseMove(wxMouseEvent &event) {
-	if (!m_menu)
+	if (!m_menu || event.RightDown())
 		return;
 
 	int x = event.GetX();
 	int y = event.GetY();
 
 	if (!event.LeftIsDown()) {
-		m_tt = GetTransformType(x, y);
+		m_action = GetEditorAction(x, y);
 		// change mouse cursor
-		SetMouseCursor(m_tt);
+		SetMouseCursor(m_action);
 		// object pointed
 		wxString pointed = HitTest(x, y);
 		if (GetPointed() != pointed) {
@@ -622,20 +690,20 @@ void MenuEditor::OnMouseMove(wxMouseEvent &event) {
 	}
 
 	// left mouse button is down => move or resize
-	if (m_tt == ttIn) {
+	if (m_action == eaMOVE) {
 		// move Object
 		int x1 = x - m_oldx;
 		int y1 = y - m_oldy;
 		if (MoveObject(x1, y1)) {
-			DoSelect(m_selected, true); // refresh
+			RefreshSelected();
 			objectIsMoving = true;
 		}
 		m_oldx += x1;
 		m_oldy += y1;
-	} else if (m_tt != ttOut) { // m_tt=TL,TR,BL,BR
+	} else if (m_action != eaNONE) { // m_action=TL,TR,BL,BR
 		// resize object
-		if (ResizeObject(x, y, m_tt)) {
-			DoSelect(m_selected, true); // refresh
+		if (ResizeObject(x, y, m_action)) {
+			DoSelect(GetSelected(), true); // refresh
 			objectIsMoving = true;
 		}
 	}
@@ -646,24 +714,34 @@ void MenuEditor::OnMouseRightButton(wxMouseEvent &event) {
 	if (event.RightDown() || !m_menu)
 		return;
 	// select Object
-	wxString sel_old = m_selected;
-	DoSelect(HitTest(event.GetX(), event.GetY()), true);
+	m_actionObject = HitTest(event.GetX(), event.GetY());
+	if (m_selectedIds.Index(m_actionObject) == wxNOT_FOUND)
+		DoSelect(m_actionObject, true);
 	m_menuPos = event.GetPosition();
 	if (event.RightUp())
-		PopupMenu(m_selected.length() ? m_objMenu : m_pmenu, event.GetPosition());
+		PopupMenu(m_selectedIds.size() ? m_objMenu : m_pmenu, event.GetPosition());
 }
 
-TransformType MenuEditor::GetTransformType(int x, int y) {
-	if (!m_menu || !m_selected.length())
-		return ttOut;
+EditorAction MenuEditor::GetEditorAction(int x, int y) {
+	if (!m_menu || m_selectedIds.size() == 0)
+		return eaNONE;
+	
+	m_actionObject = HitTest(x, y);
+	
+	if (m_selectedIds.size() > 1) {
+		if (m_selectedIds.Index(m_actionObject) == wxNOT_FOUND)
+			return eaNONE; // outside of the selected objects
+		return eaMOVE; // inside of the selected objects
+	}
 
 	// test all covered object
-	if (HitTest(x, y) != m_selected)
-		return ttOut; // outside of the selected object
-
-	MenuObject* obj = m_menu->GetObject(m_selected);
+	if (m_actionObject != GetSelected())
+		return eaNONE; // outside of the selected object
+	
+	MenuObject* obj = m_menu->GetObject(m_actionObject);
 	if (!obj)
-		return ttOut;
+		return eaNONE;
+	
 	wxRect rect = ScaleRect(obj->GetBBox(), GetScaleX(), GetScaleY());
 	wxRect rect2 = rect;
 	rect2.Deflate(3, 3);
@@ -671,147 +749,164 @@ TransformType MenuEditor::GetTransformType(int x, int y) {
 		int x1 = rect.x + rect.width / 2;
 		int y1 = rect.y + rect.height / 2;
 		if ((x < x1) && (y < y1))
-			return ttTL; // top-left
+			return eaRESIZE_TL; // top-left
 		else if ((x > x1) && (y < y1))
-			return ttTR; // top-right
+			return eaRESIZE_TR; // top-right
 		else if ((x < x1) && (y > y1))
-			return ttBL; // bottom-left
+			return eaRESIZE_BL; // bottom-left
 		else if ((x > x1) && (y > y1))
-			return ttBR; // bottom-right
+			return eaRESIZE_BR; // bottom-right
 	}
 
-	return ttIn; // inside of the selected object
+	return eaMOVE; // inside of the selected object
 }
 
-void MenuEditor::SetMouseCursor(TransformType tt) {
+void MenuEditor::SetMouseCursor(EditorAction tt) {
 	switch (tt) {
-		case ttOut: SetCursor(wxCursor(wxCURSOR_ARROW)); break;
+		case eaNONE: SetCursor(wxCursor(wxCURSOR_ARROW)); break;
 #ifdef __WXMSW__
-		case ttIn: SetCursor(wxCursor(_T("wxCURSOR_MOVE"),wxBITMAP_TYPE_CUR_RESOURCE)); break;
-		case ttTL:
-		case ttBR: SetCursor(wxCursor(wxCURSOR_SIZENWSE)); break;
-		case ttTR:
-		case ttBL: SetCursor(wxCursor(wxCURSOR_SIZENESW)); break;
+		case eaMOVE: SetCursor(wxCursor(_T("wxCURSOR_MOVE"),wxBITMAP_TYPE_CUR_RESOURCE)); break;
+		case eaRESIZE_TL:
+		case eaRESIZE_BR: SetCursor(wxCursor(wxCURSOR_SIZENWSE)); break;
+		case eaRESIZE_TR:
+		case eaRESIZE_BL: SetCursor(wxCursor(wxCURSOR_SIZENESW)); break;
 #else
-		case ttIn: SetCursor(wxImage(CURSOR_FILE(_T("move.cur"))));break;
-		case ttTL:
-		case ttBR: SetCursor(wxImage(CURSOR_FILE(_T("nwse.cur"))));break;
-		case ttTR:
-		case ttBL: SetCursor(wxImage(CURSOR_FILE(_T("nesw.cur"))));break;
+		case eaMOVE: SetCursor(wxImage(CURSOR_FILE(_T("move.cur"))));break;
+		case eaRESIZE_TL:
+		case eaRESIZE_BR: SetCursor(wxImage(CURSOR_FILE(_T("nwse.cur"))));break;
+		case eaRESIZE_TR:
+		case eaRESIZE_BL: SetCursor(wxImage(CURSOR_FILE(_T("nesw.cur"))));break;
 #endif
 	}
 }
 
 bool MenuEditor::MoveObject(int& x, int& y) {
-	if (!m_menu)
-		return false;
-
-	MenuObject* obj = m_menu->GetObject(m_selected);
-	if (!obj)
+	if (!m_menu || m_selectedIds.size() < 1)
 		return false;
+	
+	if (m_doCopy) {
+		wxArrayString objIds;
+		for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+			MenuObject* obj = m_menu->GetObject(*it);
+			wxRect rect = obj->GetBBox();
+			wxString id = m_menu->AddObject(obj->GetXML(DVDSTYLER_XML, m_dvd, ssmNORMAL, true), true);
+			if (id.length() > 0) {
+				objIds.Add(id);
+				obj = m_menu->GetObject(id);
+				if (obj) {
+					obj->SetX(rect.x);
+					obj->SetY(rect.y);
+				}
+				if (m_pointed == *it)
+					m_pointed = id;
+			}
+		}
+		for (wxArrayString::const_iterator it = objIds.begin(); it != objIds.end(); it++)
+			DoSelect(*it, true, false, it != objIds.begin());
+		wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED, this->GetId());
+		GetEventHandler()->ProcessEvent(evt);
+		
+		m_doCopy = false;
+		m_copied = true;
+	}
 
 	x = (int) round(x / GetScaleX());
 	y = (int) round(y / GetScaleY());
-	bool res = MoveObjectInt(obj, x, y);
+	bool res = MoveObjectInt(x, y);
 	x = (int) round(x * GetScaleX());
 	y = (int) round(y * GetScaleY());
 	return res;
 }
 
-bool MenuEditor::MoveObjectInt(MenuObject* obj, int& x, int& y) {
-	wxRect rect = obj->GetBBox();
-
-	if (m_copy) {
-		wxString id = m_menu->AddObject(obj->GetXML(DVDSTYLER_XML, m_dvd, ssmNORMAL, true), true);
-		if (id.length() == 0)
-			return false;
-		obj = m_menu->GetObject(id);
-		if (obj) {
-			obj->SetX(rect.x);
-			obj->SetY(rect.y);
-		}
-		DoSelect(id, true);
-		m_pointed = id;
-		wxCommandEvent evt(EVT_COMMAND_MENUEDITOR_OBJECT_POINTED, this->GetId());
-		GetEventHandler()->ProcessEvent(evt);
-		m_copy = false;
-	}
-
-	rect.x += x;
-	rect.y += y;
-
+bool MenuEditor::MoveObjectInt(int& x, int& y) {
+	MenuObject* obj = m_menu->GetObject(m_actionObject);
+	if (!obj)
+		return false;
+	
+	// span to grid
 	if (m_grid > 0) {
-		rect.y = rect.y >> m_grid << m_grid;
-		rect.x = rect.x >> m_grid << m_grid;
+		x = ((obj->GetX() + x) >> m_grid << m_grid) - obj->GetX();
+		y = ((obj->GetY() + y) >> m_grid << m_grid) - obj->GetY();
 	}
-
-	if (obj->IsButton()) {
-		if (rect.x < 0)
-			rect.x = 0;
-		else if (rect.x + rect.width >= (int) m_doc->GetRootElement()->GetWidth().GetBaseVal())
-			rect.x = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - rect.width - 1;
-		if (rect.y < 0)
-			rect.y = 0;
-		else if (rect.y + rect.height >= (int) m_doc->GetRootElement()->GetHeight().GetBaseVal())
-			rect.y = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - rect.height - 1;
-	} else {
-		if (rect.x + rect.width < 8)
-			rect.x = 8 - rect.width;
-		else if (rect.x > (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8)
-			rect.x = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8;
-		if (rect.y + rect.height < 8)
-			rect.y = 8 - rect.height;
-		else if (rect.y > (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8)
-			rect.y = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8;
+	
+	for (wxArrayString::iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+		obj = m_menu->GetObject(*it);
+		if (!obj)
+			continue;
+		
+		wxRect rect = obj->GetBBox();
+		
+		int maxWidth = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal();
+		int maxHeight = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal();
+		if (obj->IsButton()) {
+			if (rect.x + x < 0)
+				x = rect.x < 0 ? 0 : -rect.x;
+			else if (rect.x + rect.width + x >= maxWidth)
+				x = rect.x + rect.width >= maxWidth ? 0 : maxWidth - rect.x - rect.width - 1;
+			if (rect.y + y < 0)
+				y = rect.y < 0 ? 0 : -rect.y;
+			else if (rect.y + rect.height + y >= maxHeight)
+				y = rect.y + rect.height >= maxHeight ? 0 : maxHeight - rect.y - rect.height - 1;
+		} else {
+			if (rect.x + x + rect.width < 8)
+				x = rect.x + rect.width < 8 ? 0 : 8 - rect.x - rect.width;
+			else if (rect.x + x > maxWidth - 8)
+				x = rect.x > maxWidth - 8 ? 0 : maxWidth - 8 - rect.x;
+			if (rect.y + y + rect.height < 8)
+				y = rect.y + rect.height < 8 ? 0 : 8 - rect.height - rect.y;
+			else if (rect.y + y > maxHeight - 8)
+				y = rect.y > maxHeight - 8 ? 0 : maxHeight - 8 - rect.y;
+		}
 	}
-
-	x = rect.x - obj->GetX();
-	y = rect.y - obj->GetY();
-
-	if (obj->GetX() != rect.x || obj->GetY() != rect.y) {
-		obj->SetX(rect.x);
-		obj->SetY(rect.y);
-		return true;
+	
+	if (x == 0 && y == 0)
+		return false;
+	
+	// update objects
+	for (wxArrayString::iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+		obj = m_menu->GetObject(*it);
+		if (!obj)
+			continue;
+		obj->SetX(obj->GetX() + x);
+		obj->SetY(obj->GetY() + y);
 	}
-
-	return false;
+	return true;
 }
 
-bool MenuEditor::ResizeObject(int x, int y, TransformType transformType)
-{
-  x = (int) round(x/GetScaleX());
-  y = (int) round(y/GetScaleY());
-  return ResizeObjectInt(x, y, transformType);
+bool MenuEditor::ResizeObject(int x, int y, EditorAction action) {
+	x = (int) round(x/GetScaleX());
+	y = (int) round(y/GetScaleY());
+	return ResizeObjectInt(x, y, action);
 }
 
-bool MenuEditor::ResizeObjectInt(int x, int y, TransformType transformType) {
-	if (!m_menu)
+bool MenuEditor::ResizeObjectInt(int x, int y, EditorAction action) {
+	if (!m_menu || m_selectedIds.size() != 1)
 		return false;
 
-	MenuObject* obj = m_menu->GetObject(m_selected);
+	MenuObject* obj = m_menu->GetObject(GetSelected());
 	if (!obj)
 		return false;
 	wxRect rect = obj->GetBBox();
 
-	if ((transformType == ttBR || transformType == ttTR) && x < 8)
+	if ((action == eaRESIZE_BR || action == eaRESIZE_TR) && x < 8)
 		x = 8;
-	if ((transformType == ttBL || transformType == ttTL)
+	if ((action == eaRESIZE_BL || action == eaRESIZE_TL)
 			&& x > (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8)
 		x = (int) m_doc->GetRootElement()->GetWidth().GetBaseVal() - 8;
-	if ((transformType == ttBL || transformType == ttBR) && y < 8)
+	if ((action == eaRESIZE_BL || action == eaRESIZE_BR) && y < 8)
 		y = 8;
-	if ((transformType == ttTL || transformType == ttTR)
+	if ((action == eaRESIZE_TL || action == eaRESIZE_TR)
 			&& y > (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8)
 		y = (int) m_doc->GetRootElement()->GetHeight().GetBaseVal() - 8;
 
 	int sx, sy;
 
-	if ((transformType == ttBR) || (transformType == ttTR))
+	if (action == eaRESIZE_BR || action == eaRESIZE_TR)
 		sx = x - rect.x + 1;
 	else
 		sx = rect.x + rect.width - x + 1;
 
-	if ((transformType == ttBR) || (transformType == ttBL))
+	if (action == eaRESIZE_BR || action == eaRESIZE_BL)
 		sy = y - rect.y + 1;
 	else
 		sy = rect.y + rect.height - y + 1;
@@ -825,9 +920,9 @@ bool MenuEditor::ResizeObjectInt(int x, int y, TransformType transformType) {
 	obj->FixSize(sx, sy);
 
 	// set new x, y, width, height
-	if ((transformType == ttTL) || (transformType == ttBL))
+	if (action == eaRESIZE_TL || action == eaRESIZE_BL)
 		rect.x += rect.width - sx;
-	if ((transformType == ttTL) || (transformType == ttTR))
+	if (action == eaRESIZE_TL || action == eaRESIZE_TR)
 		rect.y += rect.height - sy;
 	rect.width = sx;
 	rect.height = sy;
@@ -935,7 +1030,7 @@ void MenuEditor::OnKeyDown(wxKeyEvent& event) {
 		return;
 	switch (event.GetKeyCode()) {
 		case WXK_TAB: {
-			int idx = m_selected.length() > 0 ? m_menu->GetIndexOfObject(m_selected) + 1 : 0;
+			int idx = m_selectedIds.size() > 0 ? m_menu->GetIndexOfObject(GetSelected()) + 1 : 0;
 			if (idx >= (int) m_menu->GetObjectsCount())
 				idx = 0;
 			if (m_menu->GetObject(idx))
@@ -943,19 +1038,19 @@ void MenuEditor::OnKeyDown(wxKeyEvent& event) {
 			break;
 		}
 		case WXK_DELETE:
-			if (m_selected.length()) {
+			if (m_selectedIds.size() == 1) {
 				wxCommandEvent evt;
 				OnObjectDelete(evt);
 			}
 			break;
 		case 'C':
-			if (event.ControlDown() && m_selected.length()) {
+			if (event.ControlDown() && m_selectedIds.size() == 1) {
 				wxCommandEvent evt;
 				OnObjectCopy(evt);
 			}
 			break;
 		case 'X':
-			if (event.ControlDown() && m_selected.length()) {
+			if (event.ControlDown() && m_selectedIds.size() == 1) {
 				wxCommandEvent evt;
 				OnObjectCopy(evt);
 				OnObjectDelete(evt);
@@ -977,8 +1072,10 @@ void MenuEditor::OnKeyDown(wxKeyEvent& event) {
 //////////////////////////////////////////////////////////////////////////////
 
 void MenuEditor::OnObjectCopy(wxCommandEvent& WXUNUSED(event)) {
-	CopyXmlToClipboard(m_menu->GetObject(m_selected)->GetXML(
-			DVDSTYLER_XML, m_dvd, ssmNORMAL, true), DATAFORMAT_MENU_OBJECT);
+	wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("objects"));
+	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++)
+		root->AppendChild(m_menu->GetObject(*it)->GetXML(DVDSTYLER_XML, m_dvd, ssmNORMAL, true));
+	CopyXmlToClipboard(root, DATAFORMAT_MENU_OBJECT);
 }
 
 void MenuEditor::OnObjectPaste(wxCommandEvent& WXUNUSED(event)) {
@@ -986,12 +1083,29 @@ void MenuEditor::OnObjectPaste(wxCommandEvent& WXUNUSED(event)) {
 		wxSvgXmlDocument doc;
 		if (!GetXmlFromClipboard(DATAFORMAT_MENU_OBJECT, doc))
 			return;
-		wxString id = m_menu->AddObject(doc.GetRoot(), true);
-		if (id.length() > 0) {
-			MenuObject* obj = m_menu->GetObject(id);
-			if (obj && m_menuPos.x != -1 && m_menuPos.y != -1) {
-				obj->SetX((int)(m_menuPos.x/GetScaleX()));
-				obj->SetY((int)(m_menuPos.y/GetScaleY()));
+		wxSvgXmlNode* child = doc.GetRoot()->GetFirstChild();
+		wxArrayString objIds;
+		int x = 9999;
+		int y = 9999;
+		while (child) {
+			wxString id = m_menu->AddObject(child, true);
+			if (id.length() > 0) {
+				objIds.Add(id);
+				MenuObject* obj = m_menu->GetObject(id);
+				if (obj && x > obj->GetX())
+					x = obj->GetX();
+				if (obj && y > obj->GetY())
+					y = obj->GetY();
+			}
+			child = child->GetNextSibling();
+		}
+		if (m_menuPos.x != -1 && m_menuPos.y != -1) {
+			for (wxArrayString::const_iterator it = objIds.begin(); it != objIds.end(); it++) {
+				MenuObject* obj = m_menu->GetObject(*it);
+				if (obj) {
+					obj->SetX(obj->GetX() - x + (int)(m_menuPos.x/GetScaleX()));
+					obj->SetY(obj->GetY() - y + (int)(m_menuPos.y/GetScaleY()));
+				}
 			}
 		}
 	} else if (wxTheClipboard->IsSupported(wxDF_TEXT)) {
@@ -1020,63 +1134,90 @@ void MenuEditor::OnObjectCut(wxCommandEvent& event) {
 }
 
 void MenuEditor::OnObjectDelete(wxCommandEvent& event) {
-	m_menu->RemoveObject(m_selected);
+	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++)
+		m_menu->RemoveObject(*it);
 	DoSelect(wxT(""), true, true);
 }
 
 void MenuEditor::OnFirstButton(wxCommandEvent& event) {
-	m_menu->SetFirstButton(m_selected);
+	m_menu->SetFirstButton(GetSelected());
 }
 
 void MenuEditor::OnUpdateUIFirstButton(wxUpdateUIEvent& event) {
-	event.Enable(m_menu->GetObject(m_selected)->IsButton());
-	event.Check(m_menu->IsFirstButton(m_selected));
+	event.Enable(m_selectedIds.size() == 1 && m_menu->GetObject(GetSelected())->IsButton());
+	event.Check(m_selectedIds.size() == 1 && m_menu->IsFirstButton(GetSelected()));
 }
 
 void MenuEditor::OnObjectToFront(wxCommandEvent& event) {
-	m_menu->ObjectToFront(m_selected);
-	DoSelect(m_selected, true, true);
+	m_menu->ObjectToFront(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
 void MenuEditor::OnObjectForward(wxCommandEvent& event) {
-	m_menu->ObjectForward(m_selected);
-	DoSelect(m_selected, true, true);
+	m_menu->ObjectForward(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
 void MenuEditor::OnObjectBackward(wxCommandEvent& event) {
-	m_menu->ObjectBackward(m_selected);
-	DoSelect(m_selected, true, true);
+	m_menu->ObjectBackward(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
 void MenuEditor::OnObjectToBack(wxCommandEvent& event) {
-	m_menu->ObjectToBack(m_selected);
-	DoSelect(m_selected, true, true);
+	m_menu->ObjectToBack(GetSelected());
+	DoSelect(GetSelected(), true, true);
 }
 
 void MenuEditor::OnUpdateUIObjectForward(wxUpdateUIEvent& event) {
-	event.Enable(!m_menu->IsLastObject(m_selected));
+	event.Enable(m_selectedIds.size() == 1 && !m_menu->IsLastObject(GetSelected()));
 }
 
 void MenuEditor::OnUpdateUIObjectBackward(wxUpdateUIEvent& event) {
-	event.Enable(!m_menu->IsFirstObject(m_selected));
+	event.Enable(m_selectedIds.size() == 1 && !m_menu->IsFirstObject(GetSelected()));
 }
 
 void MenuEditor::OnAlignToCenterH(wxCommandEvent& event) {
-	MenuObject* obj = m_menu->GetObject(m_selected);
-	obj->SetX((m_menu->GetResolution().GetWidth() - obj->GetWidth())/2);
-	DoSelect(m_selected, true, true);
+	int x1 = 9999;
+	int x2 = -9999;
+	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+		MenuObject* obj = m_menu->GetObject(*it);
+		if (obj->GetX() < x1)
+			x1 = obj->GetX();
+		if (obj->GetX() + obj->GetWidth() > x2)
+			x2 = obj->GetX() + obj->GetWidth();
+	}
+	int x = (m_menu->GetResolution().GetWidth() - (x2 - x1))/2;
+	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+		MenuObject* obj = m_menu->GetObject(*it);
+		obj->SetX(x + obj->GetX() - x1);
+	}
+	RefreshSelected();
+	SendChangedEvent();
 }
 
 void MenuEditor::OnAlignToCenterV(wxCommandEvent& event) {
-	MenuObject* obj = m_menu->GetObject(m_selected);
-	obj->SetY((m_menu->GetResolution().GetHeight() - obj->GetHeight())/2);
-	DoSelect(m_selected, true, true);
+	int y1 = 9999;
+	int y2 = -9999;
+	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+		MenuObject* obj = m_menu->GetObject(*it);
+		if (obj->GetY() < y1)
+			y1 = obj->GetY();
+		if (obj->GetY() + obj->GetHeight() > y2)
+			y2 = obj->GetY() + obj->GetHeight();
+	}
+	int y = (m_menu->GetResolution().GetHeight() - (y2 - y1))/2;
+	for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+		MenuObject* obj = m_menu->GetObject(*it);
+		obj->SetY(y + obj->GetY() - y1);
+	}
+	RefreshSelected();
+	SendChangedEvent();
 }
 
 void MenuEditor::OnProperties(wxCommandEvent& event) {
 	if (!m_menu)
 		return;
-	if (!m_selected.length()) {
+	if (m_selectedIds.size() == 0) {
 		// show menu properties
 		MenuPropDlg propDlg(this, m_dvd, m_tsi, m_pgci);
 		if (propDlg.ShowModal() == wxID_OK) {
@@ -1087,15 +1228,52 @@ void MenuEditor::OnProperties(wxCommandEvent& event) {
 		}
 	} else {
 		// show object properties
-		MenuObject* obj = m_menu->GetObject(m_selected);
-		if (!obj)
+		MenuObject* actionObj = m_menu->GetObject(m_actionObject);
+		if (!actionObj)
 			return;
-		MenuObjectPropDlg propDlg(this, m_selected, m_menu, m_dvd, m_tsi, m_pgci);
-		if (propDlg.ShowModal() == wxID_OK)
-			DoSelect(m_selected, true, true);
+		MenuObjectPropDlg propDlg(this, m_actionObject, m_selectedIds.size() > 1, m_menu, m_dvd, m_tsi, m_pgci);
+		if (propDlg.ShowModal() == wxID_OK) {
+			for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+				MenuObject* obj = m_menu->GetObject(*it);
+				if (obj->GetId() == m_actionObject)
+					continue;
+				for (int i = 0; i < actionObj->GetObjectParamsCount(); i++) {
+					MenuObjectParam* param = actionObj->GetObjectParam(i);
+					if (param->changeable && param->type == wxT("colour")) {
+						obj->SetParamColour(param->name, param->normalColour);
+						obj->SetParamColour(param->name, param->highlightedColour, mbsHIGHLIGHTED);
+						obj->SetParamColour(param->name, param->selectedColour, mbsSELECTED);
+					} else if (param->type == wxT("colour")) {
+						obj->SetParam(param->name, actionObj->GetParam(param->name));
+						obj->SetParam(param->name, actionObj->GetParam(param->name, wxT("-opacity")), wxT("-opacity"));
+					} else if (param->type == wxT("text")) {
+						obj->SetParamFont(param->name, actionObj->GetParamFont(param->name));
+					} else if (param->type != _T("image") && param->attribute.Find(wxT('#')) == wxNOT_FOUND) {
+						obj->SetParam(param->name, actionObj->GetParam(param->name));
+					}
+				}
+				obj->UpdateSize();
+			}
+			RefreshSelected();
+			SendChangedEvent(true);
+		}
 	}
 }
 
+void MenuEditor::OnUpdateUIProperties(wxUpdateUIEvent& event) {
+	bool enable = true;
+	if (m_selectedIds.size() > 0) {
+		wxString type = m_menu->GetObject(GetSelected())->GetFileName();
+		for (wxArrayString::const_iterator it = m_selectedIds.begin(); it != m_selectedIds.end(); it++) {
+			if (type != m_menu->GetObject(*it)->GetFileName()) {
+				enable = false;
+				break;
+			}
+		}
+	}
+	event.Enable(enable);
+}
+
 void MenuEditor::OnSafeTV(wxCommandEvent& event) {
 	m_safeTV = !m_safeTV;
 	ShowSafeTV();
