@@ -3,7 +3,7 @@
 // Purpose:     The widget to edit a DVD Menu
 // Author:      Alex Thuering
 // Created:	11.10.2003
-// RCS-ID:      $Id: MenuEditor.h,v 1.18 2010/08/04 19:05:31 ntalex Exp $
+// RCS-ID:      $Id: MenuEditor.h,v 1.21 2011/09/04 19:30:33 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -16,8 +16,9 @@
 class TitlesetManager;
 class MenuObject;
 
-enum TransformType
-{ ttOut, ttIn, ttTL, ttTR, ttBL, ttBR };
+enum EditorAction {
+	eaNONE, eaMOVE, eaRESIZE_TL, eaRESIZE_TR, eaRESIZE_BL, eaRESIZE_BR
+};
 
 class MenuEditor: public wxSVGCtrl {
 public:
@@ -36,9 +37,10 @@ public:
 	bool AddImage(wxString fname, int x = 0, int y = 0);
 	bool AddText(wxString text, int x = 0, int y = 0);
     bool AddObject(wxString fname, int x = 0, int y = 0);
-	bool AddButton(wxString fname, int x, int y, int pgci = -1, int tsi = -1, bool isMenu = false);
+	bool AddButton(wxString fname, int x, int y, int pgci = 0, int tsi = -2, bool isMenu = false);
 	
-	wxString GetSelected() { return m_selected; }
+	wxString GetSelected() { return m_selectedIds.size() > 0 ? m_selectedIds[0] : wxT(""); }
+	const wxArrayString& GetSelectedIds() { return m_selectedIds; }
     void Select(wxString id) { DoSelect(id, true, true); }
 	
 	wxString GetPointed() { return m_pointed; }
@@ -74,7 +76,7 @@ private:
     wxMenu* m_arrangeMenu;
     wxMenu* m_alignMenu;
 	wxPoint m_menuPos;
-	wxString m_selected;
+	wxArrayString m_selectedIds;
 	wxString m_pointed;
 	bool m_safeTV;
 	bool m_showGrid;
@@ -84,17 +86,21 @@ private:
     wxArrayString m_addMenuObjects;
     
     bool objectIsMoving;
-    TransformType m_tt;
+    EditorAction m_action;
+    wxString m_actionObject;
     int m_oldx, m_oldy;
-    bool m_copy;
-    TransformType GetTransformType(int x, int y);
-    void SetMouseCursor(TransformType tt);
+    bool m_doCopy;
+    bool m_copied;
+    EditorAction GetEditorAction(int x, int y);
+    void SetMouseCursor(EditorAction tt);
     bool MoveObject(int& x, int& y);
-    bool MoveObjectInt(MenuObject* object, int& x, int& y);
-	bool ResizeObject(int x, int y, TransformType transformType = ttBR);
-    bool ResizeObjectInt(int x, int y, TransformType transformType = ttBR);
+    bool MoveObjectInt(int& x, int& y);
+	bool ResizeObject(int x, int y, EditorAction action = eaRESIZE_BR);
+    bool ResizeObjectInt(int x, int y, EditorAction action = eaRESIZE_BR);
     
-    void DoSelect(wxString id = wxT(""), bool refresh = false, bool sendEvent = false);
+    void DoSelect(wxString id = wxT(""), bool refresh = false, bool sendEvent = false, bool ctrlDown = false);
+    wxRect CreateSelection(wxString id);
+    void RefreshSelected();
 	
 	void OnMouseLeftButton(wxMouseEvent &event);
     void OnMouseMove(wxMouseEvent &event);
@@ -119,6 +125,7 @@ private:
     void OnAlignToCenterH(wxCommandEvent& event);
     void OnAlignToCenterV(wxCommandEvent& event);
 	void OnProperties(wxCommandEvent& event);
+	void OnUpdateUIProperties(wxUpdateUIEvent& event);
 	void OnSafeTV(wxCommandEvent& event);
 	void OnShowGrid(wxCommandEvent& event);
 	void OnShowGridOverObjects(wxCommandEvent& event);
