@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu Object
 // Author:      Alex Thuering
 // Created:	04.11.2006
-// RCS-ID:      $Id: MenuObject.cpp,v 1.24 2010/09/05 10:54:46 ntalex Exp $
+// RCS-ID:      $Id: MenuObject.cpp,v 1.32 2011/11/23 22:02:01 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -15,16 +15,19 @@
 #include <wxSVGXML/svgxmlhelpr.h>
 #include <wx/mstream.h>
 #include <wx/filename.h>
+#include <wx/regex.h>
+#include <wx/tokenzr.h>
 
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
 #define OBJECTS_DIR wxFindDataDirectory(_T("objects"))
 
 const wxString TRANS_ELEM_ID = wxT("s_trans");
+wxRegEx s_jumpTitlesetMenu(wxT("jump titleset ([[:digit:]]+) menu;"));
 
 ///////////////////////////// MenuObject /////////////////////////////////////
 
-MenuObject::MenuObject(Menu* menu, bool vmg, wxString fileName, int x, int y, wxString param):
-		m_action(vmg, BUTTON_ACTION) {
+MenuObject::MenuObject(Menu* menu, bool vmg, wxString fileName, int x, int y, wxString param): m_action(vmg) {
+	m_menu = menu;
 	if (menu == NULL) {
 		m_deleteSVG = true;
 		m_svg = Menu::CreateSVG(400, 400);
@@ -36,7 +39,6 @@ MenuObject::MenuObject(Menu* menu, bool vmg, wxString fileName, int x, int y, wx
 	} else {
 		m_deleteSVG = false;
 		m_svg = menu->GetSVG();
-		m_menu = menu;
 	}
 	m_use = NULL;
 	m_symbol = NULL;
@@ -55,6 +57,8 @@ MenuObject::MenuObject(Menu* menu, bool vmg, wxString fileName, int x, int y, wx
 	m_minWidth.value = m_minHeight.value = 0;
 	m_minWidth.valueInc = m_minHeight.valueInc = 0;
 	m_minWidth.valuePercent = m_minHeight.valuePercent = 0;
+	m_keepAspectRatio = false;
+	m_aspectRatioElem = NULL;
 
 	if (fileName.length())
 		Init(fileName, x, y, param);
@@ -85,9 +89,14 @@ bool MenuObject::Init(wxString fileName, int x, int y, wxString param) {
 	if (m_id.length()) { // load button
 		m_symbol = (wxSVGSVGElement*) m_svg->GetElementById(wxT("s_") + m_id);
 		m_use = (wxSVGUseElement*) m_svg->GetElementById(m_id);
-	} else
+	} else {
 		// create new button
 		m_id = GenerateId(m_button ? wxT("button") : wxT("obj"));
+		wxSvgXmlNode* node = XmlFindNode(root, wxT("default-size"));
+		wxString val;
+		if (node && node->GetPropVal(wxT("keepAspectRatio"), &val) && (val == wxT("true") || val == wxT("1")))
+			m_keepAspectRatio = true;
+	}
 
 	wxSvgXmlNode* svgNode = XmlFindNode(root, wxT("svg"));
 	if (svgNode) {
@@ -101,6 +110,12 @@ bool MenuObject::Init(wxString fileName, int x, int y, wxString param) {
 				AddSymol(m_id, root);
 			if (!m_use)
 				AddUse(m_id, x, y, m_defaultWidth.value, m_defaultHeight.value);
+			wxSvgXmlNode* imageNode = XmlFindNode(svgNode, wxT("image"));
+			if (imageNode != NULL) {
+				wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(imageNode->GetAttribute(wxT("id")));
+				if (elem && elem->GetDtd() == wxSVG_IMAGE_ELEMENT)
+					m_aspectRatioElem = (wxSVGImageElement*) elem;
+			}
 		}
 	}
 
@@ -140,7 +155,7 @@ bool MenuObject::Init(wxString fileName, int x, int y, wxString param) {
 	
 	// load default size
 	wxSvgXmlNode* defaultSizeNode = XmlFindNode(root, wxT("default-size"));
-	if (defaultSizeNode) {
+	if (defaultSizeNode && defaultSizeNode->GetFirstChild()) {
 		wxString sizeElem = XmlReadValue(defaultSizeNode, wxT("element"));
 		if (sizeElem.length() > 0) {
 			m_defaultWidth.elements.Add(sizeElem);
@@ -222,6 +237,16 @@ MenuObjectParam* MenuObject::GetInitParam() {
 	return GetObjectParam(m_initParameter);
 }
 
+int MenuObject::GetChangebaleColourCount(bool drawButtonsOnBackground) {
+	int count = 0;
+	for (int i = 0; i < (int) m_params.Count(); i++) {
+		if (m_params[i]->isChangeable() && ((m_params[i]->normalColour.IsOk() && !drawButtonsOnBackground)
+				|| m_params[i]->highlightedColour.IsOk() || m_params[i]->selectedColour.IsOk()))
+			count++;
+	}
+	return count;
+}
+
 wxSVGSVGElement* MenuObject::AddSymol(wxString id, wxSVGElement* content) {
 	m_symbol = new wxSVGSVGElement;
 	m_symbol->SetId(wxT("s_") + id);
@@ -361,13 +386,29 @@ unsigned int MenuObject::CalcSize(MenuObjectSize& size, bool width) {
 	return wxMax(result, size.value);
 }
 
+#include <wxSVG/SVGCanvasItem.h>
+
 void MenuObject::FixSize(int& width, int& height) {
+	// calculate aspect ratio
+	double aspectRatio = -1;
+	if (m_keepAspectRatio && m_aspectRatioElem != NULL && m_aspectRatioElem->GetDefaultWidth() > 0)
+		aspectRatio = ((double) m_aspectRatioElem->GetDefaultWidth()) / m_aspectRatioElem->GetDefaultHeight();
+	
 	if (m_defaultSize) {
 		width = CalcSize(m_defaultWidth, true);
-		height = CalcSize(m_defaultHeight, false);
+		if (aspectRatio > 0)
+			height = width / aspectRatio;
+		else
+			height = CalcSize(m_defaultHeight, false);
 	} else {
 		width = wxMax(width, (int) CalcSize(m_minWidth, true));
-		height = wxMax(height, (int) CalcSize(m_minHeight, false));
+		if (m_keepAspectRatio) {
+			if (aspectRatio > 0)
+				height = width / aspectRatio;
+			else
+				height = width * CalcSize(m_defaultHeight, false) / CalcSize(m_defaultWidth, true);
+		} else
+			height = wxMax(height, (int) CalcSize(m_minHeight, false));
 	}
 }
 
@@ -404,8 +445,17 @@ wxString MenuObject::GetParam(wxString name, wxString attrSuffix) {
 		}
 		return elem->GetAttribute(param->attribute + attrSuffix);
 	} else if (elem->GetDtd() == wxSVG_TEXT_ELEMENT) {
-		// return content of text element
-		return elem->GetChildren() ? elem->GetChildren()->GetContent() : wxT("");
+		wxString value;
+		wxSvgXmlNode* child = elem->GetChildren();
+		while (child != NULL) {
+			if (child->GetType() == wxSVGXML_TEXT_NODE)
+				value += child->GetContent().Strip(wxString::both);
+			else if (child->GetType() == wxSVGXML_ELEMENT_NODE
+					&& ((wxSVGElement*) child)->GetDtd() == wxSVG_TBREAK_ELEMENT)
+				value += wxT("\n");
+			child = child->GetNextSibling();
+		}
+		return value;
 	}
 	return wxT("");
 }
@@ -437,10 +487,34 @@ void MenuObject::SetParam(wxString name, wxString value, wxString attrSuffix) {
 			elem->SetAttribute(param->attribute.BeforeFirst(wxT('#')), value);
 		} else
 			elem->SetAttribute(param->attribute + attrSuffix, value);
+		if (elem->GetDtd() == wxSVG_IMAGE_ELEMENT && param->attribute == wxT("xlink:href"))
+			((wxSVGImageElement*) elem)->SetCanvasItem(NULL);
 	} else if (elem->GetDtd() == wxSVG_TEXT_ELEMENT) {
-		if (!elem->GetChildren())
-			return;
-		elem->GetChildren()->SetContent(value);
+		wxStringTokenizer tokenizer(value, wxT("\r\n"), wxTOKEN_RET_EMPTY_ALL);
+		wxSvgXmlNode* child = elem->GetChildren();
+		while (tokenizer.HasMoreTokens()) {
+			wxString token = tokenizer.GetNextToken();
+			// insert text node (token)
+			if (child != NULL && child->GetType() == wxSVGXML_TEXT_NODE) {
+				child->SetContent(token);
+				child = child->GetNextSibling();
+			} else
+				elem->InsertChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString, token), child);
+			// insert t-break
+			if (tokenizer.HasMoreTokens()) {
+				if (child != NULL && child->GetType() == wxSVGXML_ELEMENT_NODE
+						&& ((wxSVGElement*) child)->GetDtd() != wxSVG_TBREAK_ELEMENT)
+					child = child->GetNextSibling();
+				else
+					elem->InsertChild(new wxSVGTBreakElement(), child);
+			}
+			((wxSVGTextElement*) elem)->SetXmlspace(wxT("preserve"));
+		}
+		while (child != NULL) {
+			wxSvgXmlNode* nextChild = child->GetNextSibling();
+			elem->RemoveChild(child);
+			child = nextChild;
+		}
 		((wxSVGTextElement*) elem)->SetCanvasItem(NULL);
 	}
 }
@@ -545,8 +619,7 @@ wxColour MenuObject::GetParamColour(wxString name, MenuButtonState state) {
 			return param->selectedColour;
 		}
 	}
-	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(
-			param->element);
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
 	if (!elem)
 		return wxT("");
 	if (param->attribute.length()) {
@@ -565,8 +638,7 @@ wxColour MenuObject::GetParamColour(wxString name, MenuButtonState state) {
 	return wxColour();
 }
 
-void MenuObject::SetParamColour(wxString name, wxColour value,
-		MenuButtonState state) {
+void MenuObject::SetParamColour(wxString name, wxColour value, MenuButtonState state) {
 	MenuObjectParam* param = GetObjectParam(name);
 	if (!param)
 		return;
@@ -585,8 +657,7 @@ void MenuObject::SetParamColour(wxString name, wxColour value,
 	}
 	if (state != mbsNORMAL)
 		return;
-	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(
-			param->element);
+	wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
 	if (!elem)
 		return;
 	if (param->attribute.length()) {
@@ -696,8 +767,10 @@ wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, SubStreamMode mode,
 			}
 		}
 		rootNode->AddProperty(wxT("id"), GetId());
-		if (IsDefaultSize())
+		if (m_defaultSize)
 			rootNode->AddProperty(wxT("defSize"), wxT("true"));
+		if (m_keepAspectRatio)
+			rootNode->AddProperty(wxT("keepAspectRatio"), wxT("true"));
 		if (!m_displayVideoFrame)
 			rootNode->AddProperty(wxT("displayVideoFrame"), wxT("false"));
 		if (m_customVideoFrame)
@@ -730,9 +803,19 @@ wxSvgXmlNode* MenuObject::GetXML(DVDFileType type, DVD* dvd, SubStreamMode mode,
 		action.Replace(wxT("vmMenu"), wxT("vmgm menu"));
 		if (dvd->GetPlayAllRegister() != -1)
 			action = wxString::Format(wxT("g%d=%d;"), dvd->GetPlayAllRegister(), GetAction().IsPlayAll()) + action;
-		if (dvd->GetRememberLastButtonRegister() != -1 && !GetAction().GetCustom())
-			action = wxString::Format(GetAction().IsMenu() ? wxT("g%d=0;") : wxT("g%d=button;"),
-					dvd->GetRememberLastButtonRegister()) + action;
+		if (dvd->GetLastMenuRegister() != -1) {
+			int rootMenu = ((Menus&) dvd->GetPgcArray(m_menu->GetTsi(), true)).GetPgciByEntry(wxT("root"));
+			if (action.Contains(wxT("jump menu;")) || action.Contains(wxT("jump menu entry root;"))
+					 || action.Contains(wxString::Format(wxT("jump menu %d;"), rootMenu >= 0 ? rootMenu + 1 : 1))
+					 || s_jumpTitlesetMenu.Matches(action))
+				action = wxString::Format(wxT("g%d=0;"), dvd->GetLastMenuRegister()) + action;
+		}
+		if (m_menu->GetRememberLastButton()) {
+			int reg = m_menu->GetRememberLastButtonRegister();
+			if (reg == -1)
+				reg = dvd->GetRememberLastButtonRegister();
+			action = wxString::Format(wxT("g%d=button;"), reg) + action;
+		}
 		rootNode->AddChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString, action));
 		break;
 	}
@@ -819,7 +902,8 @@ bool MenuObject::PutXML(wxSvgXmlNode* node) {
 
 	node->GetPropVal(wxT("id"), &m_id);
 	if (!node->GetPropVal(wxT("defSize"), &val) || (val != wxT("true") && val != wxT("1")))
-		SetDefaultSize(false);
+		m_defaultSize = false;
+	m_keepAspectRatio = node->GetPropVal(wxT("keepAspectRatio"), &val) && (val == wxT("true") || val == wxT("1"));
 	m_displayVideoFrame = !node->GetPropVal(wxT("displayVideoFrame"), &val) || (val != wxT("false") && val != wxT("0"));
 	m_customVideoFrame = node->GetPropVal(wxT("customVideoFrame"), &val) && (val == wxT("true") || val == wxT("1"));
 
