@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu Object
 // Author:      Alex Thuering
 // Created:	04.11.2006
-// RCS-ID:      $Id: MenuObject.h,v 1.17 2010/09/05 10:54:46 ntalex Exp $
+// RCS-ID:      $Id: MenuObject.h,v 1.20 2011/07/17 09:06:00 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -20,6 +20,7 @@ class wxSVGDocument;
 class wxSVGElement;
 class wxSVGUseElement;
 class wxSVGSVGElement;
+class wxSVGImageElement;
 class wxSVGRect;
 class Menu;
 class DVD;
@@ -43,6 +44,9 @@ struct MenuObjectParam {
   wxColour normalColour;
   wxColour highlightedColour;
   wxColour selectedColour;
+  inline bool isChangeable() {
+	  return changeable && (normalColour != highlightedColour || normalColour != selectedColour);
+  }
 };
 
 WX_DEFINE_ARRAY(MenuObjectParam*, MenuObjectParams);
@@ -84,6 +88,8 @@ public:
 	
 	inline bool IsDefaultSize() { return m_defaultSize; }
 	inline void SetDefaultSize(bool value = true) { m_defaultSize = value; }
+	inline bool IsKeepAspectRatio() { return m_keepAspectRatio; }
+	inline void SetKeepAspectRatio(bool value = true) { m_keepAspectRatio = value; }
     void FixSize(int& width, int& height);
     void UpdateSize();
     
@@ -91,6 +97,7 @@ public:
     MenuObjectParam* GetObjectParam(int index) { return m_params[index]; }
     MenuObjectParam* GetObjectParam(wxString name);
     MenuObjectParam* GetInitParam();
+    int GetChangebaleColourCount(bool drawButtonsOnBackground);
     
     wxString GetParam(wxString name, wxString attrSuffix = wxEmptyString);
 	void SetParam(wxString name, wxString value, wxString attrSuffix = wxEmptyString);
@@ -132,38 +139,41 @@ public:
 	wxString m_id;
 	bool m_button;
 	DVDAction m_action;
-    wxString m_fileName;
-    wxString m_title;
-    bool m_previewHighlighted;
-    
+	wxString m_fileName;
+	wxString m_title;
+	bool m_previewHighlighted;
+	
 	wxString m_direction[4]; // left, right, up, down button names
-    
-    MenuObjectParams m_params;
-    wxString m_initParameter;
-    bool m_displayVideoFrame; // sets if video frame must be displayed
-    bool m_customVideoFrame; // shows if custom video frame is selected
-    	
-    bool m_defaultSize;
-    MenuObjectSize m_defaultWidth;
-    MenuObjectSize m_defaultHeight;
-    
-    MenuObjectSize m_minWidth;
-    MenuObjectSize m_minHeight;
-    
-    bool m_deleteSVG;
-    wxSVGDocument* m_svg;
-    wxSVGUseElement* m_use;
-    wxSVGSVGElement* m_symbol;
-    wxSVGSVGElement* AddSymol(wxString id, wxSVGElement* content);
-    void AddUse(wxString id, int x, int y, int width, int height);
-    
-    void SetScale(double scaleX, double scaleY);
-    
-    bool Init(wxString filename, int x = 0, int y = 0, wxString param = wxT(""));
-    bool LoadSVG(wxSVGDocument& svg, wxSvgXmlNode* node);
-    wxString GenerateId(wxString prefix);
-    void InitSize(wxString value, MenuObjectSize& size);
-    unsigned int CalcSize(MenuObjectSize& size, bool width);
+	
+	MenuObjectParams m_params;
+	wxString m_initParameter;
+	bool m_displayVideoFrame; // sets if video frame must be displayed
+	bool m_customVideoFrame; // shows if custom video frame is selected
+	
+	bool m_keepAspectRatio;
+	wxSVGImageElement* m_aspectRatioElem;
+		
+	bool m_defaultSize;
+	MenuObjectSize m_defaultWidth;
+	MenuObjectSize m_defaultHeight;
+	
+	MenuObjectSize m_minWidth;
+	MenuObjectSize m_minHeight;
+	
+	bool m_deleteSVG;
+	wxSVGDocument* m_svg;
+	wxSVGUseElement* m_use;
+	wxSVGSVGElement* m_symbol;
+	wxSVGSVGElement* AddSymol(wxString id, wxSVGElement* content);
+	void AddUse(wxString id, int x, int y, int width, int height);
+	
+	void SetScale(double scaleX, double scaleY);
+	
+	bool Init(wxString filename, int x = 0, int y = 0, wxString param = wxT(""));
+	bool LoadSVG(wxSVGDocument& svg, wxSvgXmlNode* node);
+	wxString GenerateId(wxString prefix);
+	void InitSize(wxString value, MenuObjectSize& size);
+	unsigned int CalcSize(MenuObjectSize& size, bool width);
 };
 
 #endif // MENU_OBJECT_H
