@@ -3,7 +3,7 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.41 2010/09/13 20:22:05 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.49 2011/09/11 18:53:21 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -18,7 +18,7 @@ enum {
 	CUSTOM_ACTION_RADIO_ID,
 	TARGET_CHOICE_ID,
 	AUTO_SIZE_RADIO_ID,
-	CUSTOM_SIZE_RADIO_ID,
+	KEEP_ASPECT_CB_ID,
 	IMAGE_RADIO_ID,
 	VIDEOFRAME_RADIO_ID
 };
@@ -27,15 +27,16 @@ BEGIN_EVENT_TABLE(MenuObjectPropDlg, wxPropDlg)
 	EVT_RADIOBUTTON(JUMP_ACTION_RADIO_ID, MenuObjectPropDlg::OnJumpActionSelected)
 	EVT_RADIOBUTTON(CUSTOM_ACTION_RADIO_ID, MenuObjectPropDlg::OnCustomActionSelected)
 	EVT_CHOICE(TARGET_CHOICE_ID, MenuObjectPropDlg::OnChangeTarget)
-	EVT_RADIOBUTTON(AUTO_SIZE_RADIO_ID, MenuObjectPropDlg::OnDefaultSizeSelected)
-	EVT_RADIOBUTTON(CUSTOM_SIZE_RADIO_ID, MenuObjectPropDlg::OnCustomSizeSelected)
+	EVT_RADIOBUTTON(KEEP_ASPECT_CB_ID, MenuObjectPropDlg::OnKeepAspectRatio)
 	EVT_RADIOBUTTON(IMAGE_RADIO_ID, MenuObjectPropDlg::OnImageRadio)
 	EVT_RADIOBUTTON(VIDEOFRAME_RADIO_ID, MenuObjectPropDlg::OnImageRadio)
 END_EVENT_TABLE()
 
-MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, Menu* menu, DVD* dvd, int tsi, int pgci):
-		wxPropDlg(parent, _("Properties") + wxString(wxT(" - ")) + menu->GetObject(id)->GetId(true)) {
+MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, bool multObjects, Menu* menu, DVD* dvd, int tsi,
+		int pgci): wxPropDlg(parent, wxString(_("Properties"))
+				+ (!multObjects ? wxT(" - ") + menu->GetObject(id)->GetId(true) : wxT(""))) {
 	m_id = id;
+	m_multObjects = multObjects;
 	m_menu = menu;
 	m_dvd = dvd;
 	m_tsi = tsi;
@@ -44,6 +45,9 @@ MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent, wxString id, Menu* menu,
 	m_imageRadio = NULL;
 	m_videoFrameRadio = NULL;
 	m_videoFrameChoice = NULL;
+	m_customActionEdit = NULL;
+	m_widthEdit = NULL;
+	m_heightEdit = NULL;
 	m_object = menu->GetObject(id);
 	m_displayVideoFrame = m_object->IsDisplayVideoFrame();
 	Create();
@@ -54,7 +58,7 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 	wxFlexGridSizer* grid = NULL;
 	
 	// action
-	if (m_object->IsButton()) {
+	if (m_object->IsButton() && !m_multObjects) {
 		DVDAction& action = m_object->GetAction();
 		AddStaticLine(mainSizer, _("Action"));
 		grid = new wxFlexGridSizer(2, 2, 4, 16);
@@ -122,7 +126,8 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 			labels.Add(_("track") + wxString::Format(wxT(" %d"), i + 1));
 		AddText(s, _("subtitle") + wxString(wxT(":")));
 		s->Add(16, 16);
-		AddChoiceProp(s, wxT(""), labels[action.GetSubtitle()+1], labels, 0, true);
+		int subtitleIdx = action.GetSubtitle() + 1 < (int) labels.size() ? action.GetSubtitle() + 1 : 0;
+		AddChoiceProp(s, wxT(""), labels[subtitleIdx], labels, 0, true);
 		if (m_dvd->GetSubtitleStreamsCount() == 0)
 			GetLastControl()->Enable(false);
 		
@@ -140,12 +145,11 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		grid->Add(10,10);
 		AddDirectionProp(grid, bdDOWN);
 		grid->Add(10,10);
-		mainSizer->Add(grid, 0, wxEXPAND|wxALL, 10);
+		mainSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
+		AddStaticLine(mainSizer, _("Look"));
 	}
 	
 	// object look parameters
-	if (m_object->GetObjectParamsCount()>0)
-	AddStaticLine(mainSizer, _("Look"));
 	bool lastChangeable = false;
 	for (int i=0; i<m_object->GetObjectParamsCount(); i++) {
 		MenuObjectParam* param = m_object->GetObjectParam(i);
@@ -162,9 +166,9 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 				grid->AddGrowableCol(1);
 			}
 			if (i==0)
-				mainSizer->Add(grid, 0, wxEXPAND|wxTOP|wxBOTTOM, 8);
+				mainSizer->Add(grid, 0, wxEXPAND|wxTOP|wxBOTTOM, 4);
 			else
-				mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
+				mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 4);
 			lastChangeable = param->changeable;
 		}
 		
@@ -178,22 +182,27 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 			if (param->type == _T("text")) {
 				AddText(grid, title);
 				wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
-				AddTextProp(sizer2, wxT(""), m_object->GetParam(param->name));
+				AddTextProp(sizer2, wxT(""), m_object->GetParam(param->name), m_multObjects, -1, true, wxTE_MULTILINE);
+				if (m_multObjects)
+					GetLastControl()->Enable(false);
 				AddFontProp(sizer2, wxT(""), m_object->GetParamFont(param->name), _("Font..."));
 				grid->Add(sizer2, 0, wxEXPAND);
 			} else if (param->type == _T("string")) {
 				AddTextProp(grid, title, m_object->GetParam(param->name));
 			} else if (param->type == _T("decimal")) {
 				AddTextProp(grid, title, m_object->GetParam(param->name), false, 100);
+				if (m_multObjects && param->attribute.Find(wxT('#')) != wxNOT_FOUND)
+					GetLastControl()->Enable(false);
 			} else if (param->type.Mid(0,7) == _T("integer")) {
 				long min = 0, max = 999;
 				if (param->type.Find(wxT('(')) != -1 && param->type.Find(wxT(')')) != -1) {
 					param->type.AfterFirst(wxT('(')).BeforeFirst(wxT(',')).Strip(wxString::both).ToLong(&min);
 					param->type.BeforeFirst(wxT(')')).AfterFirst(wxT(',')).Strip(wxString::both).ToLong(&max);
 				}
-				AddSpinProp(grid, title, m_object->GetParamInt(param->name), min, max,
-						max < 1000 ? 54 : 80);
-			} else if (param->type == _T("image")) {
+				AddSpinProp(grid, title, m_object->GetParamInt(param->name), min, max, max < 1000 ? 54 : 80);
+				if (m_multObjects && param->attribute.Find(wxT('#')) != wxNOT_FOUND)
+					GetLastControl()->Enable(false);
+			} else if (param->type == _T("image") && !m_multObjects) {
 				wxString imageFile = m_object->GetParam(param->name);
 				if (m_object->IsButton()) {
 					bool hasVideo = GetVideoFilename().length();
@@ -224,10 +233,13 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 					}
 					AddSpinProp(frameSizer, wxT(""), videoFrame, 0, 99999999, 100, _("ms"), false);
 					SetLastControlCustom(GetLastControlIndex()-1, !vfAuto);
-				} else
+				} else {
 					AddFileProp(grid, title, imageFile, wxFD_OPEN,
 						wxT("..."), _("Image Files ") + wxImage::GetImageExtWildcard()
 						+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
+					if (m_multObjects)
+						GetLastControl()->Enable(false);
+				}
 			} else if (param->type == _T("colour")) {
 				int opacity = 100;
 				wxString opacityStr = m_object->GetParam(param->name, wxT("-opacity"));
@@ -241,34 +253,36 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer) {
 		}
 	}
 	
+	if (!m_multObjects)
+		CreateLocAndSize(mainSizer);
+	
+	sizer->Add(mainSizer, 1, wxEXPAND|wxALL, 4);
+}
+
+void MenuObjectPropDlg::CreateLocAndSize(wxBoxSizer* mainSizer) {
+	wxSizer* hSizer = new wxBoxSizer(wxHORIZONTAL);
+	mainSizer->Add(hSizer, 0, wxTOP, 8);
+		
 	// object location
-	AddStaticLine(mainSizer, _("Location and size"));
-	grid = new wxFlexGridSizer(1, 4, 4, 16);
-	AddSpinProp(grid, _("x:"), m_object->GetX(), 0, m_menu->GetResolution().GetWidth()-8);
-	AddSpinProp(grid, _("y:"), m_object->GetY(), 0, m_menu->GetResolution().GetHeight()-8);
-	mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
+	wxSizer* grpSizer = BeginGroup(hSizer, _("Location"));
+	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 4, 16);
+	AddSpinProp(grid, _("X:"), m_object->GetX(), 0, m_menu->GetResolution().GetWidth()-8);
+	AddSpinProp(grid, _("Y:"), m_object->GetY(), 0, m_menu->GetResolution().GetHeight()-8);
+	grpSizer->Add(grid, 0, wxALL, 4);
+	EndGroup();
+	hSizer->AddSpacer(8);
 	
 	// object size
-	grid = new wxFlexGridSizer(2, 6, 4, 16);
-	grid->AddGrowableCol(5);
-	AddRadioProp(grid, _("Default width and height"), m_object->IsDefaultSize(), wxRB_GROUP, false,
-			AUTO_SIZE_RADIO_ID);
-	grid->Add(10,10); grid->Add(10,10); grid->Add(10,10); grid->Add(10,10); grid->Add(10,10);
-	AddRadioProp(grid, _("Custom"), !m_object->IsDefaultSize(), 0, false, CUSTOM_SIZE_RADIO_ID);
-	AddSpinProp(grid, _("width:"), m_object->GetWidth(), 0, m_menu->GetResolution().GetWidth());
+	grpSizer = BeginGroup(hSizer, _("Size"), _("Custom"), !m_object->IsDefaultSize());
+	grid = new wxFlexGridSizer(2, 4, 16);
+	AddSpinProp(grid, _("Width:"), m_object->GetWidth(), 0, m_menu->GetResolution().GetWidth());
 	m_widthEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
-	AddSpinProp(grid, _("height:"), m_object->GetHeight(), 0, m_menu->GetResolution().GetHeight());
+	AddSpinProp(grid, _("Height:"), m_object->GetHeight(), 0, m_menu->GetResolution().GetHeight());
 	m_heightEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
-	grid->Add(10,10);
-	mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
-	wxCommandEvent evt;
-	if (m_object->IsDefaultSize())
-		OnDefaultSizeSelected(evt);
-	else
-		OnCustomSizeSelected(evt);
-	
-	mainSizer->Add(10, 10, 1, wxEXPAND);
-	sizer->Add(mainSizer, 1, wxEXPAND|wxALL, 10);
+	grid->AddSpacer(4);
+	AddCheckProp(grid, _("Keep Aspect Ratio"), m_object->IsKeepAspectRatio(), false, KEEP_ASPECT_CB_ID);
+	grpSizer->Add(grid, 0, wxALL, 4);
+	EndGroup();
 }
 
 int MenuObjectPropDlg::GetSelectedTsi() {
@@ -400,7 +414,7 @@ void MenuObjectPropDlg::OnChangeTarget(wxCommandEvent& evt) {
 			m_videoFrameRadio->SetValue(true);
 			wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_imageRadio->GetId());
 			GetEventHandler()->ProcessEvent(event);
-		} 
+		}
 	}
 }
 
@@ -432,28 +446,35 @@ void MenuObjectPropDlg::OnCustomActionSelected(wxCommandEvent& evt) {
 		m_customActionEdit->SetValue(m_object->GetAction().AsString(m_dvd));
 		m_object->GetAction().SetTsi(tsi);
 	}
-	if (m_videoFrameRadio) {
-		m_videoFrameRadio->Enable(false);
-		m_imageRadio->SetValue(true);
-		wxCommandEvent event(wxEVT_COMMAND_RADIOBUTTON_SELECTED, m_imageRadio->GetId());
-		GetEventHandler()->ProcessEvent(event);
-	}
 }
 
 wxString MenuObjectPropDlg::GetVideoFilename(bool withTimestamp, long position) {
-	if (!GetBool(0)) // custom action
+	int tsi = -1;
+	int pgci = 0;
+	int chapter = 0;
+	if (!GetBool(0)) { // custom action
+		DVDAction action(m_object->GetAction());
+		action.SetCustom(m_customActionEdit->GetValue());
+		int id = action.FindFirstJump(m_tsi);
+		if (id >= 0) {
+			tsi = DVD::GetTsi(id);
+			pgci = DVD::GetPgci(id);
+			chapter = DVD::GetVobi(id);
+		}
+	} else if (!IsSelectedMenu()) {
+		tsi = GetSelectedTsi();
+		pgci = GetSelectedPgci();
+		chapter = GetSelectedChapter();
+	}
+	
+	if (tsi < 0 || tsi >= (int) m_dvd->GetTitlesets().size())
 		return wxT("");
-	if (IsSelectedMenu()) // menu
+	Pgc* pgc = m_dvd->GetPgc(tsi, false, pgci);
+	if (!pgc)
 		return wxT("");
-	Vob* vob = m_dvd->GetVob(GetSelectedTsi(), false, GetSelectedPgci(), 0);
-	if (!vob)
-		return wxT("");
-	if (!withTimestamp)
-		return vob->GetVideoFrameUri(-1);
-	return vob->GetVideoFrameUri(GetSelectedChapter(), position);
+	return withTimestamp ? pgc->GetVideoFrameUri(chapter, position) : pgc->GetVideoFrameUri(-1);
 }
 
-
 void MenuObjectPropDlg::OnImageRadio(wxCommandEvent& evt){
 	if (m_videoFrameRadio->IsEnabled())
 		m_displayVideoFrame = m_videoFrameRadio->GetValue();
@@ -464,14 +485,8 @@ void MenuObjectPropDlg::OnImageRadio(wxCommandEvent& evt){
 	GetEventHandler()->ProcessEvent(event);
 }
 
-void MenuObjectPropDlg::OnDefaultSizeSelected(wxCommandEvent& evt) {
-	m_widthEdit->Enable(false);
-	m_heightEdit->Enable(false);
-}
-
-void MenuObjectPropDlg::OnCustomSizeSelected(wxCommandEvent& evt) {
-	m_widthEdit->Enable(true);
-	m_heightEdit->Enable(true);
+void MenuObjectPropDlg::OnKeepAspectRatio(wxCommandEvent& evt) {
+	m_heightEdit->SetValue(m_widthEdit->GetValue());
 }
 
 void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir) {
@@ -510,9 +525,8 @@ void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir) {
 
 bool MenuObjectPropDlg::SetValues() {
 	// check Action
-	if (m_object->IsButton()) {
-		if (GetBool(4) && GetString(5).length() == 0) // empty custom action
-		{
+	if (m_object->IsButton() && !m_multObjects) {
+		if (GetBool(4) && GetString(5).length() == 0) { // empty custom action
 			wxMessageBox(_("Please enter the action command."),
 			GetTitle(), wxOK|wxICON_ERROR, this);
 			return false;
@@ -529,7 +543,7 @@ bool MenuObjectPropDlg::SetValues() {
 	int n = 0;
 	
 	// set action
-	if (m_object->IsButton()) {
+	if (m_object->IsButton() && !m_multObjects) {
 		DVDAction& action = m_object->GetAction();
 		action.SetTsi(GetSelectedTsi() != m_tsi ? GetSelectedTsi() : -2);
 		action.SetPgci(GetSelectedTsi() != m_tsi || GetSelectedPgci() != m_pgci || !IsSelectedMenu() ? GetSelectedPgci() : -1);
@@ -590,7 +604,7 @@ bool MenuObjectPropDlg::SetValues() {
 				m_object->SetParamColour(param->name, GetColour(n++));
 				wxString opacity = wxString::Format(wxT("%g"), (double) GetInt(n++) / 100);
 				m_object->SetParam(param->name, opacity, wxT("-opacity"));
-			} else if (param->type == _T("image")) {
+			} else if (param->type == _T("image") && !m_multObjects) {
 				if (m_object->IsButton()) {
 					m_object->SetDisplayVideoFrame(m_displayVideoFrame);
 					bool image = GetBool(n++);
@@ -613,17 +627,18 @@ bool MenuObjectPropDlg::SetValues() {
 		}
 	}
 	
-	// set position
-	m_object->SetX(GetInt(n++));
-	m_object->SetY(GetInt(n++));
-	
-	// set size
-	m_object->SetDefaultSize(GetBool(n++));
-	if (!m_object->IsDefaultSize()) {
-		// set size manual by user
-		n++;
-		m_object->SetWidth(GetInt(n++));
-		m_object->SetHeight(GetInt(n++));
+	if (!m_multObjects) {
+		// set position
+		m_object->SetX(GetInt(n++));
+		m_object->SetY(GetInt(n++));
+		
+		// set size
+		m_object->SetDefaultSize(!GetBool(n++));
+		if (!m_object->IsDefaultSize()) {
+			m_object->SetWidth(GetInt(n++));
+			m_object->SetHeight(GetInt(n++));
+			m_object->SetKeepAspectRatio(GetBool(n++));
+		}
 	}
 	m_object->UpdateSize();
 	
