@@ -3,7 +3,7 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.h,v 1.14 2010/08/15 17:48:28 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.h,v 1.16 2011/09/04 19:30:33 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -21,17 +21,19 @@ enum ButtonDirCommand {
 
 class MenuObjectPropDlg: public wxPropDlg {
   public:
-    MenuObjectPropDlg(wxWindow *parent, wxString id, Menu *menu, DVD* dvd, int tsi, int pgci);
+    MenuObjectPropDlg(wxWindow *parent, wxString id, bool multObjects, Menu *menu, DVD* dvd, int tsi, int pgci);
     virtual ~MenuObjectPropDlg() {}
 
   private:
     wxString m_id;
+    bool m_multObjects;
     MenuObject* m_object;
     Menu* m_menu;
     DVD* m_dvd;
     int m_tsi; // tsi of menu
     int m_pgci; // pgci of menu
     void CreatePropPanel(wxSizer* sizer);
+    void CreateLocAndSize(wxBoxSizer* mainSizer);
     
     wxChoice* m_targetChoice;
     wxChoice* m_chapterChoice;
@@ -63,8 +65,7 @@ class MenuObjectPropDlg: public wxPropDlg {
     
     wxTextCtrl* m_widthEdit;
     wxTextCtrl* m_heightEdit;
-    void OnDefaultSizeSelected(wxCommandEvent& evt);
-    void OnCustomSizeSelected(wxCommandEvent& evt);
+    void OnKeepAspectRatio(wxCommandEvent& evt);
     
     void AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir);
     bool SetValues();
