@@ -3,14 +3,13 @@
 // Purpose:     DVD menu properties dialog
 // Author:      Alex Thuering
 // Created:     25.04.2004
-// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.35 2011/02/13 19:12:14 ntalex Exp $
+// RCS-ID:      $Id: MenuPropDlg.cpp,v 1.39 2011/08/14 08:43:49 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "MenuPropDlg.h"
 #include "TitlesetManager.h"
-#include "AVPropDlg.h"
 #include "Config.h"
 #include <wxVillaLib/utils.h>
 #include <wx/file.h>
@@ -82,9 +81,37 @@ void MenuPropDlg::CreateBackgroundGroup(wxSizer* sizer) {
 	wxSizer* grpSizer = BeginGroup(sizer, _("Background"));
 	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 2, 4, 16);
 	grid->AddGrowableCol(1);
+	// Image
 	AddFileProp(grid, _("Image:"), m_menu->GetBackground(), wxFD_OPEN, wxT("..."),
 			_("Image Files ") + wxImage::GetImageExtWildcard()
 			+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
+	// Keep aspect ratio
+	grid->AddSpacer(8);
+	wxSizer* cbSizer = new wxBoxSizer(wxHORIZONTAL);
+	bool keepAspectRatio = m_menu->GetBackgroundAlign() > wxSVG_PRESERVEASPECTRATIO_NONE;
+	BeginCheckGroup(cbSizer, _("Keep Aspect Ratio"), keepAspectRatio);
+	cbSizer->AddSpacer(4);
+	wxArrayString alignXStrings;
+	alignXStrings.Add(_("X Min"));
+	alignXStrings.Add(_("X Mid"));
+	alignXStrings.Add(_("X Max"));
+	int alignX = keepAspectRatio ? (m_menu->GetBackgroundAlign() - wxSVG_PRESERVEASPECTRATIO_XMINYMIN) % 3 : 1;
+	cbSizer->Add(AddChoiceProp(alignXStrings[alignX], alignXStrings));
+	wxArrayString alignYStrings;
+	alignYStrings.Add(_("Y Min"));
+	alignYStrings.Add(_("Y Mid"));
+	alignYStrings.Add(_("Y Max"));
+	int alignY = keepAspectRatio ? (m_menu->GetBackgroundAlign() - wxSVG_PRESERVEASPECTRATIO_XMINYMIN) / 3 : 1;
+	cbSizer->Add(AddChoiceProp(alignYStrings[alignY], alignYStrings));
+	wxArrayString meetStrings;
+	meetStrings.Add(_("Meet"));
+	meetStrings.Add(_("Slice"));
+	int meet = m_menu->GetBackgroundMeetOrSlice() == wxSVG_MEETORSLICE_SLICE ? 1 : 0;
+	cbSizer->Add(AddChoiceProp(meetStrings[meet], meetStrings));
+	cbSizer->AddStretchSpacer(1);
+	grid->Add(cbSizer, 0, wxEXPAND|wxALL, 4);
+	EndGroup();
+	// Colour
 	AddColourProp(grid, _("Colour:"), m_menu->GetBackgroundColour());
 	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
 	sizer->AddSpacer(6);
@@ -94,13 +121,23 @@ void MenuPropDlg::CreateButtonsGroup(wxSizer* sizer) {
 	wxSizer* grpSizer = BeginGroup(sizer, _("Buttons"));
 	wxFlexGridSizer* grid = new wxFlexGridSizer(2, 2, 4, 16);
 	grid->AddGrowableCol(1);
+	// Start/End
 	wxSizer* timeSizer = AddTextProp(grid, _("Start/End:"), m_menu->GetStartTime(), false, 100, false);
 	AddTextProp(timeSizer, wxT(" - "), m_menu->GetEndTime(), false, 100);
 	grpSizer->Add(grid, 0, wxEXPAND|wxALL, 4);
+	// Remember last selected button
 	wxSizer* cbSizer = new wxBoxSizer(wxHORIZONTAL);
-	AddCheckProp(cbSizer, _("Remember last selected button"), m_menu->GetRememberLastButton());
+	BeginCheckGroup(cbSizer, _("Remember last selected button"), m_menu->GetRememberLastButton());
+	cbSizer->AddSpacer(4);
+	wxArrayString registers;
+	registers.Add(_("Auto"));
+	for (int i=0; i<13; i++)
+		registers.Add(wxString::Format(wxT("g%d"), i));
+	cbSizer->Add(AddChoiceProp(registers[m_menu->GetRememberLastButtonRegister() + 1], registers));
+	cbSizer->AddStretchSpacer(1);
 	grpSizer->Add(cbSizer, 0, wxEXPAND|wxALL, 4);
 	sizer->AddSpacer(6);
+	EndGroup();
 }
 
 void MenuPropDlg::CreateVobGroup(wxSizer* sizer, bool loop) {
@@ -182,7 +219,8 @@ wxArrayString MenuPropDlg::GetCommands() {
 
 bool MenuPropDlg::SetValues() {
 	// validate audio file
-	wxString audioFile = GetString(8);
+	int AUDIO_CTRL_IDX = 13;
+	wxString audioFile = GetString(AUDIO_CTRL_IDX);
 	if (audioFile.length() > 0) {
 		if (!wxFile::Exists(audioFile)) {
 			wxLogError(_("File %s doesn't exist."), audioFile.c_str());
@@ -198,10 +236,10 @@ bool MenuPropDlg::SetValues() {
 	}
 	
 	// validate commands
-	int n = m_tsi >= 0 ? 18 : 14;
+	int n = AUDIO_CTRL_IDX + (m_tsi >= 0 ? 9 : 5);
 	wxString preCommands = GetString(n++);
 	wxString postCommands = GetString(n++);
-	DVDAction action(false, BUTTON_ACTION);
+	DVDAction action;
 	action.SetCustom(preCommands);
 	if (preCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, true, wxT(""), true, false)) {
 		return false;
@@ -227,13 +265,26 @@ bool MenuPropDlg::SetValues() {
 	m_video->SetWidescreen((WidescreenType) GetInt(n++));
 	
 	m_menu->SetBackground(GetString(n++));
-	if (!GetString(n - 1).length())
-		m_menu->SetBackgroundColour(GetColour(n));
-	n++;
+	if (m_menu->GetBackground().length()) {
+		if (GetBool(n++)) {
+			int alignX = GetInt(n++);
+			int alignY = GetInt(n++);
+			m_menu->SetBackgroundAlign((wxSVG_PRESERVEASPECTRATIO) (wxSVG_PRESERVEASPECTRATIO_XMINYMIN + alignY * 3 + alignX));
+			int meet = GetInt(n++);
+			m_menu->SetBackgroundMeetOrSlice(meet == 0 ? wxSVG_MEETORSLICE_MEET : wxSVG_MEETORSLICE_SLICE);
+		} else {
+			n += 3;
+			m_menu->SetBackgroundAlign(wxSVG_PRESERVEASPECTRATIO_NONE);
+		}
+	} else
+		n += 4;
+	m_menu->SetBackgroundColour(GetColour(n++));
 	
 	m_menu->SetStartTime(GetString(n++));
 	m_menu->SetEndTime(GetString(n++));
 	m_menu->SetRememberLastButton(GetBool(n++));
+	m_menu->SetRememberLastButtonRegister(m_menu->GetRememberLastButton() ? GetInt(n) - 1 : -1);
+	n++;
 	
 	// VOB properties
 	audioFile = GetString(n++).Strip(wxString::both);
@@ -254,7 +305,8 @@ bool MenuPropDlg::SetValues() {
 	n++;
 	
 	// set entries
-	StringSet entries;
+	StringSet& entries = m_pgc->GetEntries();
+	entries.clear();
 	if (m_tsi >= 0) {
 		if (GetBool(n++))
 			entries.insert(wxT("root"));
@@ -268,7 +320,6 @@ bool MenuPropDlg::SetValues() {
 			entries.insert(wxT("audio"));
 	} else if (GetBool(n++))
 		entries.insert(wxT("title"));
-	m_pgc->SetEntries(entries);
 	// remove same entries from other menus in the same titleset
 	PgcArray& pgcs = m_dvd->GetPgcArray(m_tsi, true);
 	for (int pgci = 0; pgci<(int)pgcs.size(); pgci++) {
