@@ -3,7 +3,7 @@
 // Purpose:     DVD menu properties dialog
 // Author:      Alex Thuering
 // Created:     25.04.2004
-// RCS-ID:      $Id: MenuPropDlg.h,v 1.9 2010/10/10 18:39:13 ntalex Exp $
+// RCS-ID:      $Id: MenuPropDlg.h,v 1.10 2011/02/13 19:12:14 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -36,9 +36,17 @@ private:
     Pgc* m_pgc;
 	Vob* m_vob;
 	Menu* m_menu;
+	wxChoice* m_aspect;
+	wxChoice* m_wsFormat;
+	wxChoice* m_audioFormat;
 	wxSpinCtrl* m_pauseCtrl;
 	wxComboBox* m_postCommandsCtrl;
-	void OnAV(wxCommandEvent& event);
+	void CreateBackgroundGroup(wxSizer* sizer);
+	void CreateButtonsGroup(wxSizer* sizer);
+	void CreateVobGroup(wxSizer* sizer, bool loop);
+	void CreateMenuGroup(wxSizer* sizer, bool loop);
+	void OnChangeAspect(wxCommandEvent& event);
+	void OnChangeAudio(wxCommandEvent& event);
 	void OnLoopCheck(wxCommandEvent& event);
 	wxArrayString GetCommands();
     DECLARE_EVENT_TABLE()
