@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.cpp,v 1.107 2011/03/10 20:07:37 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.cpp,v 1.122 2011/11/24 07:30:37 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -65,15 +65,63 @@ protected:
 	ProgressDlg* m_processDlg;
 };
 
+class SpumuxExecute: public wxPipeExecute {
+public:
+	SpumuxExecute(ProgressDlg* processDlg, double fileSize): m_processDlg(processDlg) {
+		m_fileSize = fileSize;
+		m_error = false;
+		m_progressRegEx.Compile(wxT("^INFO: ([0-9]+) bytes"));
+		m_initSubStep = m_processDlg->GetSubStep();
+		m_percent = 0;
+	}
+	virtual ~SpumuxExecute() {};
+
+	bool Execute(wxString command, wxString inputFile = wxEmptyString, wxString outputFile = wxEmptyString) {
+		m_processDlg->AddDetailMsg(_("Executing command: ") + command);
+		bool result = wxPipeExecute::Execute(command, inputFile, outputFile);
+		m_processDlg->SetSubStep(m_initSubStep + 50);
+		return result && !m_error;
+	}
+	
+	virtual void ProcessOutput(wxString line) {
+		if (line.StartsWith(wxT("ERR:"))) {
+			m_processDlg->AddDetailMsg(line, *wxRED);
+			m_error = m_error || line.StartsWith(wxT("ERR:  Cannot pick button masks"));
+		} else if (m_progressRegEx.Matches(line)) {
+			if (m_fileSize == -1)
+				return;
+			double bytes = 0;
+			m_progressRegEx.GetMatch(line, 1).ToDouble(&bytes);
+			int percent = (bytes * 100) / m_fileSize;
+			m_processDlg->SetSubStep(m_initSubStep + (int) percent/2);
+			if (percent >= m_percent) {
+				m_percent += 20;
+				m_processDlg->AddDetailText(line + _T("\n"));
+			}
+		} else if (!line.StartsWith(wxT("STAT:")))
+			m_processDlg->AddDetailText(line + _T("\n"));
+	}
+
+	virtual bool IsCanceled() {
+		return m_processDlg->IsCanceled();
+	}
+
+protected:
+	ProgressDlg* m_processDlg;
+	double m_fileSize;
+	bool m_error;
+	wxRegEx m_progressRegEx;
+	int m_initSubStep;
+	int m_percent;
+};
+
 class ProgressExecute: public ProcessExecute {
 public:
-	ProgressExecute(ProgressDlg* processDlg, wxString filter):
-	  ProcessExecute(processDlg), m_percent(0)
-	{
-      m_percentPattern.Compile(wxT("(([0-9]+[\\.,][0-9]+)|([0-9]+))%"),wxRE_ICASE);
-      m_blockPattern.Compile(wxT("([0-9]+)[[:space:]]+of[[:space:]]+([0-9]+)"),wxRE_ICASE);
-      m_filterPattern.Compile(filter,wxRE_ICASE);
-	  m_initSubStep = m_processDlg->GetSubStep();
+	ProgressExecute(ProgressDlg* processDlg, wxString filter): ProcessExecute(processDlg), m_percent(0) {
+		m_percentPattern.Compile(wxT("(([0-9]+[\\.,][0-9]+)|([0-9]+))%"),wxRE_ICASE);
+		m_blockPattern.Compile(wxT("([0-9]+)[[:space:]]+of[[:space:]]+([0-9]+)"),wxRE_ICASE);
+		m_filterPattern.Compile(filter,wxRE_ICASE);
+		m_initSubStep = m_processDlg->GetSubStep();
 	}
 	virtual ~ProgressExecute() {};
 
@@ -411,19 +459,22 @@ int ProgressDlg::GetMenuSubSteps(wxArrayPtrVoid& menuVobs, wxArrayInt& menuWSTyp
 			Pgc* pgc = pgcs[pgci];
 			if (pgc->GetVobs().Count() == 0)
 				continue;
+			// set temp file name
 			wxString menuFile = dvdTmpDir + wxString::Format(_T("menu%d-%d.mpg"), tsi + 1, pgci);
 			Vob* vob = pgc->GetVobs()[0];
 			vob->SetTmpFilename(menuFile);
-			menuSubSteps += 10; // perpare
-			if (vob->GetMenu()->HasVideoBackground()) {
-				menuSubSteps += 200; // transcode
+			// calculate sub steps
+			Menu* menu = vob->GetMenu();
+			WidescreenType widescreenType = pgcs.GetVideo().GetWidescreen();
+			if (menu->HasVideoBackground()) {
+				menuSubSteps += 400; // generate mpeg(200) + transcode(200)
 			} else if (vob->GetAudioFilenames().size() > 0)
 				menuSubSteps += 300; // generate mpeg(25+75) + transcode(200)
 			else
 				menuSubSteps += 25; // generate mpeg
-			menuSubSteps += 50; // spumux
+			menuSubSteps += (menu->GetAspectRatio() == ar4_3 ? 1 : widescreenType != wtAUTO ? 2 : 3)*50; // spumux
 			menuVobs.Add(vob);
-			menuWSTypes.Add((int)pgcs.GetVideo().GetWidescreen());
+			menuWSTypes.Add((int) widescreenType);
 		}
 	}
 	return menuSubSteps;
@@ -465,7 +516,7 @@ int ProgressDlg::GetSlideshowSubSteps(wxArrayPtrVoid& slideshowVobs, DVD* dvd, w
 			for (int vobi = 0; vobi < (int) pgc->GetVobs().Count(); vobi++) {
 				Vob* vob = pgc->GetVobs()[vobi];
 				if (vob->GetSlideshow()) {
-					slideshowSubSteps += 10 * vob->GetSlideshow()->Count();
+					slideshowSubSteps += 10 * vob->GetSlideshow()->GetCount();
 					if (vob->GetAudioFilenames().size() > 0)
 						slideshowSubSteps += 200; // transcode
 					vob->SetTmpFilename(dvdTmpDir
@@ -488,7 +539,7 @@ int ProgressDlg::GetSubtitleSubSteps(wxArrayPtrVoid& subtitleVobs, wxArrayInt& s
 			for (int vobi = 0; vobi < (int) pgc->GetVobs().Count(); vobi++) {
 				Vob* vob = pgc->GetVobs()[vobi];
 				if (vob->GetSubtitles().Count()) {
-					subtitleSubSteps += vob->GetSize(dvd) * vob->GetSubtitles().Count();
+					subtitleSubSteps += vob->GetSubtitles().Count() * 50;
 					vob->SetTmpFilename(dvdTmpDir
 							+ wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
 					subtitleVobs.Add(vob);
@@ -581,8 +632,8 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 		return;
 
 	// clear temp directory
-	if ((burnDlg->DoCreateIso() || burnDlg->DoBurn()) && s_config.GetRemoveTempFiles())
-		DeleteTempFiles(tmpDir, dvdTmpDir, dvdOutDir);
+	if (s_config.GetRemoveTempFiles())
+		DeleteTempFiles(tmpDir, dvdTmpDir, dvdOutDir, burnDlg->DoCreateIso() || burnDlg->DoBurn());
 
 	if (burnDlg->DoBurn())
 		AddSummaryMsg(_("Burning was successful."), wxEmptyString, wxColour(0, 128, 0));
@@ -595,8 +646,10 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd) {
 }
 
 bool ProgressDlg::CleanTemp(const wxString& tmpDir, const wxString& dvdTmpDir, const wxString& dvdOutDir) {
-	if (wxDir::Exists(tmpDir) && !DeleteTempFiles(tmpDir, dvdTmpDir, dvdOutDir))
+	if (wxDir::Exists(tmpDir) && !DeleteTempFiles(tmpDir, dvdTmpDir, dvdOutDir, true)) {
+		Failed(wxT(""));
 		return false;
+	}
 	
 	// create temporary directories
 	if (!wxDir::Exists(tmpDir) && !wxMkdir(tmpDir)) {
@@ -697,8 +750,8 @@ bool ProgressDlg::MultiplexSubtitles(wxArrayPtrVoid& subtitleVobs, wxArrayInt& s
 				movieSize = stream->GetSourceVideoSize();
 				videoFormat = movieSize.GetHeight() == 480 || movieSize.GetHeight() == 240 ? vfNTSC : vfPAL;
 			} else {
-				movieSize = GetFrameSize(videoFormat);
 				videoFormat = stream->GetVideoFormat();
+				movieSize = GetFrameSize(videoFormat);
 			}
 		}
 		
@@ -737,15 +790,16 @@ bool ProgressDlg::MultiplexSubtitles(const wxString& vobFile, TextSub* textSub,
 	textSub->SaveSpumux(spuFile, videoFormat);
 	cmd.Replace(wxT("$FILE_CONF"), spuFile);
 	cmd.Replace(wxT("$STREAM"), wxString::Format(wxT("%d"), streamIdx));
-	if (!Exec(cmd, vobFile, resultFile)) {
+	wxULongLong fileSize = wxFileName::GetSize(vobFile);
+	SpumuxExecute exec(this, fileSize != wxInvalidSize ? fileSize.ToDouble() : -1);
+	if (!exec.Execute(cmd, vobFile, resultFile)) {
 		Failed();
 		return false;
 	}
 	if (s_config.GetRemoveTempFiles()) {
-		if (vobFile == resultFile + _T(".old") && !DeleteFile(vobFile))
-			return false;
-		if (!DeleteFile(spuFile))
-			return false;
+		if (vobFile == resultFile + _T(".old"))
+			DeleteFile(vobFile);
+		DeleteFile(spuFile);
 	}
 
 	wxYield();
@@ -773,8 +827,7 @@ bool ProgressDlg::GenerateDvdFilesystem(DVD* dvd, const wxString& dvdTmpDir, con
 		wxDir d(dvdTmpDir);
 		wxString fname;
 		while (d.GetFirst(&fname, wxEmptyString, wxDIR_FILES))
-			if (!DeleteFile(dvdTmpDir + fname))
-				return false;
+			DeleteFile(dvdTmpDir + fname);
 	}
 	IncStep();
 	return true;
@@ -1108,15 +1161,18 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 		if (vob->GetKeepAspectRatio()) {
 			vob->UpdatePad(aspect);
 		}
-		if (vob->GetPad()[0] + vob->GetPad()[1] + vob->GetPad()[2] + vob->GetPad()[3] > 0) {
-			wxLogMessage(wxT("pad: %d %d %d %d"),
-					vob->GetPad()[0], vob->GetPad()[1], vob->GetPad()[2], vob->GetPad()[3]);
-			transcoder.SetPad(vob->GetPad());
-		}
-		if (vob->GetCrop()[0] + vob->GetCrop()[1] + vob->GetCrop()[2] + vob->GetCrop()[3] > 0) {
-			wxLogMessage(wxT("crop: %d %d %d %d"),
-					vob->GetCrop()[0], vob->GetCrop()[1], vob->GetCrop()[2], vob->GetCrop()[3]);
-			transcoder.SetCrop(vob->GetCrop());
+		
+		// set video filters
+		wxString videoFilters = vob->GetAllVideoFilters();
+		if (videoFilters.length())
+			wxLogMessage(wxT("Video filters: ") + videoFilters);
+		transcoder.SetVideoFilters(videoFilters);
+		
+		// set audio volume
+		for (unsigned int stIdx = 0; stIdx < vob->GetStreams().GetCount(); stIdx++) {
+			Stream* stream = vob->GetStreams()[stIdx];
+			if (stream->GetType() == stAUDIO && stream->GetAudioVolume() != 256)
+				transcoder.SetAudioVolume(stIdx, stream->GetAudioVolume());
 		}
 		
 		if (startTime != 0) {
@@ -1177,12 +1233,9 @@ bool ProgressDlg::Transcode(Vob* vob, AspectRatio aspect, int videoBitrate, bool
 			}
 			// remove temp files
 			if (s_config.GetRemoveTempFiles()) {
-				if (!DeleteFile(videoFile))
-					return false;
-				for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++) {
-					if (!DeleteFile(audioFiles[audioIdx]))
-						return false;
-				}
+				DeleteFile(videoFile);
+				for (unsigned int audioIdx = 0; audioIdx < audioFiles.GetCount(); audioIdx++)
+					DeleteFile(audioFiles[audioIdx]);
 			}
 		}
 	}
@@ -1194,8 +1247,7 @@ bool ProgressDlg::Multiplex(const wxString& videoFile, const wxArrayString& audi
 		const wxString& vobFile) {
 	if (IsCanceled())
 		return false;
-	AddDetailMsg(wxString::Format(_("Multiplexing video and audio streams"),
-			videoFile.c_str(), audioFiles[0].c_str()));
+	AddDetailMsg(_("Multiplexing video and audio streams"));
 	wxString cmd = s_config.GetMplexCmd();
 	cmd.Replace(_T("$FILE_VIDEO"), videoFile);
 	wxString audio;
@@ -1221,42 +1273,54 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 	wxString selFile = menuFile + _T("_select.png");
 	wxString spuFile = menuFile + _T("_spumux.xml");
 
-	bool videoMenu = menu->HasVideoBackground();
+	bool videoMenu = menu->GetSVG()->GetDuration() > 0;
 	wxYield();
 
-	// generate menu background and buttons
-	AddDetailMsg(_("Prepare"));
-	menu->SetTransparentColor();
-	wxImage bgImage = menu->GetBackgroundImage();
-	IncSubStep(10);
-
-	// convert jpeg to mpeg
-	AddDetailMsg(_("Create menu mpeg"));
-	if (s_config.GetMenuVideoBitrate()<1000)
+	AddDetailMsg(_("Create menu MPEG"));
+	if (s_config.GetMenuVideoBitrate() < 1000)
 		s_config.SetMenuVideoBitrate(1000);
-
-	if (!bgImage.Ok()) {
-		Failed(_("Error creation of menu"));
-		return false;
-	}
-
-  if (videoMenu) {
-		if (audioFile.length() || !MPEG::HasNavPacks(menu->GetBackground())) {
+	
+	if (videoMenu) {
+		bool hasAudio = audioFile.length() || menu->HasVideoBackground();
+		wxFfmpegMediaEncoder ffmpeg(s_config.GetThreadCount());
+		if (!ffmpeg.BeginEncode(hasAudio ? m2vFile : mpegFile, menu->GetVideoFormat(), hasAudio ? afNONE : audioFormat,
+				menu->GetAspectRatio(), s_config.GetMenuVideoBitrate())) {
+			Failed(_("Error creation of menu"));
+			return false;
+		}
+	    wxSVGDocument* svg = menu->GetBackgroundSVG();
+	    int width = menu->GetFrameResolution().GetWidth();
+	    int height = menu->GetFrameResolution().GetHeight();
+	    AddDetailMsg(wxString::Format(_("Video duration: %f sec"), svg->GetDuration()));
+	    int frameCount = (int) (svg->GetDuration() * (menu->GetVideoFormat() == vfPAL ? 25.0 : 30000.0 / 1001));
+	    if (frameCount == 0)
+	    	frameCount = s_config.GetMenuFrameCount();
+		for (int f = 0; f < frameCount; f++) {
+			if (IsCanceled())
+				return false;
+			if (f % 100 == 0)
+				AddDetailMsg(wxString::Format(_("Encoding frame %d of %d"), f, frameCount));
+			wxImage img = svg->Render(width, height, NULL, false);
+			if (!ffmpeg.EncodeImage(img, 1)) {
+				Failed(_("Error creation of menu"));
+				return false;
+			}
+			svg->SetCurrentTime(menu->GetVideoFormat() == vfPAL ? ((double) f) / 25 : ((double) f) * 1001 / 30000);
+			SetSubStep(200 * (f + 1) / frameCount);
+		}
+		ffmpeg.EndEncode();
+		if (hasAudio) {
+			AddDetailMsg(_("Multiplexing audio and video"));
 			Vob menuVob;
-			menuVob.SetFilename(menu->GetBackground());
-			if (audioFile.length())
-				menuVob.AddAudioFile(audioFile);
+			menuVob.SetFilename(m2vFile);
+			menuVob.GetVideoStream()->SetDestinationFormat(menu->GetVideoFormat());
+			if (hasAudio)
+				menuVob.AddAudioFile(audioFile.length() ? audioFile : menu->GetBackground());
 			for (unsigned int i=0; i<menuVob.GetStreams().GetCount(); i++) {
 				Stream* stream = menuVob.GetStreams()[i];
-				if (stream->GetType() == stVIDEO) {
-					// set destination format (default COPY) if codec is not mpeg2
-					if (stream->GetSourceCodecName() != wxT("mpeg2video"))
-						stream->SetDestinationFormat(menu->GetVideoFormat());
-				} else if (stream->GetType() == stAUDIO) {
-					// set destination format (default COPY) if codec is not mp2/ac3 or sample rate is not 48 kHz
-					if (stream->GetSourceCodecName() != wxT("mp2")
-							&& stream->GetSourceCodecName() != wxT("ac3")
-							&& stream->GetSourceCodecName() != wxT("liba52")) {
+				if (stream->GetType() == stAUDIO && stream->GetAudioFormat() == afCOPY) {
+					// change destination format if it set to COPY and codec is not mp2/ac3 or sample rate != 48 kHz
+					if (stream->GetSourceCodecName() != wxT("mp2") && stream->GetSourceCodecName() != wxT("ac3")) {
 						stream->SetDestinationFormat(audioFormat);
 					} else if (stream->GetSourceSampleRate() != 48000) {
 						stream->SetDestinationFormat(stream->GetSourceCodecName() == wxT("mp2") ? afMP2 : afAC3);
@@ -1267,9 +1331,15 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 			menuVob.SetTmpFilename(mpegFile);
 			if (!Transcode(&menuVob, menu->GetAspectRatio(), videoBitrate, s_config.GetUseMplexForMenus()))
 				return false;
-		} else
-			mpegFile = menu->GetBackground();
+			if (s_config.GetRemoveTempFiles())
+				DeleteFile(m2vFile);
+		}
 	} else {
+		wxImage bgImage = menu->GetBackgroundImage();
+		if (!bgImage.Ok()) {
+			Failed(_("Error creation of menu"));
+			return false;
+		}
 		int frameCount = s_config.GetMenuFrameCount();
 		if (audioFile.length() > 0) {
 			wxFfmpegMediaDecoder decoder;
@@ -1306,10 +1376,8 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 			for (unsigned int i=0; i<menuVob.GetStreams().GetCount(); i++) {
 				Stream* stream = menuVob.GetStreams()[i];
 				if (stream->GetType() == stAUDIO && stream->GetAudioFormat() == afCOPY) {
-					// set destination format if it set to COPY and codec is not mp2/ac3 or sample rate != 48 kHz
-					if (stream->GetSourceCodecName() != wxT("mp2")
-							&& stream->GetSourceCodecName() != wxT("ac3")
-							&& stream->GetSourceCodecName() != wxT("liba52")) {
+					// change destination format if it set to COPY and codec is not mp2/ac3 or sample rate != 48 kHz
+					if (stream->GetSourceCodecName() != wxT("mp2") && stream->GetSourceCodecName() != wxT("ac3")) {
 						stream->SetDestinationFormat(audioFormat);
 					} else if (stream->GetSourceSampleRate() != 48000) {
 						stream->SetDestinationFormat(stream->GetSourceCodecName() == wxT("mp2") ? afMP2 : afAC3);
@@ -1320,8 +1388,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 			if (!Transcode(&menuVob, menu->GetAspectRatio(), videoBitrate, s_config.GetUseMplexForMenus()))
 				return false;
 			if (s_config.GetRemoveTempFiles())
-				if (!DeleteFile(m2vFile))
-					return false;
+				DeleteFile(m2vFile);
 		}
 	}
 
@@ -1339,6 +1406,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 				return false;
 			}
 		}
+		// save subpictures
 		SubStreamMode mode = menu->GetAspectRatio() == ar4_3 ? ssmNORMAL : ssmWIDESCREEN;
 		if (stIdx == 1)
 			mode = widescreenType == wtNOLETTERBOX ? ssmPANSCAN : ssmLETTERBOX;
@@ -1354,19 +1422,22 @@ bool ProgressDlg::GenerateMenu(Menu* menu, WidescreenType widescreenType, const
 		wxString cmd = s_config.GetSpumuxCmd();
 		cmd.Replace(wxT("$FILE_CONF"), spuFile);
 		cmd.Replace(wxT("$STREAM"), wxString::Format(wxT("%d"), stIdx));
-		if (!Exec(cmd, mpegFile, menuFile)) {
+		wxULongLong fileSize = wxFileName::GetSize(mpegFile);
+		SpumuxExecute exec(this, fileSize != wxInvalidSize ? fileSize.ToDouble() : -1);
+		if (!exec.Execute(cmd, mpegFile, menuFile)) {
 			Failed();
 			return false;
 		}
 		if (s_config.GetRemoveTempFiles() || stIdx + 1 < stCount) {
-			if ((!videoMenu || mpegFile != menu->GetBackground()) && !DeleteFile(mpegFile))
-				return false;
-			if (!DeleteFile(btFile) || !DeleteFile(hlFile) || !DeleteFile(selFile) || !DeleteFile(spuFile))
-				return false;
+			if ((!videoMenu || mpegFile != menu->GetBackground()))
+				DeleteFile(mpegFile);
+			DeleteFile(btFile);
+			DeleteFile(hlFile);
+			DeleteFile(selFile);
+			DeleteFile(spuFile);
 		}
 	}
-	IncSubStep(50);
-
+	
 	wxYield();
 	return true;
 }
@@ -1387,7 +1458,7 @@ bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFil
 		return false;
 	}
 
-	for (unsigned i=0; i<slideshow->Count(); i++) {
+	for (unsigned i=0; i<slideshow->GetCount(); i++) {
 		AddDetailMsg(wxString::Format(_("Converting slide %d image to video"),i+1));
 		wxYield();
 		wxImage img = slideshow->GetImage(i);
@@ -1425,8 +1496,7 @@ bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFil
 				s_config.GetUseMplexForMenus()))
 			return false;
 		if (s_config.GetRemoveTempFiles())
-			if (!DeleteFile(m2vFile))
-				return false;
+			DeleteFile(m2vFile);
 	}
 
 	wxYield();
@@ -1435,7 +1505,7 @@ bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFil
 
 bool ProgressDlg::DeleteFile(wxString fname) {
 	if (!wxRemoveFile(fname)) {
-		Failed(wxString::Format(_("Can't remove file '%s'"), fname.c_str()));
+		AddDetailMsg(wxString::Format(_("Can't remove file '%s'"), fname.c_str()), *wxRED);
 		return false;
 	}
 	return true;
@@ -1458,12 +1528,12 @@ bool ProgressDlg::DeleteDir(wxString dir, bool subdirs) {
 	return true;
 }
 
-bool ProgressDlg::DeleteTempFiles(const wxString& tmpDir, const wxString& dvdTmpDir,
-		const wxString& dvdOutDir) {
+bool ProgressDlg::DeleteTempFiles(const wxString& tmpDir, const wxString& dvdTmpDir, const wxString& dvdOutDir,
+		bool deleteOutDir) {
 	AddDetailMsg(_("Cleaning temporary directory"));
 	if (wxDirExists(dvdTmpDir) && !DeleteDir(dvdTmpDir, false))
 		return false;
-	if (wxDirExists(dvdOutDir) && !DeleteDir(dvdOutDir, true))
+	if (deleteOutDir && wxDirExists(dvdOutDir) && !DeleteDir(dvdOutDir, true))
 		return false;
 	if (wxFileExists(tmpDir + TMP_ISO) && !DeleteFile(tmpDir + TMP_ISO))
 		return false;
