@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.h,v 1.32 2011/03/10 20:07:37 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.h,v 1.33 2011/04/14 07:29:18 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -146,7 +146,7 @@ private:
 	/** removes given directory and all files in it */
 	bool DeleteDir(wxString dir, bool subdirs = false);
 	/** removes given temporary directory and all temporary files in it */
-	bool DeleteTempFiles(const wxString& tmpDir, const wxString& dvdTmpDir, const wxString& dvdOutDir);
+	bool DeleteTempFiles(const wxString& tmpDir, const wxString& dvdTmpDir, const wxString& dvdOutDir, bool delOutDir);
 	/** executes given command */
 	bool Exec(wxString command, wxString inputFile = wxEmptyString, wxString outputFile = wxEmptyString);
 
