@@ -3,22 +3,18 @@
 // Purpose:     The subtitle properties dialog
 // Author:      Alex Thuering
 // Created:     24.02.2010
-// RCS-ID:      $Id: SubtitlePropDlg.cpp,v 1.7 2011/02/22 18:51:04 ntalex Exp $
+// RCS-ID:      $Id: SubtitlePropDlg.cpp,v 1.9 2011/08/01 20:29:04 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "SubtitlePropDlg.h"
+#include "DVD.h"
 #include "Utils.h"
 #include <wx/filename.h>
 #include <wx/colordlg.h>
 #include <wxVillaLib/utils.h>
 
-#include <fontconfig/fontconfig.h>
-#define FONTS_CONF wxFindDataFile(wxT("fonts.conf"))
-
-FontMap SubtitlePropDlg::s_fonts;
-
 // begin wxGlade: ::extracode
 
 // end wxGlade
@@ -89,10 +85,10 @@ SubtitlePropDlg::SubtitlePropDlg(wxWindow* parent, TextSub* textsub, const wxStr
     m_alignRadioBL = new wxRadioButton(this, wxID_ANY, wxEmptyString);
     m_alignRadioBC = new wxRadioButton(this, wxID_ANY, wxEmptyString);
     m_alignRadioBR = new wxRadioButton(this, wxID_ANY, wxEmptyString);
-    m_marginTop = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100);
-    m_marginLeft = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100);
-    m_marginRight = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100);
-    m_marginBottom = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 100);
+    m_marginTop = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
+    m_marginLeft = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
+    m_marginRight = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
+    m_marginBottom = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
     okBt = new wxButton(this, wxID_OK, _("OK"));
     m_cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
 
@@ -136,7 +132,7 @@ void SubtitlePropDlg::set_properties() {
 	m_charsetChoice->Append(DVD::GetCharsets());
 	m_charsetChoice->SetStringSelection(m_textsub->GetCharacterSet());
 
-    FontMap& fonts = GetFontMap();
+    FontMap& fonts = TextSub::GetFontMap();
     for (FontMap::iterator fontIt = fonts.begin(); fontIt != fonts.end(); fontIt++) {
     	m_fontFamilyBox->Append(fontIt->first);
     }
@@ -295,7 +291,7 @@ void SubtitlePropDlg::OnFontFamilySelected(wxCommandEvent& event) {
 	m_fontStyleBox->Clear();
 	if (m_fontFamilyBox->GetSelection() < 0)
 		return;
-	map<wxString, wxString>& styles = GetFontMap()[m_fontFamilyBox->GetStringSelection()];
+	map<wxString, wxString>& styles = TextSub::GetFontMap()[m_fontFamilyBox->GetStringSelection()];
 	for (map<wxString, wxString>::iterator styleIt = styles.begin(); styleIt != styles.end(); styleIt++) {
 		m_fontStyleBox->Append(styleIt->first);
 	}
@@ -356,61 +352,6 @@ void SubtitlePropDlg::OnFontChanged() {
 	}
 }
 
-/**
- * Returns font map (font family -> font style -> font filename)
- */
-FontMap& SubtitlePropDlg::GetFontMap() {
-	if (s_fonts.size() > 0)
-		return s_fonts;
-#ifdef __WXMSW__
-	wxString fontConfigFile;
-	if (!wxGetEnv(wxT("FONTCONFIG_FILE"), &fontConfigFile)) {
-		// load config and fonts
-		FcConfig* fc = FcConfigCreate ();
-		if (fc == NULL) {
-			wxLogError(wxString(wxT("SubtitlePropDlg::UpdateFontList(): ")) + wxString(wxT("FcConfigCreate failed.")));
-			return s_fonts;
-		}
-		FcConfigParseAndLoad(fc, (const FcChar8*) (const char*) FONTS_CONF.ToUTF8(), FcTrue);
-		FcConfigBuildFonts(fc);
-		FcConfigSetCurrent(fc);
-	}
-#endif
-	FcPattern* pattern = FcPatternBuild(NULL, FC_OUTLINE, FcTypeBool, 1, FC_SCALABLE, FcTypeBool, 1, NULL);
-	FcObjectSet* objectSet = FcObjectSetBuild(FC_FAMILY, FC_STYLE, FC_FULLNAME, FC_FILE, FC_INDEX, NULL);
-	FcFontSet* fontSet = FcFontList(0, pattern, objectSet);
-	FcObjectSetDestroy(objectSet);
-	FcPatternDestroy(pattern);
-	if (fontSet == NULL)
-		return s_fonts;
-	for (int fi = 0; fi < fontSet->nfont; fi++) {
-		// font file
-		FcChar8* str;
-		if (FcPatternGetString(fontSet->fonts[fi], FC_FILE, 0, &str) != FcResultMatch)
-			continue;
-		wxString fontFilename = wxString::FromUTF8((char*) str);
-#if 1
-		fontFilename = fontFilename.AfterLast(wxT('/'));
-#endif
-		// font file index
-		int fontFileIndex = 0;
-		FcPatternGetInteger(fontSet->fonts[fi], FC_INDEX, 0, &fontFileIndex);
-		if (fontFileIndex > 0)
-			continue;
-		// family name
-		if (FcPatternGetString(fontSet->fonts[fi], FC_FAMILY, 0, &str) != FcResultMatch)
-			continue;
-		wxString fontFamily = wxString::FromUTF8((char*) str);
-		// style name
-		if (FcPatternGetString(fontSet->fonts[fi], FC_STYLE, 0, &str) != FcResultMatch)
-			continue;
-		wxString fontStyle = wxString::FromUTF8((char*) str);
-		s_fonts[fontFamily][fontStyle] = fontFilename;
-	}
-	FcFontSetDestroy(fontSet);
-	return s_fonts;
-}
-
 int SubtitlePropDlg::ShowModal() {
 	int res = wxDialog::ShowModal();
 	if (res == wxID_OK) {
