@@ -3,7 +3,7 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.49 2011/02/20 18:55:38 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.cpp,v 1.53 2011/08/01 20:30:08 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -12,21 +12,26 @@
 #include "TitlesetManager.h"
 #include "MPEG.h"
 #include "Config.h"
+#include "ChaptersDlg.h"
 #include <wxVillaLib/utils.h>
 #include <wx/tokenzr.h>
 #include <wx/regex.h>
 #include <wx/file.h>
+#include <wx/spinctrl.h>
 #include "rc/add.png.h"
 #include "rc/remove.png.h"
 #include "rc/preferences.png.h"
 
+wxRegEx s_chapterRegEx(wxT("^(([[:digit:]]+:)?[[:digit:]][[:digit:]]?:)?[[:digit:]][[:digit:]]?(\\.[[:digit:]]+)?$"));
+
 enum {
-	DO_NOT_TRANSCODE_CB_ID = 7850,
-	VOB_BOX_ID,
+	VOB_BOX_ID = 7850,
 	SLIDESHOW_BOX_ID,
 	ADD_BT_ID,
 	REMOVE_BT_ID,
 	PROP_BT_ID,
+	DO_NOT_TRANSCODE_CB_ID,
+	CHAPTERS_BT_ID,
 	AUDIO_CTRL_ID
 };
 
@@ -37,6 +42,7 @@ BEGIN_EVENT_TABLE(TitlePropDlg, wxPropDlg)
 	EVT_BUTTON(REMOVE_BT_ID, TitlePropDlg::OnRemoveBt)
 	EVT_BUTTON(PROP_BT_ID, TitlePropDlg::OnPropBt)
 	EVT_CHECKBOX(DO_NOT_TRANSCODE_CB_ID, TitlePropDlg::OnDoNotTranscodeCheck)
+	EVT_BUTTON(CHAPTERS_BT_ID, TitlePropDlg::OnChaptersBt)
 	EVT_TEXT(AUDIO_CTRL_ID, TitlePropDlg::OnChangeAudio)
 END_EVENT_TABLE()
 
@@ -46,11 +52,14 @@ TitlePropDlg::TitlePropDlg(wxWindow* parent, DVD* dvd, int tsi, int pgci, int vo
 	m_pgci = pgci;
 	m_pgcs = &m_dvd->GetPgcArray(tsi, false);
 	m_pgc = (*m_pgcs)[pgci];
-	m_vob = m_pgc->GetVobs()[vobi];
+	m_vob = new Vob(*m_pgc->GetVobs()[vobi]);
 	m_vobi = vobi;
 	m_langCodes = &m_pgcs->GetAudioLangCodes();
 	m_vobListBox = NULL;
 	m_doNotTranscodeCheckBox = NULL;
+	m_chaptersCtrl = NULL;
+	m_chaptersBt = NULL;
+	m_pause = NULL;
 	m_slideShowBox = NULL;
 	m_audioFormat = NULL;
 	// set title
@@ -70,6 +79,11 @@ TitlePropDlg::TitlePropDlg(wxWindow* parent, DVD* dvd, int tsi, int pgci, int vo
 	OnSelectItem(evt);
 }
 
+TitlePropDlg::~TitlePropDlg() {
+	if (m_vob)
+		delete m_vob;
+}
+
 void TitlePropDlg::CreatePropPanel(wxSizer* sizer) {
 	if (m_vob->GetSlideshow()) {
 		CreateSlideshowGroup(sizer);
@@ -112,9 +126,22 @@ void TitlePropDlg::CreateVobGroup(wxSizer* sizer) {
 	UpdateDoNotTranscodeCheck();
 	grid->Add(2, 2);
 	grid->Add(2, 2);
-	AddTextProp(grid, _("Chapters:"), m_vob->GetChapters());
+	// chapters
+	AddText(grid, _("Chapters:"));
+	wxBoxSizer* chaptersPane = new wxBoxSizer(wxHORIZONTAL);
+	grid->Add(chaptersPane, 0, wxEXPAND);
+	AddTextProp(chaptersPane, wxT(""), m_vob->GetChapters());
+	m_chaptersCtrl = (wxTextCtrl*) GetLastControl();
+	m_chaptersBt = new wxButton(this, CHAPTERS_BT_ID, wxT("..."));
+	int h = m_chaptersBt->GetSize().GetHeight() > 24 ? m_chaptersBt->GetSize().GetHeight() : 24;
+	m_chaptersBt->SetSizeHints(h, h, h, h);
+	chaptersPane->Add(m_chaptersBt, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);
+	// Pause
 	AddSpinProp(grid, _("Pause:"), m_vob->GetPause(), -1, 254, 100, _("sec"));
+	m_pause = (wxSpinCtrl*) GetLastControl();
 	sizer->AddSpacer(6);
+	// update/enable chapters control
+	UpdateChaptersCtrl();
 }
 
 void TitlePropDlg::CreateSlideshowGroup(wxSizer* sizer) {
@@ -127,8 +154,8 @@ void TitlePropDlg::CreateSlideshowGroup(wxSizer* sizer) {
 	m_slideShowBox = new wxThumbnails(this, SLIDESHOW_BOX_ID);
 	m_slideShowBox->EnableDragging(false);
 	for (unsigned int i = 0; i < slideshow->GetCount(); i++) {
-		m_slideShowBox->InsertItem(new wxThumb(slideshow->Item(i)->GetFilename(),
-				slideshow->Item(i)->GetFilename().AfterLast(wxFILE_SEP_PATH)));
+		m_slideShowBox->InsertItem(new wxThumb(slideshow->GetSlide(i)->GetFilename(),
+				slideshow->GetSlide(i)->GetFilename().AfterLast(wxFILE_SEP_PATH)));
 	}
 	vobPane->Add(m_slideShowBox, 1, wxEXPAND);
 	
@@ -190,7 +217,7 @@ void TitlePropDlg::CreateTitleGroup(wxSizer* sizer) {
 	grid->AddGrowableCol(1);
 	
 	AddFileProp(grid, _("Palette:"), m_pgc->GetPalette(), wxFD_OPEN, wxT("..."), wxT("*.rgb;*.yuv"));
-	wxArrayString commands = GetPreCommands();
+	wxArrayString commands = GetCommandList(m_dvd, m_tsi);
 	AddComboProp(grid, _("Pre commands:"), m_pgc->GetPreCommands(), commands);
 	commands.Add(wxT("exit;"));
 	AddComboProp(grid, _("Post commands:"), m_pgc->GetPostCommands(), commands);
@@ -198,31 +225,32 @@ void TitlePropDlg::CreateTitleGroup(wxSizer* sizer) {
 }
 
 /**
- * Returns list of pre commands
+ * Returns list of commands for choice
  */
-wxArrayString TitlePropDlg::GetPreCommands() {
+wxArrayString TitlePropDlg::GetCommandList(DVD* dvd, int currentTsi) {
 	wxArrayString commands;
-	for (unsigned int pgci=0; pgci < m_dvd->GetVmgm().Count(); pgci++)
+	commands.Add(wxT("call last menu;"));
+	for (unsigned int pgci=0; pgci < dvd->GetVmgm().Count(); pgci++)
 		commands.Add(wxString::Format(wxT("call vmgm menu %d;"),pgci+1));
-	if (m_dvd->GetVmgm().Count() == 0 && m_dvd->IsEmptyMenuEnabled())
+	if (dvd->GetVmgm().Count() == 0 && dvd->IsEmptyMenuEnabled())
 		commands.Add(wxT("call vmgm menu 1;"));
-	for (int tsi=0; tsi<(int)m_dvd->GetTitlesets().Count(); tsi++) {
-		Titleset* ts = m_dvd->GetTitlesets()[tsi];
-		if (tsi == m_tsi) { // same titleset
+	for (int tsi=0; tsi<(int)dvd->GetTitlesets().Count(); tsi++) {
+		Titleset* ts = dvd->GetTitlesets()[tsi];
+		if (tsi == currentTsi) { // same titleset
 			if (ts->GetMenus().GetCount())
 				commands.Add(wxT("call menu;"));
-			StringSet entrySet = ts->GetMenus().GetEntries();
+			const StringSet& entrySet = ts->GetMenus().GetEntries();
 			for (StringSet::const_iterator entry = entrySet.begin(); entry != entrySet.end(); entry++)
 				if (*entry != wxT("root"))
 					commands.Add(wxString::Format(wxT("call menu entry %s;"), entry->c_str()));
 		}
-		if (m_dvd->IsJumppadsEnabled()) {
+		if (dvd->IsJumppadsEnabled()) {
 			for (int pgci=1; pgci<(int)ts->GetMenus().Count(); pgci++)
 				commands.Add(wxString::Format(wxT("call titleset %d menu %d;"), tsi+1, pgci+1));
 		}
-		if (m_dvd->IsJumppadsEnabled() || tsi == m_tsi) {
+		if (dvd->IsJumppadsEnabled() || tsi == currentTsi) {
 			for (int pgci=0; pgci<(int)ts->GetTitles().Count(); pgci++)
-				if (tsi == m_tsi) // same titleset
+				if (tsi == currentTsi) // same titleset
 					commands.Add(wxString::Format(wxT("jump title %d;"), pgci+1));
 				else
 					commands.Add(wxString::Format(wxT("jump titleset %d title %d;"), tsi+1, pgci+1));
@@ -254,7 +282,7 @@ bool TitlePropDlg::SetValues() {
 	int n = m_vob->GetSlideshow() ? 7 : 4;
 	wxString preCommands = GetString(n++);
 	wxString postCommands = GetString(n++);
-	DVDAction action(false, BUTTON_ACTION);
+	DVDAction action;
 	action.SetCustom(preCommands);
 	if (preCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, false, wxT(""), true, false)) {
 		return false;
@@ -263,7 +291,21 @@ bool TitlePropDlg::SetValues() {
 	if (postCommands.length() && !action.IsValid(m_dvd, m_tsi, m_pgci, false, wxT(""), true, false)) {
 		return false;
 	}
+
+	// check chapters
+	if (!m_vob->GetSlideshow() && m_chaptersCtrl->IsEnabled()) {
+		wxString chapters = m_chaptersCtrl->GetValue();
+		wxStringTokenizer tkz(chapters, _T(","));
+		while (tkz.HasMoreTokens()) {
+			wxString token = tkz.GetNextToken().Strip(wxString::both);
+			if (!s_chapterRegEx.Matches(token)) {
+				wxLogError(_("Invalid chapter timestamp '%s'"), token.c_str());
+				return false;
+			}
+		}
+	}
 	
+	// update values
 	n = 0;
 	if (m_vob->GetSlideshow()) {
 		Slideshow* slideshow = m_vob->GetSlideshow();
@@ -273,7 +315,7 @@ bool TitlePropDlg::SetValues() {
 		slideshow->SetDuration(GetInt(n++));
 		slideshow->Clear();
 		for (int i = 0; i < m_slideShowBox->GetItemCount(); i++)
-			slideshow->Add(new Slide(m_slideShowBox->GetItem(i)->GetFilename()));
+			slideshow->AddSlide(new Slide(m_slideShowBox->GetItem(i)->GetFilename()));
 		
 		// audio properties
 		wxString audioFile = GetString(n++).Strip(wxString::both);
@@ -291,32 +333,27 @@ bool TitlePropDlg::SetValues() {
 		m_langCodes->RemoveAt(0);
 		m_langCodes->Insert(GetString(n++), 0);
 	} else {
-		// check chapters
-		wxString chapters = GetString(1);
-		wxStringTokenizer tkz(chapters, _T(","));
-		while (tkz.HasMoreTokens()) {
-			wxString token = tkz.GetNextToken().Strip(wxString::both);
-			wxRegEx re(wxT("^(([[:digit:]]+:)?[[:digit:]][[:digit:]]?:)?[[:digit:]][[:digit:]]?(\\.[[:digit:]]+)?$"));
-			if (!re.Matches(token)) {
-				wxLogError(_("Invalid chapter timestamp '%s'"), token.c_str());
-				return false;
-			}
-		}
-		
 		// update vob
-		Vob* vob = m_vobListBox->SetValues();
-		vob->SetDoNotTranscode(GetBool(n++) && vob->GetAudioFilenames().GetCount() == 0);
-		vob->SetChapters(GetString(n++));
-		vob->SetPause(GetInt(n++));
-		// set new vob
-		m_pgc->GetVobs()[m_vobi] = vob;
-		m_vob = vob;
+		m_vobListBox->SetValues();
+		m_vob->SetDoNotTranscode(GetBool(n++) && m_vob->GetAudioFilenames().GetCount() == 0);
+		if (m_chaptersCtrl->IsEnabled()) {
+			m_vob->SetChapters(GetString(n++));
+			m_vob->SetPause(GetInt(n++));
+		} else {
+			m_vob->SetPause(0);
+			n += 2;
+		}
 		// update video parameters
 		m_pgcs->GetVideo().SetAspect(m_vobListBox->GetAspectRatio());
 	}
 	m_pgc->SetPalette(GetString(n++));
 	m_pgc->SetPreCommands(GetString(n++));
 	m_pgc->SetPostCommands(GetString(n++));
+	
+	// replace old VOB with new on in PGC
+	delete m_pgc->GetVobs()[m_vobi];
+	m_pgc->GetVobs()[m_vobi] = m_vob;
+	m_vob = NULL;
 	return true;
 }
 
@@ -325,7 +362,7 @@ void TitlePropDlg::OnSelectItem(wxCommandEvent& event) {
 		m_delBt->Enable(m_slideShowBox->GetSelectedCount() > 0);
 	} else {
 		m_delBt->Enable(m_vobListBox->GetSelection() > 0);
-		m_propBt->Enable(m_vobListBox->HasPropDialog());
+		m_propBt->Enable(m_vobListBox->GetSelection() > 0);
 	}
 }
 
@@ -398,6 +435,40 @@ void TitlePropDlg::UpdateDoNotTranscodeCheck() {
 	m_doNotTranscodeCheckBox->Enable(!m_vobListBox->HasAudioFiles() && MPEG::HasNavPacks(m_vob->GetFilename()));
 }
 
+void TitlePropDlg::OnChaptersBt(wxCommandEvent& event) {
+	// update
+	if (m_chaptersCtrl->IsEnabled()) {
+		wxStringTokenizer tkz(m_chaptersCtrl->GetValue(), _T(","));
+		while (tkz.HasMoreTokens()) {
+			wxString token = tkz.GetNextToken().Strip(wxString::both);
+			if (!s_chapterRegEx.Matches(token)) {
+				wxLogError(_("Invalid chapter timestamp '%s'"), token.c_str());
+				return;
+			}
+		}
+		m_vob->SetChapters(m_chaptersCtrl->GetValue());
+	}
+	ChaptersDlg dialog(this, m_dvd, m_tsi, m_pgci, m_vob);
+	if (dialog.ShowModal() == wxID_OK)
+		UpdateChaptersCtrl();
+}
+
+void TitlePropDlg::UpdateChaptersCtrl() {
+	m_chaptersCtrl->SetValue(m_vob->GetChapters());
+	bool readonly = false;
+	for (vector<Cell*>::iterator it = m_vob->GetCells().begin(); it != m_vob->GetCells().end(); it++) {
+		if (!(*it)->IsChapter() || (*it)->GetEnd() >= 0 || (*it)->GetPause() != 0
+				|| (*it)->GetCommands().length() > 0) {
+			readonly = true;
+			break;
+		}
+	}
+	m_chaptersCtrl->Enable(!readonly);
+	if (readonly)
+		m_pause->SetValue(0);
+	m_pause->Enable(!readonly);
+}
+
 void TitlePropDlg::OnChangeAudio(wxCommandEvent& event) {
 	if (!m_audioFormat)
 		return;
