@@ -3,7 +3,7 @@
 // Purpose:     DVD title properties dialog
 // Author:      Alex Thuering
 // Created:     31.01.2004
-// RCS-ID:      $Id: TitlePropDlg.h,v 1.18 2011/02/13 19:12:56 ntalex Exp $
+// RCS-ID:      $Id: TitlePropDlg.h,v 1.19 2011/04/28 21:40:59 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -20,8 +20,10 @@
 class TitlePropDlg: public wxPropDlg {
 public:
 	TitlePropDlg(wxWindow *parent, DVD* dvd, int tsi, int pgci, int vobi);
-	virtual ~TitlePropDlg() {}
-	
+	virtual ~TitlePropDlg();
+
+	/** Returns list of commands for choice */
+	static wxArrayString GetCommandList(DVD* dvd, int tsi);
 private:
 	DVD* m_dvd;
 	int m_tsi;
@@ -33,9 +35,12 @@ private:
 	wxArrayString* m_langCodes;
 	VobListBox* m_vobListBox;
 	wxCheckBox* m_doNotTranscodeCheckBox;
-	wxButton* m_addBt;
-	wxButton* m_delBt;
-	wxButton* m_propBt;
+	wxBitmapButton* m_addBt;
+	wxBitmapButton* m_delBt;
+	wxBitmapButton* m_propBt;
+	wxTextCtrl* m_chaptersCtrl;
+	wxButton* m_chaptersBt;
+	wxSpinCtrl* m_pause;
 	wxThumbnails* m_slideShowBox;
 	wxChoice* m_audioFormat;
 	void CreatePropPanel(wxSizer* sizer);
@@ -44,15 +49,15 @@ private:
 	void CreateAudioGroup(wxSizer* sizer);
 	void CreateTitleGroup(wxSizer* sizer);
 	bool SetValues();
-	/** Returns list of pre commands */
-	wxArrayString GetPreCommands();
 	void OnSelectItem(wxCommandEvent& event);
 	void OnAddBt(wxCommandEvent& event);
 	void OnRemoveBt(wxCommandEvent& event);
 	void OnPropBt(wxCommandEvent& event);
+	void OnChaptersBt(wxCommandEvent& event);
 	void OnChangeAudio(wxCommandEvent& event);
 	void OnDoNotTranscodeCheck(wxCommandEvent& event);
 	void UpdateDoNotTranscodeCheck();
+	void UpdateChaptersCtrl();
 	DECLARE_EVENT_TABLE()
 };
 
