@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.47 2011/02/20 18:56:27 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.58 2011/11/22 20:27:20 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -30,40 +30,53 @@
 ///////////////////////////// TitleDnDFile ///////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-class TitleDnDFile: public wxFileDropTarget {
+class TitleDnDFile: public wxDropTarget {
 public:
 	TitleDnDFile(TitlesetManager* pOwner): m_pOwner(pOwner) {
-		SetDataObject(new wxFileDataObject);
+		wxDataFormat actionDataFormat(DATAFORMAT_ACTION);
+		wxDataObjectComposite* dataObject = new wxDataObjectComposite();
+		dataObject->Add(m_fileDataObject = new wxFileDataObject);
+		dataObject->Add(m_actionDataObject = new wxCustomDataObject(actionDataFormat));
+		SetDataObject(dataObject);
 		SetDefaultAction(wxDragMove);
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def) {
-		wxFileDropTarget::OnEnter(x, y, def);
+		wxDropTarget::OnEnter(x, y, def);
 		return def;
 	}
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def) {
-		defDrag = def; // wxWidgets bug 1100327 (wxGTK <= 2.6.0)
-		wxFileDropTarget::OnDragOver(x, y, def);
+		wxDropTarget::OnDragOver(x, y, def);
 		return def;
 	}
 
 	virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def) {
-		defDrag = def;
-		wxFileDropTarget::OnData(x, y, def);
-		return def;
+		if (!GetData())
+			return wxDragError;
+		
+		wxDataObjectComposite* dobj = (wxDataObjectComposite*) m_dataObject;
+		
+		if (dobj->GetReceivedFormat() == wxDF_FILENAME) {
+			return OnDropFiles(x, y, m_fileDataObject->GetFilenames(), def) ? def : wxDragError;
+		} else if (dobj->GetReceivedFormat().GetId() == DATAFORMAT_ACTION) {
+			m_pOwner->MoveSelectedTo(m_pOwner->GetItemIndex(x, y));
+			return def;
+		}
+		
+		return wxDragError;
 	}
 
-	virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames) {
+	bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames, wxDragResult def) {
 		bool res = false;
 		for (int i = 0; i<(int)filenames.Count(); i++)
-			res = AddFile(filenames[i], i == 0) || res;
+			res = AddFile(filenames[i], i == 0, def) || res;
 		return res;
 	}
 	
-	bool AddFile(wxString filename, bool first) {
+	bool AddFile(wxString filename, bool first, wxDragResult def) {
 		if (wxImage::FindHandler(filename.AfterLast('.').Lower(), -1)) {
-			m_pOwner->AddImage(filename, defDrag == wxDragMove && first);
+			m_pOwner->AddImage(filename, def == wxDragMove && first);
 			return true;
 		} else if (wxThumbnails::IsVideo(filename)) {
 			wxFfmpegMediaDecoder decoder;
@@ -77,8 +90,12 @@ public:
 				else if (decoder.GetStreamType(i) == stAUDIO)
 					hasAudio = true;
 			}
+			if (!hasVideo && !hasAudio) {
+				wxLogError(_("File '%s' doesn't contain any audio or video streams."));
+				return false;
+			}
 			if (hasVideo)
-				m_pOwner->AddVideo(filename, defDrag == wxDragMove);
+				m_pOwner->AddVideo(filename, def == wxDragMove);
 			else
 				m_pOwner->AddAudio(filename);
 			return true;
@@ -94,12 +111,13 @@ public:
 		wxFfmpegMediaDecoder decoder;
 		if (!decoder.Load(filename))
 			return false;
-		m_pOwner->AddVideo(filename, defDrag == wxDragMove);
+		m_pOwner->AddVideo(filename, def == wxDragMove);
 		return true;
 	}
 protected:
 	TitlesetManager *m_pOwner;
-	wxDragResult defDrag;
+	wxFileDataObject* m_fileDataObject;
+	wxCustomDataObject* m_actionDataObject;
 };
 
 //////////////////////////////////////////////////////////////////////////////
@@ -316,6 +334,18 @@ void TitlesetManager::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 	dc.SelectObject(wxNullBitmap);
 }
 
+void TitlesetManager::Paint(wxPaintDC& dc) {
+	wxThumbnails::Paint(dc);
+	if (m_dvd->GetTitlesets().size() == 1 && m_dvd->GetTitlesets()[0]->GetTitles().size() == 0) {
+		dc.SetPen(*wxBLACK_PEN);
+		dc.SetBrush(wxBrush(GetBackgroundColour(), wxTRANSPARENT));
+		dc.SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
+		int col = m_items.GetCount() % m_cols;
+		dc.DrawText(_("Drag here your video files from file browser"),
+				col * (m_tWidth + m_tBorder) + 2 * m_tBorder, m_tBorder + m_tHeight / 2);
+	}
+}
+
 int TitlesetManager::GetRowHeight(unsigned int begRow, unsigned int count) {
 	int capHeight = 0;
 	for (unsigned int i=begRow; i<begRow+count; i++)
@@ -358,9 +388,8 @@ void TitlesetManager::UpdateItems() {
 				wxString fname;
 				wxString caption;
 				if (vob->GetSlideshow()) {
-					if (vob->GetSlideshow()->Count() > 1)
-						caption = wxString::Format(_("Slideshow (%d images)"),
-						vob->GetSlideshow()->Count());
+					if (vob->GetSlideshow()->GetCount() > 1)
+						caption = wxString::Format(_("Slideshow (%d images)"), vob->GetSlideshow()->GetCount());
 					else
 						caption = _("Slideshow (1 image)");
 				} else {
@@ -387,7 +416,6 @@ int TitlesetManager::GetIndex(int tsi, int pgci, int vobi, bool menu) {
 
 int TitlesetManager::GetIndex(int id) {
 	for (int i=0; i<(int)m_items.Count(); i++) {
-		//wxMessageBox(wxString::Format(_T("find %d ? %d"), id, m_items[i]->GetId()));
 		if (m_items[i]->GetId() == id)
 			return i;
 	}
@@ -409,16 +437,40 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 				+ _("Please add an audio track to it."));
 		return false;
 	}
-
+	
 	vob = new Vob(_T(""));
 	vob->SetFilename(fname);
+	Stream* videoStream = vob->GetVideoStream();
+	if (videoStream == NULL) {
+		wxLogError(_("%s doesn't contain video stream."), fname.c_str());
+		return false;
+	}
+	if (pgcs.Count() == 0) {
+		// first title => choose aspect ratio
+		pgcs.GetVideo().SetAspect(videoStream->GetSourceAspectRatio() > 1.4 ? ar16_9 : ar4_3);
+		// check DVD video format
+		if (m_dvd->GetTitlesets().Count() == 1) {
+			int fps = lroundf(videoStream->GetSourceFps());
+			if (isNTSC(m_dvd->GetVideoFormat()) && fps == 25) {
+				if (wxMessageBox(_("This file contains video in PAL format. Do you want to change DVD format to PAL too?"),
+						_("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxYES)
+					m_dvd->SetVideoFormat(vfPAL, true);
+			} else if (!isNTSC(m_dvd->GetVideoFormat()) && (fps == 24 || fps == 30)) {
+				if (wxMessageBox(_("This file contains video in NTSC format. Do you want to change DVD format to NTSC too?"),
+						_("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxYES)
+					m_dvd->SetVideoFormat(vfNTSC, true);
+			}
+		}
+	}
 	// check if reencoding is needed
 	for (unsigned int i=0; i<vob->GetStreams().GetCount(); i++) {
 		Stream* stream = vob->GetStreams()[i];
 		if (stream->GetType() == stVIDEO) {
 			// set destination format (default COPY) if codec is not mpeg2
-			if (stream->GetSourceCodecName() != wxT("mpeg2video")
-					|| stream->GetSourceVideoFormat() != m_dvd->GetVideoFormat())
+			if (!stream->GetSourceCodecName().StartsWith(wxT("mpeg2video"))
+					|| stream->GetSourceVideoFormat() != m_dvd->GetVideoFormat()
+					|| fabs(stream->GetSourceAspectRatio()
+							- GetFrameAspectRatio(vfPAL, pgcs.GetVideo().GetAspect())) >= 0.15)
 				stream->SetDestinationFormat(m_dvd->GetVideoFormat());
 			else if (vob->GetAudioFilenames().Count() == 0 && MPEG::HasNavPacks(vob->GetFilename()))
 				vob->SetDoNotTranscode(true);  // do not remultiplex/transcode if the video is already a VOB
@@ -429,6 +481,8 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 				stream->SetDestinationFormat(m_dvd->GetAudioFormat());
 				vob->SetDoNotTranscode(false);
 			}
+		} else if (stream->GetType() == stSUBTITLE) {
+			stream->SetDestinationFormat(sfNONE);
 		}
 	}
 	// set chapters
@@ -436,31 +490,24 @@ bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle, int tsi)
 	if (vob->GetDuration() > 0) {
 		int duration = (int) vob->GetDuration() / 60;
 		n = duration / s_config.GetDefChapterLength();
-		if (n > 0 && duration - s_config.GetDefChapterLength()*n
-				< s_config.GetDefChapterLength()/5)
+		if (n > 0 && duration - s_config.GetDefChapterLength()*n < s_config.GetDefChapterLength()/5)
 			n--;
 	}
 	if (n>100)
 		n = 100;
-	int t = s_config.GetDefChapterLength();
-	wxString cStr;
+	
+	if (!createTitle && pgc != NULL)
+		vob->GetCells().push_back(new Cell(0));
+	long t = s_config.GetDefChapterLength();
 	for (int i = 0; i < n; i++) {
-		if (cStr.length())
-			cStr += wxT(",");
-		if (t >= 60)
-			cStr += wxString::Format(wxT("%d:%02d:00"), t/60, t%60);
-		else
-			cStr += wxString::Format(wxT("%02d:00"), t%60);
+		vob->GetCells().push_back(new Cell(t * 60000));
 		t += s_config.GetDefChapterLength();
 	}
-	if (!createTitle && pgc != NULL)
-		cStr = wxT("0:00") + wxString(cStr.length() ? wxT(",") + cStr : wxT(""));
-	vob->SetChapters(cStr);
 	
 	// add vob
 	if (createTitle || pgc == NULL) {
 		pgc = new Pgc;
-		pgc->SetPostCommands(_T("call vmgm menu 1;"));
+		pgc->SetPostCommands(_T("call last menu;"));
 		pgc->GetVobs().Add(vob);
 		pgcs.Add(pgc);
 		if (!m_dvd->UpdateButtonImageFor(tsi, pgcs.GetCount() - 1) && pgcs.GetCount() > 4) {
@@ -548,7 +595,7 @@ void TitlesetManager::AddImage(const wxString& fname, bool createTitle) {
 
 	if (createTitle || pgc == NULL) {
 		pgc = new Pgc;
-		pgc->SetPostCommands(_T("call vmgm menu 1;"));
+		pgc->SetPostCommands(_T("call last menu;"));
 		pgcs.Add(pgc);
 	}
 
@@ -560,7 +607,7 @@ void TitlesetManager::AddImage(const wxString& fname, bool createTitle) {
 		Vob* vob = new Vob(slideshow);
 		pgc->GetVobs().Add(vob);
 	}
-	slideshow->Add(new Slide(fname));
+	slideshow->AddSlide(new Slide(fname));
 
 	UpdateItems();
 	SendDvdChangedEvent();
@@ -635,7 +682,7 @@ void TitlesetManager::OnMouseMove(wxMouseEvent &event) {
 	wxThumbnails::OnMouseMove(event);
 	// -- drag & drop --
 	if (event.Dragging() && m_selected >= 0 && m_beginDrag) {
-		DVDAction action(false, MENU_ACTION);
+		DVDAction action;
 		action.SetTsi(DVD::GetTsi(GetSelectedId()));
 		action.SetPgci(DVD::GetPgci(GetSelectedId()));
 		action.SetMenu(DVD::IsMenu(GetSelectedId()));
@@ -704,6 +751,90 @@ bool TitlesetManager::DuplicateMenu(int tsi, int pgci) {
 	return true;
 }
 
+void TitlesetManager::MoveSelectedTo(int idx) {
+	if (GetSelected() < 0)
+		return;
+	int id = GetSelectedId();
+	int tsi = DVD::GetTsi(id);
+	int pgci = DVD::GetPgci(id);
+	bool isMenu = DVD::IsMenu(id);
+	PgcArray& pgcs = m_dvd->GetPgcArray(tsi, isMenu);
+	
+	int id2 = -1;
+	if (idx < 0) {
+		int tsi2 = m_dvd->GetTitlesets().size() - 1;
+		int pgci2 = m_dvd->GetPgcArray(tsi2, false).size() - 1;
+		id2 = DVD::MakeId(tsi2, pgci2 >= 0 ? pgci2 : 0, 0, false);
+	} else
+		id2 = GetItem(idx)->GetId();
+	if (id == id2 || id2 < 0)
+		return;
+	int tsi2 = DVD::GetTsi(id2);
+	int pgci2 = DVD::GetPgci(id2);
+	bool isMenu2 = DVD::IsMenu(id2);
+	
+	if (!isMenu && isMenu2) {
+		pgci2 = 0;
+		isMenu2 = false;
+		if (tsi2 == -1)
+			return;
+	} else if (isMenu && !isMenu2) {
+		PgcArray& pgcs2 = m_dvd->GetPgcArray(tsi2, true);
+		pgci2 = pgcs2.size() > 0 ? pgcs2.size() - 1 : 0;
+		isMenu2 = true;
+	}
+	if (tsi2 == tsi) {
+		if (pgci2 > pgci) {
+			int pgciTmp = pgci;
+			while (pgciTmp != pgci2) {
+				Pgc* pgc = pgcs[pgciTmp];
+				pgcs[pgciTmp] = pgcs[pgciTmp + 1];
+				pgcs[++pgciTmp] = pgc;
+			}
+			if (isMenu) {
+				SelectMenu(tsi2, pgci2);
+			} else {
+				for (int i = pgci; i <= pgci2; i++)
+					m_dvd->UpdateButtonImageFor(tsi, i);
+			}
+		} else if (pgci2 < pgci) {
+			int pgciTmp = pgci;
+			while (pgciTmp != pgci2) {
+				Pgc* pgc = pgcs[pgciTmp];
+				pgcs[pgciTmp] = pgcs[pgciTmp - 1];
+				pgcs[--pgciTmp] = pgc;
+			}
+			if (isMenu) {
+				SelectMenu(tsi2, pgci2);
+			} else {
+				for (int i = pgci2; i <= pgci; i++)
+					m_dvd->UpdateButtonImageFor(tsi, i);
+			}
+		} else
+			return;
+	} else {
+		Pgc* pgc = pgcs[pgci];
+		pgcs.RemoveAt(pgci);
+		PgcArray& pgcs2 = m_dvd->GetPgcArray(tsi2, isMenu2);
+		pgcs2.Insert(pgc, pgci2);
+		if (!isMenu) {
+			for (unsigned int i = pgci; i < pgcs.size(); i++)
+				m_dvd->UpdateButtonImageFor(tsi, i);
+			for (unsigned int i = pgci2; i < pgcs2.size(); i++)
+				m_dvd->UpdateButtonImageFor(tsi2, i);
+		} else {
+			SelectMenu(tsi2, pgci2);
+		}
+		if (tsi >= 0 && m_dvd->GetPgcArray(tsi, false).size() == 0
+				&& m_dvd->GetPgcArray(tsi, true).size() == 0) {
+			delete m_dvd->GetTitlesets()[tsi];
+			m_dvd->GetTitlesets().RemoveAt(tsi);
+		}
+	}
+	UpdateItems();
+	SendDvdChangedEvent();
+}
+
 void TitlesetManager::OnAddTitlesetUpdateUI(wxUpdateUIEvent& event) {
 	int cnt = m_dvd->GetTitlesets().Count();
 	event.Enable(cnt>0 && m_dvd->GetTitlesets()[cnt-1]->GetTitles().Count());
