@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.h,v 1.16 2010/09/05 10:56:08 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.h,v 1.18 2011/08/28 10:55:20 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -37,6 +37,7 @@ class TitlesetManager: public wxThumbnails {
 	void AddVmMenu();
 	void AddTitleset();
 	bool DuplicateMenu(int tsi, int pgci);
+	void MoveSelectedTo(int idx);
     
     /** updates items to display current content of DVD */
     void UpdateItems();
@@ -46,6 +47,7 @@ class TitlesetManager: public wxThumbnails {
 	int m_selectedMenu;
 	wxBitmap GetThumbImage(wxThumb& thumb);
     void DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index);
+    void Paint(wxPaintDC& dc);
     int GetRowHeight(unsigned int begRow, unsigned int count);
 	/** returns index of thumbnail for specified vob */
 	int GetIndex(int tsi, int pgci, int vobi, bool menu);
