@@ -3,7 +3,7 @@
 // Purpose:     Miscellaneous utilities
 // Author:      Alex Thuering
 // Created:		06.04.2008
-// RCS-ID:      $Id: Utils.h,v 1.3 2010/04/03 21:46:36 ntalex Exp $
+// RCS-ID:      $Id: Utils.h,v 1.4 2011/04/28 21:35:50 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -36,5 +36,13 @@ wxString GetTextFromClipboard();
  * Sorts string arrays by value
  */
 int Sort(wxArrayString& keys, wxArrayString& values, const wxString& selectedKey = wxEmptyString);
+/**
+ * Converts time span value (milliseconds) in string
+ */
+wxString Time2String(long value, bool full = false);
+/**
+ * Converts time span value (milliseconds) in string
+ */
+long String2Time(const wxString& value);
 
 #endif // UTILS_H
