@@ -3,12 +3,14 @@
 // Purpose:     The video properties dialog
 // Author:      Alex Thuering
 // Created:     09.09.2010
-// RCS-ID:      $Id: VideoPropDlg.cpp,v 1.8 2011/03/10 20:05:41 ntalex Exp $
+// RCS-ID:      $Id: VideoPropDlg.cpp,v 1.11 2011/08/01 20:30:08 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "VideoPropDlg.h"
+#include "DVD.h"
+#include "Utils.h"
 #include <wx/artprov.h>
 #include <wx/regex.h>
 
@@ -30,8 +32,10 @@ enum {
 	CROP_RIGHT_CTRL_ID,
 	CROP_DOWN_CTRL_ID,
 	SLIDER_ID,
-	START_BT,
-	END_BT
+	TIME_CTRL_ID,
+	TIME_SPIN_BT_ID,
+	START_BT_ID,
+	END_BT_ID
 };
 
 BEGIN_EVENT_TABLE(VideoPropDlg, wxDialog)
@@ -49,8 +53,10 @@ BEGIN_EVENT_TABLE(VideoPropDlg, wxDialog)
     EVT_SPINCTRL(CROP_RIGHT_CTRL_ID, VideoPropDlg::OnChangeCrop)
     EVT_SPINCTRL(CROP_DOWN_CTRL_ID, VideoPropDlg::OnChangeCrop)
     EVT_COMMAND_SCROLL(SLIDER_ID, VideoPropDlg::OnSliderScroll)
-    EVT_BUTTON(START_BT, VideoPropDlg::OnStartBt)
-    EVT_BUTTON(END_BT, VideoPropDlg::OnEndBt)
+    EVT_TEXT(TIME_CTRL_ID, VideoPropDlg::OnChangeTime)
+    EVT_SPIN(TIME_SPIN_BT_ID, VideoPropDlg::OnTimeSpin)
+    EVT_BUTTON(START_BT_ID, VideoPropDlg::OnStartBt)
+    EVT_BUTTON(END_BT_ID, VideoPropDlg::OnEndBt)
     EVT_BUTTON(wxID_OK, VideoPropDlg::OnOkBt)
     // end wxGlade
 END_EVENT_TABLE()
@@ -61,7 +67,7 @@ END_EVENT_TABLE()
 VideoPropDlg::VideoPropDlg(wxWindow* parent, Vob* vob, AspectRatio aspectRatio):
 		wxDialog(parent, -1, wxT(""), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER) {
 	m_vob = vob;
-	m_stream = vob->GetStreams()[0];
+	m_stream = vob->GetVideoStream();
 	m_aspectRatio = aspectRatio;
 	
     // begin wxGlade: VideoPropDlg::VideoPropDlg
@@ -98,16 +104,25 @@ VideoPropDlg::VideoPropDlg(wxWindow* parent, Vob* vob, AspectRatio aspectRatio):
     m_cropLeft = new wxSpinCtrl(this, CROP_LEFT_CTRL_ID, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
     m_cropRight = new wxSpinCtrl(this, CROP_RIGHT_CTRL_ID, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
     m_cropBottom = new wxSpinCtrl(this, CROP_DOWN_CTRL_ID, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
+    m_fadeInLabel = new wxStaticText(this, wxID_ANY, _("Fade-In:"));
+    m_fadeInCtrl = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
+    m_fadeInLabel2 = new wxStaticText(this, wxID_ANY, _("sec"));
+    m_fadeOutLabel = new wxStaticText(this, wxID_ANY, _("Fade-Out:"));
+    m_fadeOutCtrl = new wxSpinCtrl(this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 999);
+    m_fadeOutLabel2 = new wxStaticText(this, wxID_ANY, _("sec"));
+    m_filtersLabel = new wxStaticText(this, wxID_ANY, _("Filters:"));
+    m_filtersCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
     m_mediaCtrl = new MediaCtrlFF(this, wxID_ANY);
     m_slider = new wxSlider(this, SLIDER_ID, 0, 0, 10, wxDefaultPosition, wxDefaultSize, wxSL_HORIZONTAL|wxSL_TOP);
     m_timeLabel = new wxStaticText(this, wxID_ANY, _("Time:"));
-    m_timeCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
+    m_timeCtrl = new wxTextCtrl(this, TIME_CTRL_ID, wxEmptyString);
+    m_timeSpinBt = new wxSpinButton(this, TIME_SPIN_BT_ID, wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS);
     m_startLabel = new wxStaticText(this, wxID_ANY, _("Start:"));
     m_startCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
-    m_startBt = new wxBitmapButton(this, START_BT, wxNullBitmap);
+    m_startBt = new wxBitmapButton(this, START_BT_ID, wxNullBitmap);
     m_endLabel = new wxStaticText(this, wxID_ANY, _("End:"));
     m_endCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
-    m_endBt = new wxBitmapButton(this, END_BT, wxNullBitmap);
+    m_endBt = new wxBitmapButton(this, END_BT_ID, wxNullBitmap);
     m_okBt = new wxButton(this, wxID_OK, _("OK"));
     m_cancelBt = new wxButton(this, wxID_CANCEL, _("Cancel"));
 
@@ -117,7 +132,16 @@ VideoPropDlg::VideoPropDlg(wxWindow* parent, Vob* vob, AspectRatio aspectRatio):
     
     wxCommandEvent evt;
     OnChangeFormat(evt);
-    m_dstChoice->SetFocus();
+    
+    if (m_vob->GetDoNotTranscode()) {
+    	m_dstChoice->Enable(false);
+    	m_startCtrl->Enable(false);
+    	m_endCtrl->Enable(false);
+    	m_startBt->Enable(false);
+    	m_endBt->Enable(false);
+    	m_aspectChoice->SetFocus();
+    } else
+    	m_dstChoice->SetFocus();
 }
 
 
@@ -136,28 +160,31 @@ void VideoPropDlg::set_properties() {
     m_cropBottom->SetMinSize(wxSize(58, -1));
     m_mediaCtrl->SetMinSize(wxSize(300, 200));
     m_mediaCtrl->SetWindowStyle(wxBORDER_NONE);
-    m_timeCtrl->SetMinSize(wxSize(80, -1));
-    m_startCtrl->SetMinSize(wxSize(80, -1));
+    m_timeSpinBt->SetMinSize(wxSize(16, -1));
     m_startBt->SetBitmapLabel(wxArtProvider::GetBitmap(wxART_CUT, wxART_BUTTON));
     m_startBt->SetSize(m_startBt->GetBestSize());
-    m_endCtrl->SetMinSize(wxSize(80, -1));
     m_endBt->SetBitmapLabel(wxArtProvider::GetBitmap(wxART_CUT, wxART_BUTTON));
     m_endBt->SetSize(m_endBt->GetBestSize());
     m_okBt->SetDefault();
     // end wxGlade
     
+    int x = 0;
+	m_timeCtrl->GetTextExtent(wxT("00:00:00.000"), &x, NULL);
+	m_timeCtrl->SetMinSize(wxSize(x + 10, -1));
+	m_startCtrl->SetMinSize(wxSize(x + 10, -1));
+	m_endCtrl->SetMinSize(wxSize(x + 10, -1));
+    
     m_fileNameText->SetLabel(m_vob->GetFilename());
     // duration
-	wxString s = _("Duration:") + wxString(wxT(" "));
+	wxString s = wxT("N/A");
 	if (m_vob->GetDuration()>0) {
 		int secs = (int) m_vob->GetDuration();
 		int mins = secs / 60;
 		secs %= 60;
 		int hours = mins / 60;
 		mins %= 60;
-		s += wxString::Format(wxT("%02d:%02d:%02d"), hours, mins, secs);
-	} else
-		s += wxT("N/A");
+		s = wxString::Format(wxT("%02d:%02d:%02d"), hours, mins, secs);
+	}
     m_durText->SetLabel(s);
     m_srcText->SetLabel(m_stream->GetSourceFormat());
     
@@ -183,6 +210,11 @@ void VideoPropDlg::set_properties() {
     m_cropTop->SetValue(m_vob->GetCrop()[2]);
     m_cropBottom->SetValue(m_vob->GetCrop()[3]);
     
+    // fade-in / fade-out
+    m_fadeInCtrl->SetValue(m_vob->GetFadeIn());
+    m_fadeOutCtrl->SetValue(m_vob->GetFadeOut());
+    m_filtersCtrl->SetValue(m_vob->GetVideoFilters());
+    
     // start time
     int ms = m_vob->GetStartTime() * 1000;
     ms = ms % 1000;
@@ -200,12 +232,14 @@ void VideoPropDlg::set_properties() {
 
 void VideoPropDlg::do_layout() {
     // begin wxGlade: VideoPropDlg::do_layout
-    wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
+    wxBoxSizer* mainVSizer = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
-    wxBoxSizer* mainSzer2 = new wxBoxSizer(wxHORIZONTAL);
+    wxBoxSizer* mainHSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* mediaSizer = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* timeSizer = new wxBoxSizer(wxHORIZONTAL);
-    wxFlexGridSizer* propSizer = new wxFlexGridSizer(7, 2, 4, 2);
+    wxFlexGridSizer* propSizer = new wxFlexGridSizer(10, 2, 4, 2);
+    wxBoxSizer* fadeOutSizer = new wxBoxSizer(wxHORIZONTAL);
+    wxBoxSizer* fadeInSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* propSizer2 = new wxBoxSizer(wxHORIZONTAL);
     wxStaticBoxSizer* m_cropBox = new wxStaticBoxSizer(m_cropBox_staticbox, wxHORIZONTAL);
     wxFlexGridSizer* cropSizer = new wxFlexGridSizer(3, 3, 4, 4);
@@ -251,34 +285,45 @@ void VideoPropDlg::do_layout() {
     propSizer2->Add(m_cropBox, 0, wxEXPAND, 0);
     propSizer2->Add(2, 2, 1, wxEXPAND, 0);
     propSizer->Add(propSizer2, 1, wxLEFT|wxRIGHT|wxTOP|wxEXPAND, 6);
+    propSizer->Add(m_fadeInLabel, 0, wxALIGN_CENTER_VERTICAL, 0);
+    fadeInSizer->Add(m_fadeInCtrl, 0, wxALIGN_CENTER_VERTICAL, 0);
+    fadeInSizer->Add(m_fadeInLabel2, 1, wxLEFT|wxALIGN_CENTER_VERTICAL, 4);
+    propSizer->Add(fadeInSizer, 1, wxEXPAND, 0);
+    propSizer->Add(m_fadeOutLabel, 0, wxALIGN_CENTER_VERTICAL, 0);
+    fadeOutSizer->Add(m_fadeOutCtrl, 0, wxALIGN_CENTER_VERTICAL, 0);
+    fadeOutSizer->Add(m_fadeOutLabel2, 1, wxLEFT|wxALIGN_CENTER_VERTICAL, 4);
+    propSizer->Add(fadeOutSizer, 1, wxEXPAND, 0);
+    propSizer->Add(m_filtersLabel, 0, 0, 0);
+    propSizer->Add(m_filtersCtrl, 1, wxEXPAND, 0);
     propSizer->Add(20, 4, 0, wxEXPAND, 0);
     propSizer->Add(20, 4, 0, wxEXPAND, 0);
     propSizer->Add(4, 4, 0, 0, 0);
     propSizer->Add(4, 4, 0, 0, 0);
-    propSizer->AddGrowableRow(6);
+    propSizer->AddGrowableRow(9);
     propSizer->AddGrowableCol(1);
-    mainSzer2->Add(propSizer, 0, wxALL|wxEXPAND, 10);
+    mainHSizer->Add(propSizer, 0, wxALL|wxEXPAND, 10);
     mediaSizer->Add(m_mediaCtrl, 1, wxEXPAND, 0);
     mediaSizer->Add(m_slider, 0, wxEXPAND, 0);
     timeSizer->Add(m_timeLabel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 2);
-    timeSizer->Add(m_timeCtrl, 0, wxRIGHT, 4);
+    timeSizer->Add(m_timeCtrl, 0, wxEXPAND, 0);
+    timeSizer->Add(m_timeSpinBt, 0, wxRIGHT|wxEXPAND|wxALIGN_CENTER_VERTICAL, 4);
     timeSizer->Add(m_startLabel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 2);
-    timeSizer->Add(m_startCtrl, 0, wxRIGHT, 2);
+    timeSizer->Add(m_startCtrl, 0, wxRIGHT|wxEXPAND, 2);
     timeSizer->Add(m_startBt, 0, wxRIGHT, 4);
     timeSizer->Add(m_endLabel, 0, wxRIGHT|wxALIGN_CENTER_VERTICAL, 2);
-    timeSizer->Add(m_endCtrl, 0, wxRIGHT, 2);
+    timeSizer->Add(m_endCtrl, 0, wxRIGHT|wxEXPAND, 2);
     timeSizer->Add(m_endBt, 0, 0, 0);
     mediaSizer->Add(timeSizer, 0, wxEXPAND, 0);
-    mainSzer2->Add(mediaSizer, 1, wxALL|wxEXPAND, 4);
-    mainSizer->Add(mainSzer2, 1, wxEXPAND, 0);
+    mainHSizer->Add(mediaSizer, 1, wxALL|wxEXPAND, 4);
+    mainVSizer->Add(mainHSizer, 1, wxEXPAND, 0);
     buttonSizer->Add(10, 10, 1, wxEXPAND, 0);
     buttonSizer->Add(m_okBt, 0, 0, 0);
     buttonSizer->Add(8, 8, 0, 0, 0);
     buttonSizer->Add(m_cancelBt, 0, 0, 0);
     buttonSizer->Add(10, 10, 1, wxEXPAND, 0);
-    mainSizer->Add(buttonSizer, 0, wxLEFT|wxRIGHT|wxBOTTOM|wxEXPAND, 4);
-    SetSizer(mainSizer);
-    mainSizer->Fit(this);
+    mainVSizer->Add(buttonSizer, 0, wxLEFT|wxRIGHT|wxBOTTOM|wxEXPAND, 4);
+    SetSizer(mainVSizer);
+    mainVSizer->Fit(this);
     Layout();
     Centre();
     // end wxGlade
@@ -300,8 +345,8 @@ double TimeToDouble(wxString timeStr) {
 	if (val.ToLong(&lval))
 		result += lval;
 	val = re.GetMatch(timeStr, 5).Mid(1);
-	if (val.ToLong(&lval))
-		result += ((double) lval) / 1000;
+	if (val.ToLong(&lval) && val.length() <= 3)
+		result += ((double) lval) / (val.length() == 1 ? 10 : val.length() == 2 ? 100 : 1000);
 	return result;
 }
 
@@ -329,6 +374,9 @@ vector<int> VideoPropDlg::GetCrop() {
 int VideoPropDlg::ShowModal() {
 	m_mediaCtrl->Load(m_vob->GetFilename());
 	m_slider->SetMax(m_mediaCtrl->Length()/1000);
+	m_timeSpinBt->SetMax(m_mediaCtrl->Length()/1000);
+	wxScrollEvent scrlEvt;
+	OnSliderScroll(scrlEvt);
 	int res = wxDialog::ShowModal();
 	if (res == wxID_OK) {
 		m_stream->SetDestinationFormat(GetVideoFormat());
@@ -336,8 +384,10 @@ int VideoPropDlg::ShowModal() {
 		m_vob->GetCrop() = GetCrop();
 		m_vob->SetInterlaced(m_interlacedChoice->GetSelection() == 1);
 		m_vob->SetFirstField((FirstField) (m_ffChoice->GetSelection() - 1));
-		m_aspectRatio = GetAspectRatio();
 		m_vob->SetKeepAspectRatio(m_keepAspectCtrl->GetValue());
+		m_vob->SetFadeIn(m_fadeInCtrl->GetValue());
+		m_vob->SetFadeOut(m_fadeOutCtrl->GetValue());
+		m_vob->SetVideoFilters(m_filtersCtrl->GetValue());
 		double time = TimeToDouble(m_startCtrl->GetValue());
 		m_vob->SetStartTime(time >= 0 ? time : 0);
 		time = TimeToDouble(m_endCtrl->GetValue());
@@ -361,6 +411,14 @@ void VideoPropDlg::UpdatePad() {
 	m_mediaCtrl->SetVideoFormat(GetVideoFormat(), GetAspectRatio(), GetPad(), GetCrop());
 }
 
+void VideoPropDlg::SeekVideo(long pos, bool updateTimeCtrl) {
+	m_slider->SetValue(lround(pos / 1000));
+	m_timeSpinBt->SetValue(lround(pos / 1000));
+	m_mediaCtrl->Seek((wxFileOffset) pos);
+	if (updateTimeCtrl)
+		m_timeCtrl->ChangeValue(Time2String(pos, true));
+}
+
 // wxGlade: add VideoPropDlg event handlers
 void VideoPropDlg::OnChangeFormat(wxCommandEvent &event) {
 	// update keepAspect, border and crop controls
@@ -394,6 +452,9 @@ void VideoPropDlg::OnChangeFormat(wxCommandEvent &event) {
 	m_cropRight->Enable(enable);
 	m_cropTop->Enable(enable);
 	m_cropBottom->Enable(enable);
+	m_fadeInCtrl->Enable(enable);
+	m_fadeOutCtrl->Enable(enable);
+	m_filtersCtrl->Enable(enable);
 	wxCommandEvent evt;
 	OnInterlaced(evt);
 	UpdatePad();
@@ -435,10 +496,17 @@ void VideoPropDlg::OnChangeCrop(wxSpinEvent &event) {
 }
 
 void VideoPropDlg::OnSliderScroll(wxScrollEvent &event) {
-	m_mediaCtrl->Seek(((wxFileOffset)m_slider->GetValue())*1000);
-	int ms = m_slider->GetValue() != 0 ? m_mediaCtrl->Tell() % 1000 : 0;
-	int t = m_mediaCtrl->Tell() / 1000;
-	m_timeCtrl->SetValue(wxString::Format(wxT("%d:%02d:%02d.%03d"), t/3600, (t/60) % 60, t%60, ms));
+	SeekVideo(((long)m_slider->GetValue())*1000);
+}
+
+void VideoPropDlg::OnChangeTime(wxCommandEvent &event) {
+	wxRegEx re(wxT("^(([[:digit:]]+:)?[[:digit:]][[:digit:]]?:)?[[:digit:]][[:digit:]]?(\\.[[:digit:]]+)?$"));
+	if (re.Matches(m_timeCtrl->GetValue()))
+		SeekVideo(String2Time(m_timeCtrl->GetValue()), false);
+}
+
+void VideoPropDlg::OnTimeSpin(wxSpinEvent &event) {
+	SeekVideo(((long)m_timeSpinBt->GetValue())*1000);
 }
 
 void VideoPropDlg::OnStartBt(wxCommandEvent &event) {
