@@ -3,7 +3,7 @@
 // Purpose:     The list box to display information about streams in given VOB
 // Author:      Alex Thuering
 // Created:     03.05.2009
-// RCS-ID:      $Id: VobListBox.cpp,v 1.16 2011/03/10 20:09:08 ntalex Exp $
+// RCS-ID:      $Id: VobListBox.cpp,v 1.19 2011/08/01 20:30:08 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -11,6 +11,7 @@
 #include "VobListBox.h"
 #include "Config.h"
 #include "VideoPropDlg.h"
+#include "AudioPropDlg.h"
 #include "SubtitlePropDlg.h"
 #include <wx/filedlg.h>
 #include <wx/arrimpl.cpp>
@@ -26,13 +27,14 @@ WX_DEFINE_OBJARRAY(StringListOfList);
 
 BEGIN_EVENT_TABLE(VobListBox, wxVListBox)
 	EVT_LEFT_DCLICK(VobListBox::OnDoubleClick)
+	EVT_BUTTON(wxID_ANY, VobListBox::OnButton)
 END_EVENT_TABLE()
 
 const int ITEM_HEIGHT = 50;
 #define ITEM_FONT wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT)
 
 VobListBox::VobListBox(wxWindow* parent, wxWindowID id, Vob* vob, PgcArray* pgcs, DVD* dvd): wxVListBox(parent, id) {
-	m_vob = new Vob(*vob);
+	m_vob = vob;
 	m_pgcs = pgcs;
 	m_dvd = dvd;
 	m_aspectRatio = pgcs->GetVideo().GetAspect();
@@ -72,6 +74,7 @@ void VobListBox::RefreshInfo() {
 	m_infoRect.Clear();
 	
 	int choiceIdx = 0;
+	int buttonIdx = 0;
 	int itemY = 0;
 	unsigned int audioIdx = 0;
 	unsigned int subtitleIdx = 0;
@@ -111,19 +114,29 @@ void VobListBox::RefreshInfo() {
 			wxString srcFormat = stream->GetSourceFormat();
 			switch (stream->GetType()) {
 			case stVIDEO: {
+				m_videoChoiceIdx = choiceIdx;
 				y = AddInfo(_("Video:") + wxString(wxT(" ")) + srcFormat, n, dc, x, y);
 				wxRect& rect = m_infoRect[n][m_infoRect[n].size()-1];
-				y += AddChoiceCtrl(DVD::GetVideoFormatLabels(true), stream->GetVideoFormat() - 1, rect, itemY,
+				int top = rect.GetTop();
+				int x = rect.GetRight() + 4;
+				AddChoiceCtrl(DVD::GetVideoFormatLabels(true), stream->GetVideoFormat() - 1, x, itemY + top,
 						choiceIdx, !m_vob->GetDoNotTranscode());
+				y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
+				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+				// button
+				AddButton(x, itemY + top, buttonIdx, true);
 				break;
 			}
 			case stAUDIO: {
 				y = AddInfo(_("Audio:") + wxString(wxT(" ")) + srcFormat, n, dc, x, y);
 				wxRect& rect = m_infoRect[n][m_infoRect[n].size()-1];
-				wxRect rect2 = rect; // copy before it will be changed
-				y += AddChoiceCtrl(DVD::GetAudioFormatLabels(true, true), stream->GetAudioFormat(), rect, itemY,
+				int top = rect.GetTop();
+				int x = rect.GetRight() + 4;
+				AddChoiceCtrl(DVD::GetAudioFormatLabels(true, true), stream->GetAudioFormat(), x, itemY + top,
 						choiceIdx, !m_vob->GetDoNotTranscode());
-				rect2.x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+				y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
+				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+				// langCode
 				wxString langCode = s_config.GetDefAudioLanguage();
 				if (stream->GetAudioFormat() != afNONE) {
 					if (audioIdx < m_pgcs->GetAudioLangCodes().size())
@@ -131,16 +144,22 @@ void VobListBox::RefreshInfo() {
 					audioIdx++;
 				}
 				int langIdx = DVD::GetAudioLanguageCodes().Index(langCode);
-				AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, rect2, itemY, choiceIdx, true);
+				AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, x, itemY + top, choiceIdx, true);
+				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+				// button
+				AddButton(x, itemY + top, buttonIdx, true);
 				break;
 			}
 			case stSUBTITLE: {
 				y = AddInfo(_("Subtitle:") + wxString(wxT(" ")) + srcFormat, n, dc, x, y);
 				wxRect& rect = m_infoRect[n][m_infoRect[n].size()-1];
-				wxRect rect2 = rect; // copy before it will be changed
-				y += AddChoiceCtrl(DVD::GetSubtitleFormatLabels(true, true), stream->GetSubtitleFormat(), rect, itemY,
-						choiceIdx, !m_vob->GetDoNotTranscode());
-				rect2.x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+				int top = rect.GetTop();
+				int x = rect.GetRight() + 4;
+				AddChoiceCtrl(DVD::GetSubtitleFormatLabels(true, true), stream->GetSubtitleFormat(), rect.GetRight() + 4,
+						itemY + top, choiceIdx, !m_vob->GetDoNotTranscode());
+				y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
+				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+				// lang code
 				wxString langCode = s_config.GetDefSubtitleLanguage();
 				if (stream->GetSubtitleFormat() != sfNONE) {
 					if (subtitleIdx < m_pgcs->GetSubPictures().size())
@@ -148,7 +167,10 @@ void VobListBox::RefreshInfo() {
 					subtitleIdx++;
 				}
 				int langIdx = DVD::GetAudioLanguageCodes().Index(langCode);
-				AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, rect2, itemY, choiceIdx, true);
+				AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, x, itemY + top, choiceIdx, true);
+				x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+				// button
+				AddButton(x, itemY + top, buttonIdx, true);
 				break;
 			}
 			default:
@@ -178,10 +200,17 @@ void VobListBox::RefreshInfo() {
 				n, dc, x, y);
 		
 		wxRect& rect = m_infoRect[n][m_infoRect[n].size()-1];
+		int top = rect.GetTop();
+		x = rect.GetRight() + 4;
 		int langIdx = DVD::GetAudioLanguageCodes().Index(subtitleIdx < m_pgcs->GetSubPictures().size()
 				? m_pgcs->GetSubPictures()[subtitleIdx]->GetLangCode() : s_config.GetDefSubtitleLanguage());
 		subtitleIdx++;
-		y += AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, rect, itemY, choiceIdx, true);
+		AddChoiceCtrl(DVD::GetAudioLanguageCodes(), langIdx, x, itemY + top, choiceIdx, true);
+		y += UpdateRect(rect, m_choiceList[choiceIdx-1]);
+		x += m_choiceList[choiceIdx-1]->GetSize().GetWidth() + 2;
+		// button
+		AddButton(x, itemY + top, buttonIdx, true);
+		
 		itemY += y > ITEM_HEIGHT ? y + 1 : ITEM_HEIGHT + 1;
 	}
 }
@@ -195,7 +224,8 @@ int VobListBox::AddInfo(const wxString& s, int n, wxDC& dc, int x, int y) {
 	return y + h + 2;
 }
 
-int VobListBox::AddChoiceCtrl(wxArrayString formats, int selection, wxRect& rect, int itemY, int& choiceIdx,
+/** Creates choice control and adds it to the list m_choiceList */
+void VobListBox::AddChoiceCtrl(wxArrayString formats, int selection, int x, int y, int& choiceIdx,
 		bool enabled) {
 	wxChoice* ctrl = NULL;
 	if (choiceIdx >= (int) m_choiceList.size() || m_choiceList[choiceIdx] == NULL) {
@@ -209,7 +239,11 @@ int VobListBox::AddChoiceCtrl(wxArrayString formats, int selection, wxRect& rect
 		ctrl = m_choiceList[choiceIdx];
 	choiceIdx++;
 	ctrl->Enable(enabled);
-	ctrl->SetPosition(wxPoint(rect.GetRight() + 4, itemY + rect.GetTop()));
+	ctrl->SetPosition(wxPoint(x, y));
+}
+
+/** Increase y and height if height of control is greater that actual height */
+int VobListBox::UpdateRect(wxRect& rect, wxControl* ctrl) {
 	if (ctrl->GetSize().GetHeight() > rect.GetHeight()) {
 		int p = (ctrl->GetSize().GetHeight() - rect.GetHeight() + 1)/2;
 		rect.SetY(rect.GetY() + p);
@@ -217,6 +251,24 @@ int VobListBox::AddChoiceCtrl(wxArrayString formats, int selection, wxRect& rect
 		return 2*p;
 	}
 	return 0;
+	
+}
+
+/** Creates button and adds it to the list m_buttonList */
+void VobListBox::AddButton(int x, int y, int& buttonIdx, bool enabled) {
+	wxButton* ctrl = NULL;
+	if (buttonIdx >= (int) m_buttonList.size() || m_buttonList[buttonIdx] == NULL) {
+		int h = (*m_choiceList.begin())->GetSize().GetHeight();
+		ctrl = new wxButton(this, wxID_ANY, wxT("..."), wxDefaultPosition, wxSize(h, h));
+		if (buttonIdx >= (int) m_buttonList.size())
+			m_buttonList.push_back(ctrl);
+		else
+			m_buttonList[buttonIdx] = ctrl;
+	} else
+		ctrl = m_buttonList[buttonIdx];
+	buttonIdx++;
+	ctrl->Enable(enabled);
+	ctrl->SetPosition(wxPoint(x, y));
 }
 
 wxImage VobListBox::LoadFrame(const wxString& filename) const {
@@ -225,16 +277,18 @@ wxImage VobListBox::LoadFrame(const wxString& filename) const {
 	if (decoder.Load(filename)) {
 		double duration = decoder.GetDuration();
 		if (duration > 0) {
-			double dpos = duration * 0.05;
-			decoder.SetPosition(dpos);
-			for (int i = 0; i < 100; i++) {
+			image = decoder.GetNextFrame();
+			double dpos = duration < 6000 ? duration * 0.05 : 300;
+			decoder.SetPosition(dpos - 1.0);
+			for (int i = 0; i < 60; i++) {
 				image = decoder.GetNextFrame();
-				double pos = decoder.GetPosition();
-				if (pos >= dpos || pos < 0)
+				if (decoder.GetPosition() >= dpos)
 					break;
 			}
-		} else
-			image = decoder.GetNextFrame();
+		} else {
+			for (int i = 0; i < 30; i++)
+				image = decoder.GetNextFrame();
+		}
 		decoder.Close();
 	}
 	return image.Ok() ? image : wxBITMAP_FROM_MEMORY(video).ConvertToImage();
@@ -255,6 +309,9 @@ void VobListBox::RemoveItem(int index) {
 		int choiceIdx = m_subtitleChoiceIdx + subtitleIdx;
 		m_choiceList[choiceIdx]->Hide();
 		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
+		int buttonIdx = m_vob->GetStreams().size() + subtitleIdx;
+		m_buttonList[buttonIdx]->Hide();
+		m_buttonList.erase(m_buttonList.begin() + buttonIdx);
 	} else {
 		// removed audio file
 		int audioIdx = index - 1;
@@ -265,6 +322,9 @@ void VobListBox::RemoveItem(int index) {
 		m_choiceList[choiceIdx + 1]->Hide();
 		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
 		m_choiceList.erase(m_choiceList.begin() + choiceIdx);
+		int buttonIdx = m_vob->GetStreams().size() - m_vob->GetAudioFilenames().size() + audioIdx;
+		m_buttonList[buttonIdx]->Hide();
+		m_buttonList.erase(m_buttonList.begin() + buttonIdx);
 	}
 	RefreshInfo();
 	RefreshAll();
@@ -330,7 +390,7 @@ bool VobListBox::HasAudioFiles() {
 	return m_vob->GetAudioFilenames().size() > 0;
 }
 
-Vob* VobListBox::SetValues() {
+void VobListBox::SetValues() {
 	unsigned int choiceIdx = 0;
 	unsigned int audioIdx = 0;
 	unsigned int subtitleIdx = 0;
@@ -372,7 +432,56 @@ Vob* VobListBox::SetValues() {
 			m_pgcs->GetSubPictures()[subtitleIdx]->SetLangCode(m_choiceList[choiceIdx++]->GetStringSelection());
 		subtitleIdx++;
 	}
-	return m_vob;
+}
+
+/** Returns index of choice control for given stream */
+int VobListBox::GetChoiceIdx(unsigned int streamIdx) {
+	int choiceIdx = 0;
+	for (unsigned int stIdx = 0; stIdx < streamIdx; stIdx++) {
+		Stream* stream = m_vob->GetStreams()[stIdx];
+		switch (stream->GetType()) {
+		case stVIDEO:
+			choiceIdx++;
+			break;
+		case stAUDIO:
+		case stSUBTITLE:
+			choiceIdx += 2;
+			break;
+		default:
+			break;
+		}
+	}
+	return choiceIdx;
+}
+
+/** Get video format */
+int VobListBox::GetVideoFormat() {
+	return m_choiceList[m_videoChoiceIdx]->GetSelection();
+}
+
+/** Sets audio format */
+void VobListBox::SetVideoFormat(int videoFormat) {
+	m_choiceList[m_videoChoiceIdx]->SetSelection(videoFormat);
+}
+
+/** Get audio format */
+int VobListBox::GetAudioFormat(unsigned int streamIdx) {
+	return m_choiceList[GetChoiceIdx(streamIdx)]->GetSelection();
+}
+
+/** Sets audio format */
+void VobListBox::SetAudioFormat(unsigned int streamIdx, int audioFormat) {
+	m_choiceList[GetChoiceIdx(streamIdx)]->SetSelection(audioFormat);
+}
+
+/** Get audio language code */
+wxString VobListBox::GetAudioLangCode(unsigned int streamIdx) {
+	return m_choiceList[GetChoiceIdx(streamIdx) + 1]->GetStringSelection();
+}
+
+/** Sets audio language code */
+void VobListBox::SetAudioLangCode(unsigned int streamIdx, wxString langCode) {
+	m_choiceList[GetChoiceIdx(streamIdx) + 1]->SetSelection(DVD::GetAudioLanguageCodes().Index(langCode));
 }
 
 /**
@@ -389,41 +498,73 @@ void VobListBox::SetSubtitleLangCode(int subtitleIndex, wxString langCode) {
 	m_choiceList[m_subtitleChoiceIdx + subtitleIndex]->SetSelection(DVD::GetAudioLanguageCodes().Index(langCode));
 }
 
-/**
- * Returns true if selected item has properties dialog
- */
-bool VobListBox::HasPropDialog() {
-	return GetSelection() == 0 || GetSelection() >= 1 + (int) m_vob->GetAudioFilenames().GetCount();
-}
-
 /**
  * Shows subtitle properties dialog  
  */
 void VobListBox::ShowPropDialog() {
 	if (GetSelection() == 0) {
-		// video
-		SetValues(); // update destination format
-		VideoPropDlg dialog(this, m_vob, m_aspectRatio);
-		if (dialog.ShowModal() == wxID_OK) {
-			if (m_vob->GetStreams()[0]->GetDestinationFormat() > vfCOPY)
-				SetDoNotTranscode(false);
-			m_choiceList[0]->SetSelection(m_vob->GetStreams()[0]->GetDestinationFormat() - 1);
-			m_aspectRatio = dialog.GetSelectedAspectRatio();
-		}
-	} else if (GetSelection() >= 1 + (int) m_vob->GetAudioFilenames().GetCount()) {
+		for (unsigned int i = 0; i < m_vob->GetStreams().GetCount(); i++)
+			if (m_vob->GetStreams()[i]->GetType() == stVIDEO)
+				ShowPropDialog(i);
+	} else if (GetSelection() < 1 + (int) m_vob->GetAudioFilenames().GetCount()) {
+		// audio
+		int audioIdx = GetSelection() - 1;
+		int streamIdx = m_vob->GetStreams().size() - m_vob->GetAudioFilenames().GetCount() + audioIdx;
+		ShowPropDialog(streamIdx);
+	} else {
 		// subtitle
 		int idx = GetSelection() - 1 - m_vob->GetAudioFilenames().GetCount();
+		ShowPropDialog(m_vob->GetStreams().size() + idx);
+	}
+}
+
+/** Shows subtitle properties dialog */
+void VobListBox::ShowPropDialog(unsigned int streamIdx) {
+	if (streamIdx >= m_vob->GetStreams().size()) {
+		// subtitle
+		unsigned int idx = streamIdx - m_vob->GetStreams().size();
+		if (idx >= m_vob->GetSubtitles().size())
+			return;
 		TextSub* textsub = m_vob->GetSubtitles()[idx];
 		SubtitlePropDlg dialog(this, textsub, GetSubtitleLangCode(idx));
 		if (dialog.ShowModal() == wxID_OK) {
 			SetSubtitleLangCode(idx, dialog.GetLangCode());
 		}
+	} else if (m_vob->GetStreams()[streamIdx]->GetType() == stVIDEO) {
+		// video
+		SetValues(); // update destination format
+		VideoPropDlg dialog(this, m_vob, m_aspectRatio);
+		if (dialog.ShowModal() == wxID_OK) {
+			if (dialog.GetVideoFormat() > vfCOPY)
+				SetDoNotTranscode(false);
+			SetVideoFormat(dialog.GetVideoFormat() - 1);
+			m_aspectRatio = dialog.GetAspectRatio();
+		}
+	} else if (m_vob->GetStreams()[streamIdx]->GetType() == stAUDIO) {
+		// audio
+		SetValues(); // update destination format
+		int audioIdx = streamIdx - (m_vob->GetStreams().size() - m_vob->GetAudioFilenames().GetCount());
+		wxString audioFile = audioIdx >= 0 ? m_vob->GetAudioFilenames()[audioIdx] : wxT("");
+		AudioPropDlg dialog(this, m_vob, audioFile, GetAudioLangCode(streamIdx), streamIdx);
+		if (dialog.ShowModal() == wxID_OK) {
+			SetAudioFormat(streamIdx, dialog.GetAudioFormat());
+			SetAudioLangCode(streamIdx, dialog.GetLangCode());
+		}
 	}
 }
 
-/**
- * Processes a double click event
- */
+/** Processes a double click event */
 void VobListBox::OnDoubleClick(wxMouseEvent& evt) {
 	ShowPropDialog();
 }
+
+/** Processes a button event */
+void VobListBox::OnButton(wxCommandEvent& event) {
+	int streamIdx = 0;
+	for (vector<wxButton*>::iterator btIt = m_buttonList.begin(); btIt < m_buttonList.end(); btIt++) {
+		if (*btIt == event.GetEventObject())
+			break;
+		streamIdx++;
+	}
+	ShowPropDialog(streamIdx);
+}
