@@ -3,7 +3,7 @@
 // Purpose:     Media Encoder Interface
 // Author:      Alex Thuering
 // Created:     2.05.2010
-// RCS-ID:      $Id: mediaenc.cpp,v 1.1 2010/05/03 06:04:03 ntalex Exp $
+// RCS-ID:      $Id: mediaenc.cpp,v 1.3 2011/03/17 20:33:14 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -36,6 +36,11 @@ wxSize GetFrameSize(VideoFormat format) {
 	return wxSize(0, 0);
 }
 
+/** Returns frame aspect ratio */
+double GetFrameAspectRatio(VideoFormat format, AspectRatio aspect) {
+	return aspect == ar4_3 ? 4.0/3 : 16.0/9;
+}
+
 /** Returns true if video format is NTSC */
 bool isNTSC(VideoFormat format) {
 	return format == vfNTSC || format == vfNTSC_CROPPED || format == vfNTSC_HALF_D1 || format == vfNTSC_VCD; 
