@@ -3,7 +3,7 @@
 // Purpose:     FFMPEG Media Transcoder
 // Author:      Alex Thuering
 // Created:     26.04.2008
-// RCS-ID:      $Id: mediatrc_ffmpeg.h,v 1.17 2011/02/13 18:49:58 ntalex Exp $
+// RCS-ID:      $Id: mediatrc_ffmpeg.h,v 1.22 2011/11/27 22:52:56 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -14,46 +14,51 @@
 #include "mediatrc.h"
 #include <stdint.h>
 #include <math.h>
+#include <map>
 
 #define INT64_C(val) val##LL
 #define UINT64_C(val) val##ULL
 extern "C" {
 #define __STDC_CONSTANT_MACROS
 #define __STDC_LIMIT_MACROS
-#ifdef HAVE_FFMPEG_AVUTIL_H
-#include <avformat.h>
-#include <swscale.h>
-#else
 #include <libavformat/avformat.h>
 #include <libswscale/swscale.h>
-#endif
 }
 
 // select an input stream for an output stream
-typedef struct AVStreamMap {
+typedef struct StreamMap {
     int file_index;
     int stream_index;
     int sync_file_index;
     int sync_stream_index;
-} AVStreamMap;
+} StreamMap;
 
 // select an input file for an output file
-typedef struct AVMetaDataMap {
-    int out_file;
+typedef struct MetadataMap {
+	int  file;      // file index  
+	char type;      // type of metadata to copy -- (g)lobal, (s)tream, (c)hapter or (p)rogram
+	int  index;     // stream/chapter/program number 
+} MetadataMap;
+
+typedef struct ChapterMap {
     int in_file;
-} AVMetaDataMap;
+    int out_file;
+} ChapterMap;
 
-struct AVInputStream;
-struct AVOutputStream;
+struct InputStream;
+struct OutputStream;
+struct AVDictionary;
+struct InputFile;
+struct OutputFile;
 
-#define MAX_FILES 20
-#define MAX_STREAMS_ 20
+#define MAX_FILES 50
 #define CODEC_TYPE_COUNT (int) CODEC_TYPE_NB
 
 class wxFfmpegMediaTranscoder : public wxMediaTranscoder {
 public:
 	wxFfmpegMediaTranscoder(int threadCount = 1);
 	~wxFfmpegMediaTranscoder();
+	static void Init();
 	
 	/** Adds input file. */
 	bool AddInputFile(const wxString& fileName, bool disableVideo = false, bool disableAudio = false,
@@ -65,10 +70,6 @@ public:
 	/** Sets output file and video/audio/subtitle formats. */
 	bool SetOutputFile(const wxString& fileName, VideoFormat videoFormat, wxArrayInt audioFormats,
 			wxArrayInt subtitleFormats, int videoBitrate = 6000, bool vbr = false, int audioBitrate = 224);
-	/** Sets pad values */
-	void SetPad(const vector<int>& pad);
-	/** Sets crop values */
-	void SetCrop(const vector<int>& crop);
 	/** Sets interlaced encoding flag */
 	void SetInterlaced(bool value);
 	/** Sets first field flag (Auto, TFF, BFF) */
@@ -77,6 +78,10 @@ public:
 	void SetStartTime(double startTime);
 	/** Sets recording time */
 	void SetRecordingTime(double recordingTime);
+	/** Sets video filters */
+	void SetVideoFilters(const wxString& videoFilters) { m_videoFilters = videoFilters; }
+	/** Sets audio volume (256=normal) */
+	void SetAudioVolume(int streamIndex, int audioVolume) { m_audioVolume[streamIndex] = audioVolume; }
 	
 	/** Starts transcoding. */
 	bool Run(bool& canceled);
@@ -84,53 +89,52 @@ public:
 	void End();
 	
 private:
+	wxString m_videoFilters;
 	void PrintError(const wxString& fileName, int err);
+	bool AddInputStreams(AVFormatContext *ic, bool disableVideo, bool disableAudio, bool disableSubtitle);
+	OutputStream* NewOutputStream(AVFormatContext *oc, AVMediaType type, int format);
 	bool AddVideoStream(AVFormatContext *oc, VideoFormat videoFormat, int bitrate, bool vbr);
 	bool AddAudioStream(AVFormatContext *oc, AudioFormat audioFormat, int bitrate);
 	bool AddSubtitleStream(AVFormatContext *oc, SubtitleFormat subtitleFormat);
-	bool OutputPacket(AVInputStream *ist, int ist_index, AVOutputStream **ost_table,
-			int nb_ostreams, const AVPacket *pkt, int bit_buffer_size, uint8_t* bit_buffer);
-	double get_sync_ipts(const AVOutputStream *ost);
+	bool OutputPacket(InputStream *ist, int ist_index, OutputStream *ost_table, int nb_ostreams, const AVPacket *pkt);
+	double get_sync_ipts(const OutputStream *ost);
 	bool WriteFrame(AVFormatContext *s, AVPacket *pkt, AVCodecContext *avctx,
 			AVBitStreamFilterContext *bsfc);
-	bool DoAudioOut(AVFormatContext *s, AVOutputStream *ost, AVInputStream *ist, unsigned char *buf, int size);
-	void pre_process_video_frame(AVInputStream *ist, AVPicture *picture, void **bufp);
-	bool DoVideoOut(AVFormatContext *s, AVOutputStream *ost, AVInputStream *ist,
+	bool DoAudioOut(AVFormatContext *s, OutputStream *ost, InputStream *ist, unsigned char *buf, int size);
+	void pre_process_video_frame(InputStream *ist, AVPicture *picture, void **bufp);
+	bool DoVideoOut(AVFormatContext *s, OutputStream *ost, InputStream *ist,
 			AVFrame *in_picture, int *frame_size, int bit_buffer_size, uint8_t* bit_buffer);
-	bool DoSubtitleOut(AVFormatContext *s, AVOutputStream *ost, AVInputStream *ist,
+	bool DoSubtitleOut(AVFormatContext *s, OutputStream *ost, InputStream *ist,
 			AVSubtitle *sub, int64_t pts);
-	void PrintReport(AVFormatContext **output_files, AVOutputStream **ost_table,
-			int nb_ostreams, int is_last_report);
-    AVFormatContext *input_files[MAX_FILES];
-    int64_t input_files_ts_offset[MAX_FILES];
-	double input_files_ts_scale[MAX_FILES][MAX_STREAMS];
-    int nb_input_files;
+	void PrintReport(OutputFile *output_files, OutputStream *ost_table, int nb_ostreams, int is_last_report);
+	int CopyChapters(int infile, int outfile);
+	AVDictionary *ts_scale;
+	int  nb_ts_scale;
 
-    AVFormatContext *output_files[MAX_FILES];
-	AVCodec *output_codecs[MAX_FILES*MAX_STREAMS];
-    int nb_output_files;
-	int nb_ocodecs;
-
-    AVStreamMap stream_maps[MAX_FILES*MAX_STREAMS];
+    StreamMap* stream_maps;
 	int nb_stream_maps;
 
-    AVMetaDataMap meta_data_maps[MAX_FILES];
+	MetadataMap (*meta_data_maps)[2];
     int nb_meta_data_maps;
+    int metadata_global_autocopy;
+    int metadata_streams_autocopy;
+    int metadata_chapters_autocopy;
+
+    ChapterMap* chapter_maps;
+    int nb_chapter_maps;
+    
+    int chapters_input_file;
+    
+    /* indexed by output file stream index */
+    int *streamid_map;
+    unsigned int nb_streamid_map;
 
     int frame_width;
     int frame_height;
     float frame_aspect_ratio;
     PixelFormat frame_pix_fmt;
+    int frame_bits_per_raw_sample;
     SampleFormat audio_sample_fmt;
-	int frame_padtop;
-    int frame_padbottom;
-    int frame_padleft;
-    int frame_padright;
-    int padcolor[3]; /* default to black */
-    int frame_topBand;
-    int frame_bottomBand;
-    int frame_leftBand;
-    int frame_rightBand;
     int max_frames[4];
     AVRational frame_rate;
     float video_qscale;
@@ -139,7 +143,7 @@ private:
     const char *video_rc_override_string;
     unsigned int video_codec_tag;
     char *video_language; 
-    int same_quality;
+    int same_quant;
     int do_interlace;
     int do_deinterlace;
     int top_field_first;
@@ -151,24 +155,21 @@ private:
 
     int intra_only;
     int audio_sample_rate;
-    int64_t channel_layout;
     float audio_qscale;
     int audio_channels;
     unsigned int audio_codec_tag;
     char *audio_language;
-
+    
     char *subtitle_language;
     unsigned int subtitle_codec_tag;
-
+    
     float mux_preload;
     float mux_max_delay;
 
     int64_t recording_time;
     int64_t start_time;
-    int64_t recording_timestamp;
     int64_t input_ts_offset;
-    int metadata_count;
-    AVMetadataTag *metadata;
+    AVDictionary *metadata;
     int do_psnr;
     int do_pass;
     const char *pass_logfilename_prefix;
@@ -176,8 +177,8 @@ private:
     int audio_sync_method;
     float audio_drift_threshold;
     int copy_ts;
-    int opt_shortest; //
-    int video_global_header;
+    int copy_tb;
+    int opt_shortest;
     char *vstats_filename;
     FILE *vstats_file;
     int opt_programid;
@@ -185,10 +186,7 @@ private:
 
     int rate_emu;
 
-    int  video_channel;
-    char *video_standard;
-
-    int audio_volume;
+    map<int, int> m_audioVolume; // stream index -> audio volume
 
     int exit_on_error;
     int verbose;
@@ -205,9 +203,6 @@ private:
 
     int pgmyuv_compatibility_hack;
     float dts_delta_threshold;
-
-    unsigned int sws_flags;
-
     int64_t timer_start;
     
     uint8_t *audio_buf;
@@ -215,11 +210,25 @@ private:
     unsigned int allocated_audio_out_size, allocated_audio_buf_size;
 
     short *samples;
+    int bit_buffer_size;
+    uint8_t *bit_buffer;
 
     AVBitStreamFilterContext *video_bitstream_filters;
     AVBitStreamFilterContext *audio_bitstream_filters;
     AVBitStreamFilterContext *subtitle_bitstream_filters;
-    AVBitStreamFilterContext *bitstream_filters[MAX_FILES][MAX_STREAMS_];
+    
+    InputStream *input_streams;
+    int nb_input_streams;
+    InputFile *input_files;
+    int nb_input_files;
+    
+    OutputStream *output_streams;
+	int nb_output_streams;
+	OutputFile *output_files;
+    int nb_output_files;
+    
+    AVDictionary *format_opts;
+    AVDictionary *codec_opts;
 };
 
 #endif // WX_FFMPEG_MEDIA_TRANSCODER_H
