@@ -3,7 +3,7 @@
 // Purpose:     Properties dialog (base class for all properties dialogs)
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.cpp,v 1.25 2011/02/13 17:56:31 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.cpp,v 1.29 2011/07/24 16:35:22 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -127,10 +127,13 @@ enum PropertyType {
   propFONT
 };
 
+wxColourData wxPropDlg::m_colourData;
+
 wxPropDlg::wxPropDlg(wxWindow* parent, wxString title): wxDialog(parent, -1, title, wxDefaultPosition,
 		wxDefaultSize, wxDEFAULT_DIALOG_STYLE| wxRESIZE_BORDER) {
 	m_currGroupId = 0;
 	m_updateIndex = -1;
+	SetIcon(((wxTopLevelWindow*)wxGetTopLevelParent(parent))->GetIcon());
 }
 
 void wxPropDlg::Create(bool resetButton, bool dontShowCheckbox)
@@ -223,7 +226,7 @@ void wxPropDlg::AddText(wxSizer* sizer, wxString text, int flag) {
 }
 
 wxSizer* wxPropDlg::AddTextProp(wxSizer* sizer, const wxString& label, const wxString& value,
-		bool readonly, int width, bool addSpacer) {
+		bool readonly, int width, bool addSpacer, long style) {
   if (!sizer) { // only update value
 	wxTextCtrl* ctrl = (wxTextCtrl*) m_controls[m_updateIndex];
 	ctrl->SetValue(value);
@@ -236,7 +239,7 @@ wxSizer* wxPropDlg::AddTextProp(wxSizer* sizer, const wxString& label, const wxS
 	sizer->Add(new wxStaticText(propWindow, -1, label), 0, wxALIGN_CENTER_VERTICAL);
   // add text control
   wxSize size = width > 0 ? wxSize(width, -1) : wxDefaultSize;
-  wxTextCtrl* ctrl = new wxTextCtrl(propWindow, -1, value, wxDefaultPosition, size);
+  wxTextCtrl* ctrl = new wxTextCtrl(propWindow, -1, value, wxDefaultPosition, size, style);
   ctrl->SetEditable(!readonly);
   wxBoxSizer* pane = NULL;
   if (width > 0) {
@@ -273,7 +276,8 @@ void wxPropDlg::AddStaticTextProp(wxSizer* sizer, const wxString& label, const w
 void wxPropDlg::AddStaticLine(wxSizer* sizer, wxString caption) {
 	wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
 	wxStaticText* staticText = new wxStaticText(this, -1, caption);
-	staticText->SetFont(wxFont(12, wxDEFAULT, wxNORMAL, wxBOLD, 0, _T("")));
+	int fontSize = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT).GetPointSize() + 2;
+	staticText->SetFont(wxFont(fontSize, wxDEFAULT, wxNORMAL, wxBOLD, 0, _T("")));
 	sizer2->Add(staticText, 0, wxRIGHT | wxTOP | wxBOTTOM, 5);
 	sizer2->Add(new wxStaticLine(this, -1), 1, wxALIGN_CENTER_VERTICAL, 5);
 	sizer->Add(sizer2, 0, wxEXPAND, 0);
@@ -326,7 +330,7 @@ void wxPropDlg::AddCheckProp(wxSizer* sizer, const wxString& label, bool value,
   ctrl->SetValue(value);
   ctrl->Enable(!readonly);
   ctrl->SetName(wxString::Format(_T("%d"), m_controls.GetCount()));
-  sizer->Add(ctrl, 1, wxEXPAND, 0);
+  sizer->Add(ctrl, 0, wxEXPAND, 0);
   m_types.Add(propCHECK);
   m_controls.Add(ctrl);
   m_groupIds.Add(m_currGroupId);
@@ -690,16 +694,16 @@ void wxPropDlg::AddColourProp(wxSizer* sizer, const wxString& label,
   sizer->Add(sizer2);
 }
 
-void wxPropDlg::OnSelectColour(wxCommandEvent& event)
-{
-  long index = -1;
-  ((wxButton*)event.GetEventObject())->GetName().ToLong(&index);
-  ColourPanel* panel = ((ColourPanel*)m_controls[index]);
-  wxColourData data;
-  data.SetColour(panel->GetColour());
-  wxColourDialog dialog(propWindow, &data);
-  if (dialog.ShowModal() == wxID_OK)
-	panel->SetColour(dialog.GetColourData().GetColour());
+void wxPropDlg::OnSelectColour(wxCommandEvent& event) {
+	long index = -1;
+	((wxButton*) event.GetEventObject())->GetName().ToLong(&index);
+	ColourPanel* panel = ((ColourPanel*) m_controls[index]);
+	m_colourData.SetColour(panel->GetColour());
+	wxColourDialog dialog(propWindow, &m_colourData);
+	if (dialog.ShowModal() == wxID_OK) {
+		panel->SetColour(dialog.GetColourData().GetColour());
+		m_colourData = dialog.GetColourData();
+	}
 }
 
 void wxPropDlg::AddFileProp(wxSizer* sizer, const wxString& label,
@@ -790,55 +794,59 @@ void wxPropDlg::OnSelectDir(wxCommandEvent& event)
 
 ////////////////////////////// Group /////////////////////////////////////////
 
-wxSizer* wxPropDlg::BeginGroup(wxSizer* sizer,
-  wxString title, wxString checkTitle, bool value, bool readonly, bool vertical, int proportion)
-{
-  if (!sizer) // only update value
-  {
-	if (checkTitle.length())
-	  AddCheckProp(NULL, wxEmptyString, value, readonly, GROUP_CHECK_ID);
-	return NULL;
-  }
-  m_currGroupId++;
-  wxStaticBox* sb = new wxStaticBox(propWindow, -1, title);
-  wxStaticBoxSizer* sbsizer = new wxStaticBoxSizer(sb, vertical ? wxVERTICAL : wxHORIZONTAL);
-  sizer->Add(sbsizer, proportion, wxEXPAND);
-  if (checkTitle.length())
-  {
-    wxFlexGridSizer* groupSizer = new wxFlexGridSizer(2, 2, 8, 4);
-    groupSizer->AddGrowableCol(1);
-    sbsizer->Add(groupSizer, 0, wxEXPAND|wxALL, 4);
-    AddCheckProp(groupSizer, wxEmptyString, value, readonly, GROUP_CHECK_ID);
-    m_currGroupChecker = m_controls[m_controls.GetCount()-1];
-	AddText(groupSizer, checkTitle);
-	AddText(groupSizer, wxEmptyString);
-    return groupSizer;
-  }
-  m_currGroupChecker = NULL;
-  return sbsizer;
+wxSizer* wxPropDlg::BeginGroup(wxSizer* sizer, wxString title, wxString checkTitle, bool value, bool readonly,
+		bool vertical, int proportion) {
+	if (!sizer) { // only update value
+		if (checkTitle.length())
+			AddCheckProp(NULL, wxEmptyString, value, readonly, GROUP_CHECK_ID);
+		return NULL;
+	}
+	m_currGroupId++;
+	wxStaticBox* sb = new wxStaticBox(propWindow, -1, title);
+	wxStaticBoxSizer* sbsizer = new wxStaticBoxSizer(sb, vertical ? wxVERTICAL : wxHORIZONTAL);
+	sizer->Add(sbsizer, proportion, wxEXPAND);
+	if (checkTitle.length()) {
+		wxFlexGridSizer* groupSizer = new wxFlexGridSizer(2, 2, 8, 4);
+		groupSizer->AddGrowableCol(1);
+		sbsizer->Add(groupSizer, 0, wxEXPAND | wxALL, 4);
+		AddCheckProp(groupSizer, wxEmptyString, value, readonly, GROUP_CHECK_ID);
+		m_currGroupChecker = m_controls[m_controls.GetCount() - 1];
+		AddText(groupSizer, checkTitle);
+		AddText(groupSizer, wxEmptyString);
+		return groupSizer;
+	}
+	m_currGroupChecker = NULL;
+	return sbsizer;
 }
 
-void wxPropDlg::EndGroup()
-{
-  if (m_currGroupChecker)
-  {
-	wxCommandEvent evt;
-	evt.SetEventObject((wxObject*)m_currGroupChecker);
-	OnGroupCheck(evt);
-  }
-  m_currGroupChecker = NULL;
-  m_currGroupId++;
+void wxPropDlg::BeginCheckGroup(wxSizer* sizer, wxString label, bool value, bool readonly) {
+	if (!sizer) { // only update value
+		AddCheckProp(NULL, label, value, readonly, GROUP_CHECK_ID);
+		return;
+	}
+	m_currGroupId++;
+	AddCheckProp(sizer, label, value, readonly, GROUP_CHECK_ID);
+	m_currGroupChecker = m_controls[m_controls.GetCount() - 1];
 }
 
-void wxPropDlg::OnGroupCheck(wxCommandEvent& event)
-{
-  long index = -1;
-  wxCheckBox* ctrl = (wxCheckBox*)event.GetEventObject();
-  ctrl->GetName().ToLong(&index);
-  int groupId = m_groupIds[index];
-  for (int i = 0; i<(int)m_groupIds.GetCount(); i++)
-    if (i != index && m_groupIds[i] == groupId && m_types[i] != propFONT)
-	  ((wxWindow*) m_controls[i])->Enable(ctrl->GetValue());
+void wxPropDlg::EndGroup() {
+	if (m_currGroupChecker) {
+		wxCommandEvent evt;
+		evt.SetEventObject((wxObject*) m_currGroupChecker);
+		OnGroupCheck(evt);
+	}
+	m_currGroupChecker = NULL;
+	m_currGroupId++;
+}
+
+void wxPropDlg::OnGroupCheck(wxCommandEvent& event) {
+	long index = -1;
+	wxCheckBox* ctrl = (wxCheckBox*) event.GetEventObject();
+	ctrl->GetName().ToLong(&index);
+	int groupId = m_groupIds[index];
+	for (int i = 0; i < (int) m_groupIds.GetCount(); i++)
+		if (i != index && m_groupIds[i] == groupId && m_types[i] != propFONT)
+			((wxWindow*) m_controls[i])->Enable(ctrl->GetValue());
 }
 
 wxControl* wxPropDlg::GetControl(int index) {
