@@ -3,7 +3,7 @@
 // Purpose:     Base class for property dialogs
 // Author:      Alex Thuering
 // Created:     18.11.2003
-// RCS-ID:      $Id: PropDlg.h,v 1.22 2011/02/13 17:56:31 ntalex Exp $
+// RCS-ID:      $Id: PropDlg.h,v 1.25 2011/07/24 16:35:22 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -15,6 +15,9 @@
 #include <wx/grid.h>
 #include <wx/dynarray.h>
 #include <wx/filedlg.h>
+#if wxCHECK_VERSION(2,9,0)
+#include <wx/fontdata.h>
+#endif
 
 WX_DECLARE_OBJARRAY(wxBitmap, BitmapArray);
 
@@ -32,6 +35,9 @@ public:
      * Destructor
      */
     virtual ~wxPropDlg();
+    
+    /** Returns list of custom colours for colour selection dialog */
+    static wxColourData& GetColourData() { return m_colourData; }
 	
 protected:
 	wxArrayInt m_types;
@@ -40,6 +46,7 @@ protected:
 	int m_updateIndex; /** index of property by update (reset) */
 	int m_currGroupId;
 	void* m_currGroupChecker;
+	static wxColourData m_colourData;
 	
 	/**
 	 * Creates dialog with properties panel and button pane
@@ -74,7 +81,7 @@ protected:
 	void AddText(wxSizer* sizer, wxString text, int flag = wxALIGN_CENTER_VERTICAL);
 
 	wxSizer* AddTextProp(wxSizer* sizer, const wxString& label, const wxString& value,
-	  bool readonly = false, int width = -1, bool addSpacer = true);
+	  bool readonly = false, int width = -1, bool addSpacer = true, long style = 0);
 	void AddStaticTextProp(wxSizer* sizer, const wxString& label, const wxString& value);
 	/**
 	 * Adds a static line with given caption to the window
@@ -138,9 +145,9 @@ protected:
 	void AddDirectoryProp(wxSizer* sizer, const wxString& label,
 	  const wxString& value, wxString caption = _T("..."));
 
-	wxSizer* BeginGroup(wxSizer* sizer, wxString title = wxEmptyString,
-	  wxString checkTitle = wxEmptyString, bool value = false,
-	  bool readonly = false, bool vertical = true, int proportion = 0);
+	wxSizer* BeginGroup(wxSizer* sizer, wxString title = wxEmptyString, wxString checkTitle = wxEmptyString,
+			bool value = false, bool readonly = false, bool vertical = true, int proportion = 0);
+	void BeginCheckGroup(wxSizer* sizer, wxString label, bool value, bool readonly = false);
 	void EndGroup();
 	
 	/**
@@ -181,7 +188,7 @@ protected:
 	virtual void OnCellChange(wxGridEvent& event);
 	virtual void OnRowDelete(wxCommandEvent& event);
 	
-  private:
+private:
     DECLARE_EVENT_TABLE()
 };
 
