@@ -3,7 +3,7 @@
 // Purpose:     wxThumbnails class
 // Author:      Alex Thuering
 // Created:		2.02.2003
-// RCS-ID:      $Id: Thumbnails.cpp,v 1.31 2011/02/23 20:24:10 ntalex Exp $
+// RCS-ID:      $Id: Thumbnails.cpp,v 1.36 2011/11/22 20:28:11 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -128,7 +128,7 @@ wxThumbnails::wxThumbnails(wxWindow* parent, int id) :
 	m_tCaptionBorder = 8;
 	m_tOutline = wxTHUMB_OUTLINE_IMAGE;
 	m_tOutlineColour = *wxBLACK;
-	m_tOutlineSelectedColour = *wxBLACK;
+	m_tOutlineSelectedColour = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);
 	m_tImageSelectedColour = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);
 	m_filter = -1;
 	m_selected = -1;
@@ -200,8 +200,8 @@ void wxThumbnails::ShowDir(wxString dir, int filter) {
 		if (m_filter == -1 || (m_filter & wxTHUMB_FILTER_VIDEOS && IsVideo(fname))
 				|| (m_filter & wxTHUMB_FILTER_AUDIOS && IsAudio(fname))
 				|| (m_filter & wxTHUMB_FILTER_AUDIOS && IsSubtitle(fname))
-				|| (m_filter & wxTHUMB_FILTER_IMAGES &&
-						wxImage::FindHandler(fname.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY))
+				|| (m_filter & wxTHUMB_FILTER_IMAGES
+						&& wxImage::FindHandler(fname.AfterLast('.').Lower(), wxBITMAP_TYPE_ANY))
 				|| (m_filter & wxTHUMB_FILTER_CUSTOM && IsCustomFile(fname)))
 			m_items.Add(new wxThumb(fname, caption));
 		fname = wxFindNextFile();
@@ -220,7 +220,7 @@ void wxThumbnails::SetSelected(int value) {
 }
 
 wxString wxThumbnails::GetAudioVideoExtWildcard() {
-	return wxT("*.asf;*.avi;*.dv;*.flv;*.m2p;*.mkv;*.mov;*.mp4;*.mpeg;*.mpg;*.ogg;*.ogm;*.vob;*.vro;*.webm;*.wmv;");
+	return wxT("*.asf;*.avi;*.avs;*.dv;*.flv;*.m2p;*.m2ts;*.mkv;*.mov;*.mp4;*.mpeg;*.mpg;*.mts;*.ogg;*.ogm;*.vob;*.vro;*.webm;*.wmv;");
 }
 
 wxString wxThumbnails::GetVideoExtWildcard() {
@@ -228,7 +228,7 @@ wxString wxThumbnails::GetVideoExtWildcard() {
 }
 
 wxString wxThumbnails::GetAudioExtWildcard() {
-	return wxT("*.mpa;*.mp2;*.mp3;*.m4a;*.ac3;*.dts;*.pcm;");
+	return wxT("*.mpa;*.mp2;*.mp3;*.m4a;*.ac3;*.dts;*.pcm;*.lpcm;*.wav;");
 }
 
 wxString wxThumbnails::GetSubtitleExtWildcard() {
@@ -441,7 +441,8 @@ void wxThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 	int tHeight = imgHeight + (m_tOutline != wxTHUMB_OUTLINE_NONE ? 4 : 0) + (drawShadow ? 4 : 0) + 1;
 	// image
 	wxBitmap img = GetThumbImage(thumb);
-	if (IsSelected(index) && m_tImageSelectedColour.IsOk()) {
+	if (IsSelected(index) && m_tImageSelectedColour.IsOk() && (m_tOutline == wxTHUMB_OUTLINE_NONE
+			|| (m_tOutline == wxTHUMB_OUTLINE_IMAGE && thumb.IsNoOutline()))) {
 		wxImage imgTmp = img.ConvertToImage();
 		wxAdjustToColour(imgTmp, m_tImageSelectedColour);
 		img = wxBitmap(imgTmp);
@@ -455,7 +456,7 @@ void wxThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 			img.GetWidth(), img.GetHeight());
 	dc.DrawBitmap(img, imgRect.x, imgRect.y, true);
 	// outline
-	if (m_tOutline != wxTHUMB_OUTLINE_NONE && (m_tOutline!=wxTHUMB_OUTLINE_IMAGE || !thumb.IsNoOutline())) {
+	if (m_tOutline != wxTHUMB_OUTLINE_NONE && (m_tOutline != wxTHUMB_OUTLINE_IMAGE || !thumb.IsNoOutline())) {
 		if (m_tOutline == wxTHUMB_OUTLINE_FULL || m_tOutline == wxTHUMB_OUTLINE_RECT) {
 			imgRect.x = x;
 			imgRect.y = y;
@@ -465,9 +466,10 @@ void wxThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 				imgRect.height = tHeight - (drawShadow ? 4 : 0);
 		}
 		dc.SetBrush(wxBrush(*wxBLACK, wxTRANSPARENT));
-		dc.SetPen(*wxLIGHT_GREY_PEN);
+		bool selOutline = IsSelected(index) && m_tOutlineSelectedColour.IsOk();
+		dc.SetPen(selOutline ? wxPen(m_tOutlineSelectedColour, 0, wxSOLID) : *wxLIGHT_GREY_PEN);
 		dc.DrawRectangle(imgRect.x - 1, imgRect.y - 1, imgRect.width + 2, imgRect.height + 2);
-		dc.SetPen(wxPen(IsSelected(index) ? m_tOutlineSelectedColour : m_tOutlineColour, 0, wxSOLID));
+		dc.SetPen(wxPen(selOutline ? m_tOutlineSelectedColour : m_tOutlineColour, 0, wxSOLID));
 		dc.DrawRectangle(imgRect.x - 2, imgRect.y - 2, imgRect.width + 4, imgRect.height + 4);
 		
 		if (drawShadow) {
@@ -524,10 +526,14 @@ void wxThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index) {
 }
 
 void wxThumbnails::OnPaint(wxPaintEvent& WXUNUSED(event)) {
-	wxRect paintRect = GetPaintRect();
-
 	wxPaintDC dc(this);
 	PrepareDC(dc);
+	Paint(dc);
+}
+
+void wxThumbnails::Paint(wxPaintDC& dc) {
+	wxRect paintRect = GetPaintRect();
+	
 	// items
 	int row = -1;
 	for (int i = 0; i < (int) m_items.GetCount(); i++) {
